@@ -36,13 +36,16 @@
 #include <sys/time.h>
 #include <regex.h>
 #else
-#include "../../visual/gnu_regex_dist/regex.h"
+#include "../../visual/gnu_regex/regex.h"
 #endif
 
 #include <libwzd-auth/wzd_auth.h>
 
 #include <libwzd-core/wzd_backend.h>
+#include <libwzd-core/wzd_group.h>
+#include <libwzd-core/wzd_log.h>
 #include <libwzd-core/wzd_misc.h>
+#include <libwzd-core/wzd_user.h>
 #include <libwzd-core/wzd_debug.h>
 
 #include "libplaintext_main.h"
@@ -77,42 +80,30 @@ static unsigned int find_directive(const char *name)
 /* IP allowing */
 static int __user_ip_add(wzd_user_t * user, const char *newip)
 {
-  int i;
-
   /* of course this should never happen :) */
   if (user == NULL || newip==NULL) return -1;
 
   if (strlen(newip) < 1) return -1;
   if (strlen(newip) >= MAX_IP_LENGTH) return -1; /* upper limit for an hostname */
 
-  /* tail insertion, be aware that order is important */
-  for (i=0; i<HARD_IP_PER_USER; i++) {
-    if (user->ip_allowed[i][0] == '\0') {
-      strncpy(user->ip_allowed[i],newip,MAX_IP_LENGTH-1);
-      return 0;
-    }
-  }
-  return 1; /* full */
+  if (ip_add_check(&user->ip_list, newip, 1 /* is_allowed */))
+    return 1;
+
+  return 0;
 }
 
 static int __group_ip_add(wzd_group_t * group, const char *newip)
 {
-  int i;
-
   /* of course this should never happen :) */
   if (group == NULL || newip==NULL) return -1;
 
   if (strlen(newip) < 1) return -1;
   if (strlen(newip) >= MAX_IP_LENGTH) return -1; /* upper limit for an hostname */
 
-  /* tail insertion, be aware that order is important */
-  for (i=0; i<HARD_IP_PER_GROUP; i++) {
-    if (group->ip_allowed[i][0] == '\0') {
-      strncpy(group->ip_allowed[i],newip,MAX_IP_LENGTH-1);
-      return 0;
-    }
-  }
-  return 1; /* full */
+  if (ip_add_check(&group->ip_list, newip, 1 /* is_allowed */))
+    return 1;
+
+  return 0;
 }
 
 int write_single_user(FILE * file, const wzd_user_t * user)
@@ -120,6 +111,7 @@ int write_single_user(FILE * file, const wzd_user_t * user)
   unsigned int j;
   wzd_group_t * loop_group;
   char buffer[4096], errbuf[1024];
+  struct wzd_ip_list_t * current_ip;
 
   fprintf(file,"name=%s\n",user->username);
   fprintf(file,"pass=%s\n",user->userpass);
@@ -128,7 +120,7 @@ int write_single_user(FILE * file, const wzd_user_t * user)
   /* write ALL groups */
 
   if (user->group_num>0) {
-    loop_group = plaintext_get_group_from_gid(user->groups[0]);
+    loop_group = group_get_by_id(user->groups[0]);
     if (!loop_group) {
       /* FIXME warn user */
       snprintf(errbuf,sizeof(errbuf),"Invalid MAIN group %u for user %s\n",user->groups[0],user->username);
@@ -137,7 +129,7 @@ int write_single_user(FILE * file, const wzd_user_t * user)
       strcpy(buffer,loop_group->groupname);
       for (j=1; j<user->group_num; j++) {
         strcat(buffer,",");
-        loop_group = plaintext_get_group_from_gid(user->groups[j]);
+        loop_group = group_get_by_id(user->groups[j]);
         if (!loop_group) {
           /* FIXME warn user */
           snprintf(errbuf,sizeof(errbuf),"Invalid MAIN group %u for user %s\n",user->groups[j],user->username);
@@ -152,10 +144,8 @@ int write_single_user(FILE * file, const wzd_user_t * user)
   fprintf(file,"rights=0x%lx\n",user->userperms);
   if (strlen(user->tagline)>0)
     fprintf(file,"tagline=%s\n",user->tagline);
-  for (j=0; j<HARD_IP_PER_USER; j++)
-  {
-    if (user->ip_allowed[j][0] != '\0')
-      fprintf(file,"ip_allowed=%s\n",user->ip_allowed[j]);
+  for (current_ip = user->ip_list; current_ip != NULL; current_ip = current_ip->next_ip) {
+    fprintf(file,"ip_allowed=%s\n",current_ip->regexp);
   }
   if (user->max_ul_speed)
     fprintf(file,"max_ul_speed=%u\n",user->max_ul_speed);
@@ -191,7 +181,7 @@ int write_single_user(FILE * file, const wzd_user_t * user)
 
 int write_single_group(FILE * file, const wzd_group_t * group)
 {
-  unsigned int j;
+  struct wzd_ip_list_t * current_ip;
 
   fprintf(file,"privgroup\t%s\n",group->groupname);
   if (group->max_idle_time)
@@ -202,13 +192,12 @@ int write_single_group(FILE * file, const wzd_group_t * group)
     fprintf(file,"max_ul_speed=%u\n",group->max_ul_speed);
   if (group->max_dl_speed)
     fprintf(file,"max_dl_speed=%u\n",group->max_dl_speed);
+  fprintf(file,"rights=0x%lx\n",group->groupperms);
   if (strlen(group->tagline)>0)
     fprintf(file,"tagline=%s\n",group->tagline);
   fprintf(file,"gid=%d\n",group->gid);
-  for (j=0; j<HARD_IP_PER_GROUP; j++)
-  {
-    if (group->ip_allowed[j][0] != '\0')
-      fprintf(file,"ip_allowed=%s\n",group->ip_allowed[j]);
+  for (current_ip = group->ip_list; current_ip != NULL; current_ip = current_ip->next_ip) {
+    fprintf(file,"ip_allowed=%s\n",current_ip->regexp);
   }
   if (strlen(group->defaultpath)>0)
     fprintf(file,"default_home=%s\n",group->defaultpath);
@@ -222,7 +211,7 @@ int write_single_group(FILE * file, const wzd_group_t * group)
 
 int write_user_file(void)
 {
-#ifndef _MSC_VER
+#ifndef WIN32
   sigset_t mask;
 #endif
   char filename[256];
@@ -245,10 +234,12 @@ int write_user_file(void)
     "# section begins by [SECTIONNAME]",
     NULL /* you MUST keep this array NULL-ended ! */
   };
-  ListElmt * elmnt;
   wzd_user_t * loop_user;
   wzd_group_t * loop_group;
+  uid_t * user_list;
+  gid_t * group_list;
 
+#if 0
   /* this loop has no real interest ...
    * it is only used to access each struct once, to provoque SEGFAULT
    * before we start to erase file in case there is a memory corruption.
@@ -262,6 +253,7 @@ int write_user_file(void)
     }
     j = loop_user->username[0];
   }
+#endif
 
   strcpy (filename,USERS_FILE);
   strcpy (filenamenew,USERS_FILE);
@@ -326,9 +318,10 @@ int write_user_file(void)
 
   fprintf(file,"# groups definitions\n");
   fprintf(file,"[GROUPS]\n");
-  for (elmnt=list_head(&group_list); elmnt; elmnt=list_next(elmnt))
+  group_list = group_get_list(1 /* backend id */);
+  for (i=0; group_list[i]!=INVALID_GROUP; i++)
   {
-    if (!(loop_group = list_data(elmnt))) {
+    if (!(loop_group = group_get_by_id(group_list[i]))) {
       ERRLOG("EMPTY NODE IN GROUP LIST !\n");
       continue;
     }
@@ -340,13 +333,15 @@ int write_user_file(void)
       continue;
     }
   }
+  wzd_free(group_list);
 
   fprintf(file,"# users definitions\n");
   fprintf(file,"# users MUST begin by line name=<>\n");
   fprintf(file,"[USERS]\n");
-  for (elmnt=list_head(&user_list); elmnt; elmnt=list_next(elmnt))
+  user_list = user_get_list(1 /* backend id */);
+  for (i=0; user_list[i]!=INVALID_USER; i++)
   {
-    if (!(loop_user = list_data(elmnt))) {
+    if (!(loop_user = user_get_by_id(user_list[i]))) {
       ERRLOG("EMPTY NODE IN USER LIST !\n");
       continue;
     }
@@ -358,11 +353,7 @@ int write_user_file(void)
       continue;
     }
   }
-
-  fprintf(file,"# per hosts rights\n");
-  fprintf(file,"[HOSTS]\n");
-  fprintf(file,"all = *\n");
-  fprintf(file,"\n");
+  wzd_free(user_list);
 
   fclose(file);
 
@@ -386,38 +377,370 @@ int write_user_file(void)
   return 0;
 }
 
-int read_section_hosts(FILE * file_user, char * line)
+/* Read a group
+ * Return a newly allocated group or NULL
+ */
+wzd_group_t * read_single_group(FILE * file, const char *groupname, char * buffer, size_t length)
 {
-  char c;
+  char errbuf[1024];
+  int err;
+  long num;
+  char * ptr;
+  wzd_group_t * group;
 
-#if 0
-fprintf(stderr,"Entering section HOSTS\n");
-#endif
-  while ( (c = getc(file_user)) != (char)EOF ) {
-    if (c=='\n') continue;
-    if (c=='#') { fgets(line+1,MAX_LINE-2,file_user); continue; } /* comment */
-    if (c == '[') { /* another section */
-      ungetc(c,file_user);
-      return 0;
+  group = group_allocate();
+
+  strncpy(group->groupname,groupname,HARD_GROUPNAME_LENGTH-1);
+
+  group->gid = group_find_free_gid(0); /* default value for gid, for compat */
+
+  while ( fgets(buffer,MAX_LINE-1,file) != NULL ) {
+    chop(buffer);
+
+    if (strlen(buffer) == 0) return group;
+
+    err = regexec(&reg_line,buffer,3,regmatch,0);
+    if (err) {
+      snprintf(errbuf,sizeof(errbuf),"Line '%s' does not respect config line format - ignoring\n",buffer);
+      ERRLOG(errbuf);
+      continue;
     }
-    line[0] = c; /* we avoid a stupid ungetc */
-    fgets(line+1,MAX_LINE-2,file_user);
-    while ( line[strlen(line)-1] == '\r' || line[strlen(line)-1] == '\n')
-      line[strlen(line)-1] = '\0'; /* clear trailing \n */
-/*fprintf(stderr,"i read '%s'\n",line);*/
-  }
-  return 0;
+    memcpy(varname,buffer+regmatch[1].rm_so,regmatch[1].rm_eo-regmatch[1].rm_so);
+    varname[regmatch[1].rm_eo-regmatch[1].rm_so]='\0';
+    memcpy(value,buffer+regmatch[2].rm_so,regmatch[2].rm_eo-regmatch[2].rm_so);
+    value[regmatch[2].rm_eo-regmatch[2].rm_so]='\0';
+
+    if (strcmp("gid",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid gid %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->gid = num;
+    }
+    else if (strcasecmp(varname,"max_idle_time")==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_idle_time %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->max_idle_time = num;
+    } /* max_idle_time */
+    else if (strcmp("num_logins",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid num_logins %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->num_logins = (unsigned short)num;
+    } /* else if (strcmp("num_logins",... */
+
+    else if (strcmp("ip_allowed",varname)==0) {
+      err = __group_ip_add(group,value);
+      if (err != 0 ) {
+        snprintf(errbuf,sizeof(errbuf),"ERROR unable to add ip %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+    } /* ip_allowed */
+    else if (strcmp("default_home",varname)==0) {
+      strncpy(group->defaultpath,value,WZD_MAX_PATH);
+    } /* default_home */
+    else if (strcmp("ratio",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid ratio %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->ratio = num;
+    } /* else if (strcmp("ratio",... */
+    else if (strcmp("rights",varname)==0) {
+      num = strtoul(value, &ptr, 0);
+      group->groupperms = num;
+    }
+    else if (strcmp("max_dl_speed",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_dl_speed %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->max_dl_speed = num;
+    } /* max_dl_speed */
+    else if (strcmp("max_ul_speed",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_ul_speed %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      group->max_ul_speed = num;
+    } /* max_ul_speed */
+    else if (strcmp("tagline",varname)==0) {
+      strncpy(group->tagline,value,MAX_TAGLINE_LENGTH);
+    } /* tagline */
+    else {
+      snprintf(errbuf,sizeof(errbuf),"ERROR Variable '%s' is not correct (value %s) - ignoring\n",varname,value);
+      ERRLOG(errbuf);
+    }
+  };
+
+  return group;
+}
+
+/* Read a user
+ * Return a newly allocated user or NULL
+ */
+wzd_user_t * read_single_user(FILE * file, const char *username, char * buffer, size_t length)
+{
+  char errbuf[1024];
+  int err;
+  char * ptr;
+  wzd_user_t * user;
+  long num;
+  unsigned long u_num;
+  u64_t ul_num;
+
+  user = user_allocate();
+
+  strncpy(user->username,username,HARD_USERNAME_LENGTH-1);
+
+  user->uid = user_find_free_uid(0); /* default value for uid, for compat */
+
+  while ( fgets(buffer,MAX_LINE-1,file) != NULL ) {
+    chop(buffer);
+
+    if (strlen(buffer) == 0) return user;
+
+    err = regexec(&reg_line,buffer,3,regmatch,0);
+    if (err) {
+      snprintf(errbuf,sizeof(errbuf),"Line '%s' does not respect config line format - ignoring\n",buffer);
+      ERRLOG(errbuf);
+      continue;
+    }
+    memcpy(varname,buffer+regmatch[1].rm_so,regmatch[1].rm_eo-regmatch[1].rm_so);
+    varname[regmatch[1].rm_eo-regmatch[1].rm_so]='\0';
+    memcpy(value,buffer+regmatch[2].rm_so,regmatch[2].rm_eo-regmatch[2].rm_so);
+    value[regmatch[2].rm_eo-regmatch[2].rm_so]='\0';
+
+    if (strcmp("uid",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid uid %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->uid = num;
+    }
+    else if (strcmp("home",varname)==0) {
+      /* remove trailing / */
+      if (value[strlen(value)-1] == '/' && strcmp(value,"/")!=0)
+        value[strlen(value)-1] = '\0';
+      DIRNORM(value,strlen(value),0);
+      strncpy(user->rootpath,value,WZD_MAX_PATH-1);
+    }
+    else if (strcmp("pass",varname)==0) {
+      strncpy(user->userpass,value,MAX_PASS_LENGTH-1);
+    }
+    else if (strcmp("flags",varname)==0) {
+      num = strlen(value);
+      if (num <= 0 || num >= MAX_FLAGS_NUM) { /* suspicious length ! */
+        continue;
+      }
+      strncpy(user->flags,value,MAX_FLAGS_NUM);
+    } /* flags */
+    else if (strcmp("uid",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid uid %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->uid = num;
+    }
+    else if (strcmp("rights",varname)==0) {
+      num = strtoul(value, &ptr, 0);
+      /* FIXME by default all users have CWD right FIXME */
+      user->userperms = num | RIGHT_CWD;
+    }
+    else if (strcmp("groups",varname)==0) {
+      wzd_group_t * _group;
+      char * group_ptr;
+
+      /* first group */
+      ptr = strtok_r(value,",",&group_ptr);
+      if (!ptr) continue;
+      _group = group_get_by_name(ptr);
+      if (_group != NULL) {
+        user->groups[user->group_num++] = _group->gid;
+      }
+
+      while ( (ptr = strtok_r(NULL,",",&group_ptr)) )
+      {
+        _group = group_get_by_name(ptr);
+        if (_group != NULL) {
+          user->groups[user->group_num++] = _group->gid;
+        }
+      }
+    } /* "groups" */
+    else if (strcmp("tagline",varname)==0) {
+      strncpy(user->tagline,value,MAX_TAGLINE_LENGTH);
+    } /* tagline */
+    else if (strcmp("max_ul_speed",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_ul_speed %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->max_ul_speed = num;
+    } /* max_ul_speed */
+    else if (strcmp("last_login",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid last_login %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->last_login = num;
+    } /* last_login */
+    else if (strcmp("max_dl_speed",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_dl_speed %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->max_dl_speed = num;
+    } /* max_dl_speed */
+    else if (strcmp("bytes_ul_total",varname)==0) {
+      ul_num = strtoull(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid bytes_ul_total %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->stats.bytes_ul_total = ul_num;
+    } /* bytes_ul_total */
+    else if (strcmp("bytes_dl_total",varname)==0) {
+      ul_num = strtoull(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid bytes_dl_total %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->stats.bytes_dl_total = ul_num;
+    } /* bytes_dl_total */
+    else if (strcmp("files_dl_total",varname)==0) {
+      u_num = strtoul(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid files_dl_total %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->stats.files_dl_total = u_num;
+    } /* files_dl_total */
+    else if (strcmp("files_ul_total",varname)==0) {
+      u_num = strtoul(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid files_ul_total %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->stats.files_ul_total = u_num;
+    } /* files_ul_total */
+   else if (strcmp("credits",varname)==0) {
+      ul_num = strtoull(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid credits %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->credits = ul_num;
+    } /* credits */
+
+    else if (strcmp("num_logins",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid number %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->num_logins = (unsigned short)num;
+    } /* else if (strcmp("num_logins",... */
+    else if (strcmp("ratio",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid ratio %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->ratio = num;
+    } /* else if (strcmp("ratio",... */
+    else if (strcmp("user_slots",varname)==0) {
+      u_num = strtoul(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid user_slots %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->user_slots = (unsigned short)u_num;
+    } /* else if (strcmp("user_slots",... */
+    else if (strcmp("leech_slots",varname)==0) {
+      u_num = strtoul(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0') { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid leech_slots %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->leech_slots = (unsigned short)u_num;
+    } /* else if (strcmp("user_slots",... */
+    else if (strcmp("max_idle_time",varname)==0) {
+      num = strtol(value, &ptr, 0);
+      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
+        snprintf(errbuf,sizeof(errbuf),"Invalid max_idle_time %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+      user->max_idle_time = num;
+    } /* max_idle_time */
+    else if (strcmp("ip_allowed",varname)==0) {
+      err = __user_ip_add(user,value);
+      if (err != 0 ) {
+        snprintf(errbuf,sizeof(errbuf),"ERROR unable to add ip %s\n",value);
+        ERRLOG(errbuf);
+        continue;
+      }
+    } /* ip_allowed */
+  };
+
+  return user;
+}
+
+/* ignore section
+ */
+void section_ignore(FILE * file, const char *sectionname, char * buffer, size_t length)
+{
+  while ( fgets(buffer,MAX_LINE-1,file) != NULL ) {
+    chop(buffer);
+
+    if (strlen(buffer) == 0) return;
+  };
+
+  return;
 }
 
 int read_section_groups(FILE * file_user, char * line)
 {
   char c;
-  char *token, *ptr;
+  char *token;
   char errbuf[1024];
   unsigned int directive;
   int err;
-  long num;
-  gid_t gid=1; /* default gid counter */
   wzd_group_t * group_new = NULL;
 
 #if 0
@@ -440,6 +763,8 @@ fprintf(stderr,"Entering section GROUPS\n");
     token = strtok(line," \t");
     if (!token) continue;
 #endif
+
+    /** \todo XXX convert "privgroup  toto" to "groupname=toto", and keep compatibility */
     directive = find_directive(line);
     switch (directive) {
     case D_PRIVGROUP:
@@ -450,109 +775,24 @@ fprintf(stderr,"Entering section GROUPS\n");
         ERRLOG("privgroup should be followed by the group name !\n");
         continue;
       }
-#if 0
-fprintf(stderr,"Defining new private group %s\n",token);
-#endif
-/*
-      if ((++group_count % 256)==0) {
-	group_pool = realloc(group_pool,group_count+256);
-      }*/
+
       if (++group_count >= group_count_max) {
         snprintf(errbuf,sizeof(errbuf),"Too many groups: %d\n",group_count);
         ERRLOG(errbuf);
         continue;
       }
-      group_new = group_allocate_new();
-      list_ins_next(&group_list, list_tail(&group_list), group_new);
 
-      strncpy(group_new->groupname,token,HARD_GROUPNAME_LENGTH-1);
-      group_new->gid = gid++; /* default value for gid */
+      group_new = read_single_group(file_user, token, line, MAX_LINE);
+      /** \todo check if group is valid (gid != -1) and register it */
 
-      break;
-    case D_NONE:
-      err = regexec(&reg_line,line,3,regmatch,0);
-      if (err) {
-        snprintf(errbuf,sizeof(errbuf),"Line '%s' does not respect config line format - ignoring\n",line);
-        ERRLOG(errbuf);
-        continue;
+      if (group_new->gid != (gid_t)-1) {
+        err = group_register(group_new, 1 /* XXX backend id */);
+        if ((gid_t)err != group_new->gid) {
+          snprintf(errbuf,sizeof(errbuf),"ERROR Could not register group %s\n",group_new->groupname);
+          ERRLOG(errbuf);
+        }
       }
-      memcpy(varname,line+regmatch[1].rm_so,regmatch[1].rm_eo-regmatch[1].rm_so);
-      varname[regmatch[1].rm_eo-regmatch[1].rm_so]='\0';
-      memcpy(value,line+regmatch[2].rm_so,regmatch[2].rm_eo-regmatch[2].rm_so);
-      value[regmatch[2].rm_eo-regmatch[2].rm_so]='\0';
 
-      if (!group_new) break;
-
-      if (strcmp("gid",varname)==0) {
-        if (!group_count) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid gid %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->gid = num;
-      }
-      else if (strcasecmp(varname,"max_idle_time")==0) {
-        if (!group_count) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid max_idle_time %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->max_idle_time = num;
-      } /* max_idle_time */
-      else if (strcmp("num_logins",varname)==0) {
-        if (!group_count) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid num_logins %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->num_logins = (unsigned short)num;
-      } /* else if (strcmp("num_logins",... */
-
-      else if (strcmp("ip_allowed",varname)==0) {
-        __group_ip_add(group_new,value);
-      } /* ip_allowed */
-      else if (strcmp("default_home",varname)==0) {
-        strncpy(group_new->defaultpath,value,WZD_MAX_PATH);
-      } /* default_home */
-      else if (strcmp("ratio",varname)==0) {
-        if (!group_count) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid ratio %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->ratio = num;
-      } /* else if (strcmp("ratio",... */
-      else if (strcmp("max_dl_speed",varname)==0) {
-        if (!group_count || !group_new) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid max_dl_speed %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->max_dl_speed = num;
-      } /* max_dl_speed */
-      else if (strcmp("max_ul_speed",varname)==0) {
-        if (!group_count || !group_new) break;
-        num = strtol(value, &ptr, 0);
-        if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-          snprintf(errbuf,sizeof(errbuf),"Invalid max_ul_speed %s\n",value);
-          ERRLOG(errbuf);
-          continue;
-        }
-        group_new->max_ul_speed = num;
-      } /* max_ul_speed */
-      else if (strcmp("tagline",varname)==0) {
-        strncpy(group_new->tagline,value,MAX_TAGLINE_LENGTH);
-      } /* tagline */
       break;
     default:
       ERRLOG("Houston, we have a problem (invalid varname)\n");
@@ -566,10 +806,6 @@ int read_section_users(FILE * file_user, char * line)
 {
   char c;
   int err;
-  long num;
-  unsigned long u_num;
-  u64_t ul_num;
-  char *ptr;
   char errbuf[1024];
   wzd_user_t * user_new = NULL;
 
@@ -602,214 +838,25 @@ fprintf(stderr,"Entering section USERS\n");
     value[regmatch[2].rm_eo-regmatch[2].rm_so]='\0';
 
     if (strcmp("name",varname)==0) {
-      /* begin a new user */
-      user_new = user_allocate_new();
-      list_ins_next(&user_list, list_tail(&user_list), user_new);
-
       if (++user_count >= user_count_max) {
         snprintf(errbuf,sizeof(errbuf),"Too many users defined %d\n",user_count);
         ERRLOG(errbuf);
         continue;
       }
-      strncpy(user_new->username,value,HARD_USERNAME_LENGTH-1);
-    }
-    else if (strcmp("home",varname)==0) {
-      if (!user_count || !user_new) break;
-      /* remove trailing / */
-      if (value[strlen(value)-1] == '/' && strcmp(value,"/")!=0)
-        value[strlen(value)-1] = '\0';
-      DIRNORM(value,strlen(value),0);
-      strncpy(user_new->rootpath,value,WZD_MAX_PATH);
-    }
-    else if (strcmp("pass",varname)==0) {
-      if (!user_count || !user_new) break;
-      strncpy(user_new->userpass,value,MAX_PASS_LENGTH-1);
-    }
-    else if (strcmp("flags",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strlen(value);
-      if (num <= 0 || num >= MAX_FLAGS_NUM) { /* suspicious length ! */
-        continue;
-      }
-      strncpy(user_new->flags,value,MAX_FLAGS_NUM);
-    } /* flags */
-    else if (strcmp("uid",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid uid %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->uid = num;
-    }
-    else if (strcmp("rights",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtoul(value, &ptr, 0);
-      /* FIXME by default all users have CWD right FIXME */
-      user_new->userperms = num | RIGHT_CWD;
-    }
-    else if (strcmp("groups",varname)==0) {
-      gid_t _gid;
-      char * group_ptr;
 
-      if (!user_new) continue;
-      /* first group */
-      ptr = strtok_r(value,",",&group_ptr);
-      if (!ptr) continue;
-      /* we use exported function: FCN_FIND_GROUP */
-      _gid = FCN_FIND_GROUP(ptr,NULL);
-      if (_gid != (gid_t)-1) {
-        user_new->groups[user_new->group_num++] = _gid; /* ouch */
-      }
+      user_new = read_single_user(file_user, value, line, MAX_LINE);
 
-      while ( (ptr = strtok_r(NULL,",",&group_ptr)) )
-      {
-        _gid = FCN_FIND_GROUP(ptr,NULL);
-        if (_gid != (gid_t)-1) {
-          user_new->groups[user_new->group_num++] = _gid; /* ouch */
+      /** \todo check if user is valid (uid != -1, homedir != NULL etc.) */
+
+      if (user_new->uid != (uid_t)-1) {
+        err = user_register(user_new,1 /* XXX backend id */);
+        if ((uid_t)err != user_new->uid) {
+          snprintf(errbuf,sizeof(errbuf),"ERROR Could not register user %s\n",user_new->username);
+          ERRLOG(errbuf);
         }
       }
-    } /* "groups" */
-    else if (strcmp("tagline",varname)==0) {
-      if (!user_new) continue;
-      strncpy(user_new->tagline,value,MAX_TAGLINE_LENGTH);
-    } /* tagline */
-    else if (strcmp("max_ul_speed",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid max_ul_speed %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->max_ul_speed = num;
-    } /* max_ul_speed */
-    else if (strcmp("last_login",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid last_login %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->last_login = num;
-    } /* last_login */
-    else if (strcmp("max_dl_speed",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid max_dl_speed %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->max_dl_speed = num;
-    } /* max_dl_speed */
-    else if (strcmp("bytes_ul_total",varname)==0) {
-      if (!user_count || !user_new) break;
-      ul_num = strtoull(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid bytes_ul_total %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->stats.bytes_ul_total = ul_num;
-    } /* bytes_ul_total */
-    else if (strcmp("bytes_dl_total",varname)==0) {
-      if (!user_count || !user_new) break;
-      ul_num = strtoull(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid bytes_dl_total %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->stats.bytes_dl_total = ul_num;
-    } /* bytes_dl_total */
-    else if (strcmp("files_dl_total",varname)==0) {
-      if (!user_count || !user_new) break;
-      u_num = strtoul(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid files_dl_total %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->stats.files_dl_total = u_num;
-    } /* files_dl_total */
-    else if (strcmp("files_ul_total",varname)==0) {
-      if (!user_count || !user_new) break;
-      u_num = strtoul(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid files_ul_total %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->stats.files_ul_total = u_num;
-    } /* files_ul_total */
-   else if (strcmp("credits",varname)==0) {
-      if (!user_count || !user_new) break;
-      ul_num = strtoull(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid credits %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->credits = ul_num;
-    } /* credits */
 
-    else if (strcmp("num_logins",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid number %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->num_logins = (unsigned short)num;
-    } /* else if (strcmp("num_logins",... */
-    else if (strcmp("ratio",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid ratio %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->ratio = num;
-    } /* else if (strcmp("ratio",... */
-    else if (strcmp("user_slots",varname)==0) {
-      if (!user_count || !user_new) break;
-      u_num = strtoul(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid user_slots %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->user_slots = (unsigned short)u_num;
-    } /* else if (strcmp("user_slots",... */
-    else if (strcmp("leech_slots",varname)==0) {
-      if (!user_count || !user_new) break;
-      u_num = strtoul(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0') { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid leech_slots %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->leech_slots = (unsigned short)u_num;
-    } /* else if (strcmp("user_slots",... */
-    else if (strcmp("max_idle_time",varname)==0) {
-      if (!user_count || !user_new) break;
-      num = strtol(value, &ptr, 0);
-      if (ptr == value || *ptr != '\0' || num < 0) { /* invalid number */
-        snprintf(errbuf,sizeof(errbuf),"Invalid max_idle_time %s\n",value);
-        ERRLOG(errbuf);
-        continue;
-      }
-      user_new->max_idle_time = num;
-    } /* max_idle_time */
-    else if (strcmp("ip_allowed",varname)==0) {
-      if (!user_new) continue;
-      __user_ip_add(user_new,value);
-    } /* ip_allowed */
+    }
   }
   return 0;
 }
@@ -819,8 +866,6 @@ int read_files(const char *filename)
   FILE *file_user;
   char * line, * token, *ptr;
   int ret;
-  wzd_user_t * user;
-  wzd_group_t * group;
   char errbuf[1024];
 
   if (!filename || strlen(filename)>=256) {
@@ -860,8 +905,9 @@ int read_files(const char *filename)
   group_count=0;
 /*  group_pool = malloc(256*sizeof(wzd_group_t));*/
 
+#if 0
   /* XXX We always add a user nobody and a group nogroup */
-  user = user_allocate_new();
+  user = user_allocate();
   list_ins_next(&user_list,NULL,user);
   strcpy(user->username,"nobody");
   strcpy(user->userpass,"------");
@@ -874,13 +920,16 @@ int read_files(const char *filename)
   user->max_ul_speed = 1; /* at this rate, even if you can download it will be ... slow ! */
   user->max_dl_speed = 1;
   user_count++;
+#endif
 
-  group = group_allocate_new();
+#if 0
+  group = group_allocate();
   list_ins_next(&group_list,NULL,group);
   strcpy(group->groupname,"nogroup");
   group->gid = (gid_t)-1;
   group->groupperms = 0; /* should be enough ! */
   group_count++;
+#endif
 
   while (1) {
     ptr = fgets(line,MAX_LINE-1,file_user);
@@ -896,7 +945,7 @@ int read_files(const char *filename)
       token = strtok_r(line+1,"]",&ptr);
       if (strcasecmp("USERS",token)==0) ret = read_section_users(file_user,line);
       else if (strcasecmp("GROUPS",token)==0) ret = read_section_groups(file_user,line);
-      else if (strcasecmp("HOSTS",token)==0) ret = read_section_hosts(file_user,line);
+      else if (strcasecmp("HOSTS",token)==0) section_ignore(file_user,token,line,MAX_LINE);
       else {
         snprintf(errbuf,sizeof(errbuf),"Unkown section %s\n",token);
         ERRLOG(errbuf);
