@@ -30,8 +30,11 @@
  */
 
 void encrypt (char *block, int edflag);
-void setkey (char *key);
+void wzd_setkey (char *key);
+
+#ifndef crypt
 char * crypt (const char *key, const char *salt);
+#endif
 
 /*! @} */
 
