@@ -25,7 +25,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "../config.h"
+# include "config.h"
 #endif
 
 #include <stdio.h>
@@ -43,7 +43,11 @@
 # include <security/pam_appl.h>
 #endif
 
-#include <security/pam_misc.h>
+#ifdef HAVE_OPENPAM_H
+# include <security/openpam.h>
+#elif defined (HAVE_PAM_MISC_H)
+# include <security/pam_misc.h>
+#endif
 
 #include "wzd_auth.h"
 
