@@ -29,7 +29,7 @@
   */
 
 #ifdef HAVE_CONFIG_H
-# include "../config.h"
+# include "config.h"
 #endif
 
 #include <stdio.h>
