@@ -59,9 +59,11 @@
 #include "wzd_cache.h"
 #include "wzd_configfile.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_misc.h"
 #include "wzd_libmain.h"
 #include "wzd_log.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -73,6 +75,7 @@
 
 #endif /* WZD_USE_PCH */
 
+
 static int _trigger_user_max_dl(wzd_user_t * user);
 static int _trigger_user_max_ul(wzd_user_t * user);
 
@@ -111,175 +114,128 @@ static void backend_clear_struct(wzd_backend_def_t *backend)
   backend->filename = NULL;
   backend->handle = NULL;
 
+  if (backend->b) {
+    wzd_free(backend->b->name);
+  }
+
   wzd_free(backend->b);
   backend->b = NULL;
 }
 
-int backend_validate(const char *backend, const char *pred, const char *version)
+wzd_backend_def_t * backend_validate(const char *backend, const char *pred, const char *version)
 {
   fs_filestat_t st;
   int ret;
-  void * handle;
-  char filename[1024];
-  char path[1024];
-  int length;
+  wzd_backend_def_t * def;
 
-  /* default: current path */
-  strcpy(path,".");
-  length=(int)strlen(path); /* FIXME wtf are these 4 lines for ? */
-  /* add a / at the end if not present - XXX will conflict if last char is \ ? */
-  if (path[length-1]!='/') {
-    path[length++] = '/';
-    path[length]='\0';
-  }
-
-  DIRNORM((char*)backend,strlen(backend),0);
-  /* TODO if backend name already contains .so, do not add .o */
-  /* if backend name contains /, do not add path */
-  if (strchr(backend,'/')==NULL)
-#ifdef WIN32
-    length = snprintf(filename,1024,"%slibwzd%s.dll",path,backend);
-#else
-    length = snprintf(filename,1024,"%slibwzd%s.so",path,backend);
-#endif
-  else
-    length = snprintf(filename,1024,"%s",backend);
-  if (length<0)
-  {
-    out_err(LEVEL_HIGH,"Backend name too long (%s:%d)\n",__FILE__,__LINE__);
-    return 1;
-  }
-  ret = fs_file_lstat(filename,&st);
+  ret = fs_file_lstat(backend,&st);
   if (ret) {
-    out_err(LEVEL_HIGH,"Could not stat backend '%s'\n",filename);
-    out_err(LEVEL_HIGH,"errno: %d error: %s\n",errno, strerror(errno));
-    return 1;
+    out_log(LEVEL_HIGH,"Could not stat backend '%s'\n",backend);
+    out_log(LEVEL_HIGH,"errno: %d error: %s\n",errno, strerror(errno));
+    return NULL;
   }
 
-  /* test dlopen */
-  handle = dlopen(filename,DL_ARG);
-  if (!handle) {
-    out_err(LEVEL_HIGH,"Could not dlopen backend '%s'\n",filename);
-    out_err(LEVEL_HIGH,"errno: %d error: %s\n",errno, strerror(errno));
-    out_err(LEVEL_HIGH,"dlerror: %s\n",dlerror());
-    return 1;
+  def = backend_register(backend, NULL /* dynamic backend */);
+  if (def == NULL) {
+    out_err(LEVEL_HIGH,"ERROR while registering backend '%s'\n",backend);
+    return NULL;
   }
 
-  /* search wzd_backend_init. If found, use the new interface */
-  {
-    typedef int (*backend_init_function)(wzd_backend_t*);
-
-    backend_init_function fcn;
-
-    fcn = (backend_init_function)dlsym(handle, DL_PREFIX "wzd_backend_init");
-    if (fcn) {
-      dlclose(handle);
-      return 0;
-    }
-  }
-
-  out_err(LEVEL_HIGH,"%s does not seem to be a valid backend - there are missing functions\n",backend);
-  dlclose(handle);
-  return 1;
+  return def;
 }
 
-int backend_init(const char *backend, unsigned int user_max, unsigned int group_max)
+/**
+ * \brief Initialize backend
+ * \param backend The backend name
+ */
+int backend_init(wzd_backend_def_t * backend)
 {
-  void * handle;
-  char filename[1024];
-  char path[1024];
-  int length;
+  wzd_backend_t * b;
   int ret;
 
-  /* default: current path */
-  strcpy(path,".");
-  length=(int)strlen(path); /* FIXME wtf are these 4 lines for ? */
-  /* add a / at the end if not present - XXX will conflict if last char is \ ? */
-  if (path[length-1]!='/') {
-    path[length++] = '/';
-    path[length]='\0';
-  }
+  WZD_ASSERT(backend != NULL);
+  if (backend == NULL) return -1;
 
-  DIRNORM((char*)backend,strlen(backend),0);
-  /* TODO if backend name already contains .so, do not add .o */
-  /* if backend name contains /, do not add path */
-  if (strchr(backend,'/')==NULL)
-#ifdef __CYGWIN__
-    length = snprintf(filename,1024,"%slibwzd%s.dll",path,backend);
-#else
-    length = snprintf(filename,1024,"%slibwzd%s.so",path,backend);
-#endif
-  else
-    length = snprintf(filename,1024,"%s",backend);
-  if (length<0)
-  {
-    out_err(LEVEL_HIGH,"Backend name too long (%s:%d)\n",__FILE__,__LINE__);
-    return 1;
-  }
+  b = backend->b;
 
-  /* test dlopen */
-  handle = dlopen(filename,DL_ARG);
-  if (!handle) {
-    out_log(LEVEL_HIGH,"Could not dlopen backend '%s'\n",filename);
-    out_log(LEVEL_HIGH,"errno: %d error: %s\n",errno, strerror(errno));
-    out_log(LEVEL_HIGH,"dlerror: %s\n",dlerror());
-    return 1;
-  }
+  WZD_ASSERT(backend != NULL);
+  if (backend == NULL) return -1;
 
-  mainConfig->backend.handle = handle;
+  if (b->backend_init) {
+    wzd_string_t * str;
+    /* LOGFILE */
+    str = config_get_string(mainConfig->cfg_file, b->name, "param", NULL);
+    if (str) {
+      wzd_free(backend->param);
+      backend->param = wzd_strdup(str_tochar(str));
+      str_deallocate(str);
+    }
 
-  /* search wzd_backend_init. If found, use the new interface */
-  {
-    typedef int (*backend_init_function)(wzd_backend_t*);
-
-    backend_init_function fcn;
-    wzd_backend_t * b;
-
-    fcn = (backend_init_function)dlsym(handle, DL_PREFIX "wzd_backend_init");
-    if (fcn) {
-
-      b = mainConfig->backend.b = wzd_malloc(sizeof(wzd_backend_t));
-      memset(b,0,sizeof(wzd_backend_t));
-      b->struct_version = STRUCT_BACKEND_VERSION;
-
-      if (backend != mainConfig->backend.filename) /* strings must not overlap */
-      {
-        wzd_free(mainConfig->backend.filename);
-        mainConfig->backend.filename = wzd_strdup(backend);
-      }
-
-      ret = (*fcn)(b);
-
-      if (b->backend_init) {
-        wzd_string_t * str;
-        /* LOGFILE */
-        str = config_get_string(mainConfig->cfg_file, b->name, "param", NULL);
-        if (str) {
-          wzd_free(mainConfig->backend.param);
-          mainConfig->backend.param = wzd_strdup(str_tochar(str));
-          str_deallocate(str);
-        }
-
-        ret = (b->backend_init)(mainConfig->backend.param);
-        if (ret) { /* backend says NO */
-          backend_clear_struct(&mainConfig->backend);
-          dlclose(handle);
-          return ret;
-        }
-      } else {
-        out_log(LEVEL_CRITICAL,"FATAL: backend %s does not define init method\n",b->name);
-        backend_clear_struct(&mainConfig->backend);
-        dlclose(handle);
-        return -1;
-      }
-
-      out_log(LEVEL_INFO,"Backend %s loaded (new interface)\n",backend);
-
-      return ret;
+    ret = (b->backend_init)(backend->param);
+    if (ret) {
+      out_log(LEVEL_HIGH,"ERROR could not backend %s, init function returned %d\n",backend->filename,ret);
+      return 1;
     }
   }
 
-  return -1;
+  return 0;
+}
+
+/** \brief Register backend
+ * Use \a filename for dynamic modules (shared libraries)
+ * \a fcn for static modules.
+ * When loading a static module, \a filename is used as a comment
+ */
+struct wzd_backend_def_t * backend_register(const char * filename, backend_init_function_t fcn)
+{
+  wzd_backend_def_t * def = NULL;
+  wzd_backend_t * b;
+  void * handle = NULL;
+  int ret;
+
+  if (filename == NULL && fcn == NULL) return NULL;
+
+  if (fcn == NULL) {
+    /* test dlopen */
+    handle = dlopen(filename,DL_ARG);
+    if (!handle) {
+      out_log(LEVEL_HIGH,"Could not dlopen backend '%s'\n",filename);
+      out_log(LEVEL_HIGH,"errno: %d error: %s\n",errno, strerror(errno));
+      out_log(LEVEL_HIGH,"dlerror: %s\n",dlerror());
+      return NULL;
+    }
+
+    fcn = (backend_init_function_t)dlsym(handle, DL_PREFIX "wzd_backend_init");
+    if (fcn == NULL) {
+      out_err(LEVEL_HIGH,"%s does not seem to be a valid backend - function %s was not found\n",filename,"wzd_backend_init");
+      dlclose(handle);
+      return NULL;
+    }
+  }
+
+  b = wzd_malloc(sizeof(wzd_backend_t));
+  memset(b,0,sizeof(wzd_backend_t));
+  b->struct_version = STRUCT_BACKEND_VERSION;
+  b->backend_id = 1; /** \todo auto-increment */
+
+  ret = (*fcn)(b);
+
+  if (ret != 0) {
+    out_log(LEVEL_HIGH,"ERROR could not backend %s, init function returned %d\n",filename,ret);
+    wzd_free(b);
+    if (handle != NULL) dlclose(handle);
+    return NULL;
+  }
+
+  def = wzd_malloc(sizeof(wzd_backend_def_t));
+  memset(def,0,sizeof(wzd_backend_def_t));
+  if (handle != NULL) {
+    def->handle = handle;
+    def->filename = wzd_strdup(filename);
+  }
+  def->b = b;
+
+  return def;
 }
 
 int backend_close(const char *backend)
@@ -287,48 +243,56 @@ int backend_close(const char *backend)
   int (*fini_fcn)(void) = NULL;
   int ret;
 
-  if (!backend || !mainConfig->backend.filename) return 1;
+  if (!backend || !mainConfig->backends) return 1;
+
+  if (backend_inuse(backend) > 0) {
+    out_log(LEVEL_NORMAL,"Attempting to close backend %s while in use\n",backend);
+    return 1;
+  }
 
   /* step 1: check that backend == mainConfig->backend.name */
-  if (strcmp(backend,mainConfig->backend.filename)!=0) return 1;
+  if (strcmp(backend,mainConfig->backends->filename)!=0) return 1;
 
   /* step 2: call end function */
-  if (mainConfig->backend.b) {
-    fini_fcn = ((wzd_backend_t*)mainConfig->backend.b)->backend_exit;
+  if (mainConfig->backends->b) {
+    fini_fcn = mainConfig->backends->b->backend_exit;
   }
   if (fini_fcn) {
     ret = (*fini_fcn)();
     if (ret) {
       out_log(LEVEL_CRITICAL,"Backend %s reported errors on exit (handle %lu)\n",
-          backend,mainConfig->backend.handle);
+          backend,(unsigned long)mainConfig->backends->handle);
 /*      return 1;*/
     }
   }
 
   /* close backend */
   ret = 0;
-  if (mainConfig->backend.handle)
+  if (mainConfig->backends->handle)
   {
     char * tempname = strdup(backend);
-    ret = dlclose(mainConfig->backend.handle);
+    ret = dlclose(mainConfig->backends->handle);
     if (ret) {
 #ifdef WIN32
       ret = GetLastError();
-      out_log(LEVEL_INFO," Could not close backend %s (handled %lu)\n Error %d %s\n",
-          tempname,mainConfig->backend.handle, ret,strerror(ret));
-      backend_clear_struct(&mainConfig->backend);
+      out_log(LEVEL_INFO," Could not close backend %s (handle %lu)\n Error %d %s\n",
+          tempname,mainConfig->backends->handle, ret,strerror(ret));
 #else
       out_log(LEVEL_INFO,"Could not close backend %s (handle %lu)\n",
-          tempname,mainConfig->backend.handle);
+          tempname,(unsigned long)mainConfig->backends->handle);
       out_log(LEVEL_INFO," Error '%s'\n",dlerror());
 #endif
+      backend_clear_struct(mainConfig->backends);
+
       free(tempname);
       return 1;
     }
     free(tempname);
   }
 
-  backend_clear_struct(&mainConfig->backend);
+  backend_clear_struct(mainConfig->backends);
+
+  /** \todo XXX remove backend from list */
 
   return 0;
 }
@@ -337,19 +301,25 @@ int backend_reload(const char *backend)
 {
   int ret;
 
+#if 0
   ret = backend_close(backend);
   if (ret) return 1;
 
   ret = backend_init(backend,0 /* max users */,0 /* max groups */);
   if (ret) return 1;
+#endif
+  out_log(LEVEL_HIGH,"backend_reload: not yet implemented\n");
 
-  return 0;
+  return 1;
 }
 
 wzd_user_t * backend_get_user(uid_t userid)
 {
   wzd_backend_t * b;
-  if ( (b = mainConfig->backend.b) && b->backend_get_user)
+
+  if (userid == (uid_t)-1) return NULL;
+
+  if ( (b = mainConfig->backends->b) && b->backend_get_user)
     return b->backend_get_user(userid);
 
   if (b == NULL)
@@ -365,7 +335,7 @@ int backend_find_user(const char *name, wzd_user_t * user, int * userid)
   int ret;
   wzd_backend_t * b;
 
-  if ( (b = mainConfig->backend.b) && b->backend_find_user)
+  if ( (b = mainConfig->backends->b) && b->backend_find_user)
     ret = b->backend_find_user(name,user);
   else {
     if (b == NULL)
@@ -391,16 +361,14 @@ int backend_find_user(const char *name, wzd_user_t * user, int * userid)
 /** wrappers to user list */
 wzd_user_t * GetUserByID(uid_t id)
 {
-  wzd_user_t *user, *user_return;
+  wzd_user_t *user;
   wzd_backend_t * b;
 
   if (!mainConfig) return NULL;
 
-  /* try cache first */
-  if ( (user = usercache_getbyuid( id )) )
-    return user;
+  if (id == (uid_t)-1) return NULL;
 
-  if ( (b = mainConfig->backend.b) && b->backend_get_user) {
+  if ( (b = mainConfig->backends->b) && b->backend_get_user) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     user = b->backend_get_user(id);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -413,10 +381,7 @@ wzd_user_t * GetUserByID(uid_t id)
     return NULL;
   }
 
-  if (!user) return NULL;
-  user_return = usercache_add( user );
-  wzd_free(user);
-  return user_return;
+  return user;
 }
 
 wzd_user_t * GetUserByName(const char *name)
@@ -428,11 +393,7 @@ wzd_user_t * GetUserByName(const char *name)
   if (!mainConfig || !name || strlen(name)<=0) return NULL;
 out_err(LEVEL_CRITICAL,"GetUserByName %s\n",name);
 
-  /* try cache first */
-  if ( (user = usercache_getbyname( name )) )
-    return user;
-
-  if ( (b = mainConfig->backend.b) && b->backend_find_user) {
+  if ( (b = mainConfig->backends->b) && b->backend_find_user) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     uid = b->backend_find_user(name,user);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -468,7 +429,7 @@ uid_t GetUserIDByName(const char *name)
 wzd_group_t * backend_get_group(gid_t groupid)
 {
   wzd_backend_t * b;
-  if ( (b = mainConfig->backend.b) && b->backend_get_group)
+  if ( (b = mainConfig->backends->b) && b->backend_get_group)
     return b->backend_get_group(groupid);
 
   if (b == NULL)
@@ -483,7 +444,7 @@ int backend_find_group(const char *name, wzd_group_t * group, int * groupid)
   int ret;
   wzd_backend_t * b;
 
-  if ( (b = mainConfig->backend.b) && b->backend_find_group)
+  if ( (b = mainConfig->backends->b) && b->backend_find_group)
     ret = b->backend_find_group(name,group);
   else {
     if (b == NULL)
@@ -510,16 +471,12 @@ int backend_find_group(const char *name, wzd_group_t * group, int * groupid)
 /** wrappers to Group list */
 wzd_group_t * GetGroupByID(gid_t id)
 {
-  wzd_group_t * group = NULL, * group_return;
+  wzd_group_t * group = NULL;
   wzd_backend_t * b;
 
   if (!mainConfig) return NULL;
 
-  /* try cache first */
-  if ( (group = groupcache_getbygid( id )) )
-    return group;
-
-  if ( (b = mainConfig->backend.b) && b->backend_get_group) {
+  if ( (b = mainConfig->backends->b) && b->backend_get_group) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     group = b->backend_get_group(id);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -532,10 +489,7 @@ wzd_group_t * GetGroupByID(gid_t id)
     return NULL;
   }
 
-  if (!group) return NULL;
-  group_return = groupcache_add( group );
-  wzd_free(group);
-  return group_return;
+  return group;
 }
 
 wzd_group_t * GetGroupByName(const char *name)
@@ -546,11 +500,7 @@ wzd_group_t * GetGroupByName(const char *name)
 
   if (!mainConfig || !name || strlen(name)<=0) return NULL;
 
-  /* try cache first */
-  if ( (group = groupcache_getbyname( name )) )
-    return group;
-
-  if ( (b = mainConfig->backend.b) && b->backend_find_group) {
+  if ( (b = mainConfig->backends->b) && b->backend_find_group) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     gid = b->backend_find_group(name,group);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -586,7 +536,7 @@ int backend_validate_login(const char *name, wzd_user_t * user, uid_t * userid)
   uid_t ret;
   wzd_backend_t * b;
 
-  if ( (b = mainConfig->backend.b) && b->backend_validate_login) {
+  if ( (b = mainConfig->backends->b) && b->backend_validate_login) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     ret = b->backend_validate_login(name,user);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -617,7 +567,7 @@ int backend_validate_pass(const char *name, const char *pass, wzd_user_t *user,
   uid_t ret;
   wzd_backend_t * b;
 
-  if ( (b = mainConfig->backend.b) && b->backend_validate_pass) {
+  if ( (b = mainConfig->backends->b) && b->backend_validate_pass) {
     WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
     ret = b->backend_validate_pass(name,pass,user);
     WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
@@ -647,7 +597,7 @@ int backend_commit_changes(const char *backend)
 {
   wzd_backend_t * b;
 
-  if ( (b = mainConfig->backend.b) && b->backend_commit_changes)
+  if ( (b = mainConfig->backends->b) && b->backend_commit_changes)
     return b->backend_commit_changes();
 
   if (b == NULL)
@@ -657,32 +607,42 @@ int backend_commit_changes(const char *backend)
   return -1;
 }
 
+/** \brief Check if a backend is currently used
+ * \return The number of users connected currently using this backend
+ */
 int backend_inuse(const char *backend)
 {
   int count;
   ListElmt * elmnt;
   wzd_context_t * context;
+  u16_t backend_id = 0;
+  wzd_user_t * user;
   /* unusually, if backend is not loaded it is not in use ... so no error here */
-  if (!mainConfig->backend.handle) {
+  if (mainConfig->backends==NULL || mainConfig->backends->handle==NULL)
     return -1;
-  }
-  /* TODO we should check here that if someone is loggued he is using the
-   * specific backend
-   */
 
-  /* count user logged */
+  /* find backend id */
+  if (strcmp(backend,mainConfig->backends->b->name)==0)
+    backend_id = mainConfig->backends->b->backend_id;
+
+  if (backend_id == 0) return -1;
+
+  /* count user logged in */
   count = 0;
   for (elmnt=list_head(context_list); elmnt != NULL; elmnt = list_next(elmnt)) {
     context = list_data(elmnt);
     if (context->magic == CONTEXT_MAGIC) {
-      count++;
+      user = GetUserByID(context->userid);
+      if (user->backend_id == backend_id)
+        count++;
     }
   }
+
   return count;
 }
 
 /* if user does not exist, add it */
-int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, unsigned long mod_type)
+int backend_mod_user(const char *backend, uid_t uid, wzd_user_t * user, unsigned long mod_type)
 {
   int ret;
   wzd_backend_t * b;
@@ -690,8 +650,8 @@ int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, u
 
   WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
 
-  if ( (b = mainConfig->backend.b) && b->backend_mod_user)
-    ret = b->backend_mod_user(name,user,mod_type);
+  if ( (b = mainConfig->backends->b) && b->backend_mod_user)
+    ret = b->backend_mod_user(uid,user,mod_type);
   else {
     if (b == NULL)
       out_log(LEVEL_CRITICAL,"Attempt to call a backend function on %s:%d while there is no available backend !\n", __FILE__, __LINE__);
@@ -709,7 +669,7 @@ int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, u
 /*  usercache_invalidate( predicate_name, (void *)name );*/
 
   if (!ret && user) { /* modification ok, reload user */
-    if ( (b = mainConfig->backend.b) && b->backend_get_user)
+    if ( (b = mainConfig->backends->b) && b->backend_get_user)
       new_user = b->backend_get_user(user->uid);
     else {
       if (b == NULL)
@@ -722,14 +682,11 @@ int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, u
 
     if (new_user != user) {
       if (new_user) {
-        wzd_user_t * _tmp_user = usercache_getbyuid( user->uid );
-        if (_tmp_user) *_tmp_user = *new_user;
         *user = *new_user;
         wzd_free(new_user);
-    }
-    } else {
-      /* user was deleted */
-      usercache_invalidate( predicate_name, (void *)name );
+      } else {
+        wzd_free(user);
+      }
     }
   }
 
@@ -738,18 +695,16 @@ int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, u
 }
 
 /* if group does not exist, add it */
-int backend_mod_group(const char *backend, const char *name, wzd_group_t * group, unsigned long mod_type)
+int backend_mod_group(const char *backend, gid_t gid, wzd_group_t * group, unsigned long mod_type)
 {
   int ret;
   wzd_backend_t * b;
   wzd_group_t * new_group;
 
-  WZD_ASSERT( group != NULL);
-
   WZD_MUTEX_LOCK(SET_MUTEX_BACKEND);
 
-  if ( (b = mainConfig->backend.b) && b->backend_mod_group)
-    ret = b->backend_mod_group(name,group,mod_type);
+  if ( (b = mainConfig->backends->b) && b->backend_mod_group)
+    ret = b->backend_mod_group(gid,group,mod_type);
   else {
     if (b == NULL)
       out_log(LEVEL_CRITICAL,"Attempt to call a backend function on %s:%d while there is no available backend !\n", __FILE__, __LINE__);
@@ -762,7 +717,7 @@ int backend_mod_group(const char *backend, const char *name, wzd_group_t * group
 /*  groupcache_invalidate( predicate_groupname, (void *)name );*/
 
   if (!ret && group) { /* modification ok, reload group */
-    if ( (b = mainConfig->backend.b) && b->backend_get_group)
+    if ( (b = mainConfig->backends->b) && b->backend_get_group)
       new_group = b->backend_get_group(group->gid);
     else {
       if (b == NULL)
@@ -773,17 +728,14 @@ int backend_mod_group(const char *backend, const char *name, wzd_group_t * group
       return -1;
     }
 
-    if (group != new_group) {
+    if (new_group != group) {
       if (new_group) {
-        wzd_group_t * _tmp_group = groupcache_getbygid( group->gid );
-        if (_tmp_group) *_tmp_group = *new_group;
         *group = *new_group;
         wzd_free(new_group);
+      } else {
+        wzd_free(group);
       }
     }
-    } else {
-      /* group was deleted */
-      groupcache_invalidate( predicate_groupname, (void *)name );
   }
 
   WZD_MUTEX_UNLOCK(SET_MUTEX_BACKEND);
