@@ -29,7 +29,10 @@
 
 #include "wzd_structs.h"
 
-#define STRUCT_BACKEND_VERSION  100
+/*
+ * 101 backend_id
+ */
+#define STRUCT_BACKEND_VERSION  101
 
 /** \brief Initialization for backends */
 struct wzd_backend_t {
@@ -47,13 +50,16 @@ struct wzd_backend_t {
   uid_t (*backend_find_user) (const char *, wzd_user_t *);
   gid_t (*backend_find_group) (const char *, wzd_group_t *);
   int (*backend_chpass) (const char *, const char *);
-  int (*backend_mod_user) (const char *, wzd_user_t *, unsigned long);
-  int (*backend_mod_group) (const char *, wzd_group_t *, unsigned long);
+  int (*backend_mod_user) (uid_t, wzd_user_t *, unsigned long);
+  int (*backend_mod_group) (gid_t, wzd_group_t *, unsigned long);
   int (*backend_commit_changes) (void);
 
   int (*backend_exit)(void);
+
+  u16_t backend_id;
 };
 
+
 /* used to know what was modified in update functions */
 #define	_USER_NOTHING		0
 #define	_USER_USERNAME		1<<0
@@ -76,7 +82,8 @@ struct wzd_backend_t {
 #define	_USER_USERSLOTS		1<<17
 #define	_USER_LEECHSLOTS	1<<18
 #define	_USER_RATIO		1<<19
-#define _USER_ALL	0xffffffff
+#define _USER_ALL               0x0000ffff
+#define _USER_CREATE            0x01000000
 
 #define _GROUP_NOTHING		0
 #define	_GROUP_GROUPNAME	1<<0
@@ -90,25 +97,34 @@ struct wzd_backend_t {
 #define	_GROUP_NUMLOGINS	1<<8
 #define	_GROUP_TAGLINE		1<<9
 #define	_GROUP_GID		1<<10
-#define _GROUP_ALL	0xffffffff
+#define _GROUP_ALL              0x0000ffff
+#define _GROUP_CREATE           0x01000000
 
 
 char *backend_get_version(wzd_backend_def_t *backend);
 char *backend_get_name(wzd_backend_def_t *backend);
-int backend_validate(const char *backend, const char *pred, const char *version);
+
+wzd_backend_def_t * backend_validate(const char *backend, const char *pred, const char *version);
+
+/** \brief Register backend
+ * Use \a filename for dynamic modules (shared libraries)
+ * \a fcn for static modules.
+ * When loading a static module, \a filename is used as a comment
+ */
+struct wzd_backend_def_t * backend_register(const char * filename, backend_init_function_t fcn);
 
 /**
- * \brief loads backend
+ * \brief Initialize backend
  * \param backend The backend name
- * \param user_max Max number of users to store in user_list (NOT used! !)
- * \param group_max Max number of group to store in group_list (NOT used! !)
  */
-int backend_init(const char *backend, unsigned int user_max, unsigned int group_max);
+int backend_init(wzd_backend_def_t * backend);
 
 int backend_close(const char *backend);
 
 int backend_reload(const char *backend);
 
+enum { INVALID_USER = (uid_t)-1, GET_USER_LIST = (uid_t)-2 };
+
 /**
  * \brief Get user informations
  * \param userid The user id, or the special value (uid_t)-2
@@ -121,6 +137,8 @@ int backend_reload(const char *backend);
  */
 wzd_user_t * backend_get_user(uid_t userid);
 
+enum { INVALID_GROUP = (gid_t)-1, GET_GROUP_LIST = (gid_t)-2 };
+
 /**
  * \brief Get group informations
  * \param groupid The group id, or the special value (gid_t)-2
@@ -144,15 +162,18 @@ int backend_validate_pass(const char *name, const char *pass, wzd_user_t *user,
 /* if user does not exist, add it
  * if struct user is NULL, delete user
  */
-int backend_mod_user(const char *backend, const char *name, wzd_user_t * user, unsigned long mod_type);
+int backend_mod_user(const char *backend, uid_t uid, wzd_user_t * user, unsigned long mod_type);
 
 /* if group does not exist, add it
  * if struct group is null, delete group
  */
-int backend_mod_group(const char *backend, const char *name, wzd_group_t * group, unsigned long mod_type);
+int backend_mod_group(const char *backend, gid_t gid, wzd_group_t * group, unsigned long mod_type);
 
 int backend_commit_changes(const char *backend);
 
+/** \brief Check if a backend is currently used
+ * \return The number of users connected currently using this backend
+ */
 int backend_inuse(const char *backend);
 
 #ifndef STRINGIFY
