@@ -55,9 +55,11 @@
 
 #include "wzd_structs.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_libmain.h"
 #include "wzd_log.h"
 #include "wzd_misc.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -582,244 +584,3 @@ void wzd_cache_purge(void)
   WZD_MUTEX_UNLOCK(SET_MUTEX_CACHE);
 }
 
-
-
-static CHTBL index_user_name;
-static CHTBL index_user_uid;
-static CHTBL index_group_name;
-static CHTBL index_group_gid;
-
-int hash_uid(const void *key)
-{
-  return ( (unsigned int) key );
-}
-
-int uidcmp(const void *key1, const void *key2)
-{
-  return ( ((unsigned int)key1) != ((unsigned int)key2) );
-}
-
-int predicate_uid(wzd_user_t * user, void * arg)
-{
-  return (user->username[0] != '\0' && user->uid == (unsigned int)arg);
-}
-
-int predicate_name(wzd_user_t * user, void * arg)
-{
-  return (strcmp(user->username,(char*)arg)==0);
-}
-
-int predicate_gid(wzd_group_t * group, void * arg)
-{
-  return (group->groupname[0] != '\0' && group->gid == (unsigned int)arg);
-}
-
-int predicate_groupname(wzd_group_t * group, void * arg)
-{
-  return (strcmp(group->groupname,(char*)arg)==0);
-}
-
-int predicate_groupname(wzd_group_t * group, void * arg);
-
-#ifdef WZD_DBG_UGCACHE
-static void _user_free(wzd_user_t *user)
-{
-  out_err(LEVEL_CRITICAL,"Freeing user %s (%p)\n",user->username,user);
-}
-#else
-#define _user_free wzd_free
-#endif
-
-void usercache_init(void)
-{
-  chtbl_init(&index_user_name,128,(hash_function)hash_str,(cmp_function)strcmp,wzd_free);
-  chtbl_init(&index_user_uid,37,(hash_function)hash_uid,(cmp_function)uidcmp,wzd_free);
-  chtbl_init(&index_group_name,128,(hash_function)hash_str,(cmp_function)strcmp,wzd_free);
-  chtbl_init(&index_group_gid,37,(hash_function)hash_uid,(cmp_function)uidcmp,wzd_free);
-}
-
-void usercache_fini(void)
-{
-  chtbl_destroy(&index_user_uid);
-  chtbl_destroy(&index_user_name);
-  chtbl_destroy(&index_group_gid);
-  chtbl_destroy(&index_group_name);
-}
-
-
-wzd_user_t * usercache_add(wzd_user_t * user)
-{
-  wzd_user_t * loop_user;
-  wzd_user_t * data;
-
-  if (!user) return NULL;
-
-  if ((chtbl_lookup(&index_user_name, user->username, (void**)&data))==0)
-  {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"user cache refresh %s\n",user->username);
-#endif
-    memcpy(data, user, sizeof(wzd_user_t));
-    return data;
-  }
-
-  /* insert entry */
-  loop_user = wzd_malloc(sizeof(wzd_user_t));
-  memcpy(loop_user,user,sizeof(wzd_user_t));
-  if ((chtbl_insert(&index_user_name, loop_user->username, loop_user, NULL, NULL, _user_free))==0 &&
-       chtbl_insert(&index_user_uid, (void*)loop_user->uid, loop_user, NULL, NULL, NULL)==0)
-  {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"user cache add %s (%p)\n",user->username,loop_user);
-#endif
-    return loop_user;
-  }
-  wzd_free(loop_user);
-
-#ifdef WZD_DBG_UGCACHE
-  out_log(LEVEL_NORMAL,"No more free space in cache\n");
-  out_log(LEVEL_NORMAL,"%s:%d     user: %s\n",__FILE__,__LINE__,user->username);
-#endif
-  return NULL;
-}
-
-wzd_user_t * usercache_getbyname( const char * name )
-{
-  wzd_user_t * user;
-
-  if ((chtbl_lookup(&index_user_name, name, (void**)&user))==0) {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"user cache hit NAME %s\n",user->username);
-#endif
-    return user;
-  }
-
-  return NULL;
-}
-
-wzd_user_t * usercache_getbyuid( unsigned int uid )
-{
-  wzd_user_t * user;
-
-  if ((chtbl_lookup(&index_user_uid, (void*)uid, (void**)&user))==0) {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"user cache hit UID %s\n",user->username);
-#endif
-    return user;
-  }
-
-  return NULL;
-}
-
-wzd_user_t * usercache_search( predicate_user_t p, void * arg )
-{
-  wzd_user_t * user;
-
-  if ((chtbl_search(&index_user_name, (cmp_function)p, arg, (void**)&user))==0)
-  {
-    return user;
-  }
-
-  return NULL;
-}
-
-void usercache_invalidate( predicate_user_t p, void * arg )
-{
-  wzd_user_t * user;
-
-  user = usercache_search( p, arg );
-
-  if (user) {
-    chtbl_remove(&index_user_uid, (void*)user->uid);
-    chtbl_remove(&index_user_name, user->username);
-  }
-}
-
-
-
-wzd_group_t * groupcache_add(wzd_group_t * group)
-{
-  wzd_group_t * loop_group;
-  wzd_group_t * data;
-
-  if (!group) return NULL;
-
-  if ((chtbl_lookup(&index_group_name, group->groupname, (void**)&data))==0)
-  {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"group cache refresh %s\n",group->groupname);
-#endif
-    memcpy(data, group, sizeof(wzd_group_t));
-    return data;
-  }
-
-  /* insert entry */
-  loop_group = wzd_malloc(sizeof(wzd_group_t));
-  memcpy(loop_group,group,sizeof(wzd_group_t));
-  if ((chtbl_insert(&index_group_name, loop_group->groupname, loop_group, NULL, NULL, wzd_free))==0 &&
-       chtbl_insert(&index_group_gid, (void*)loop_group->gid, loop_group, NULL, NULL, NULL)==0)
-  {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"group cache add %s\n",group->groupname);
-#endif
-    return loop_group;
-  }
-  wzd_free(loop_group);
-
-#ifdef WZD_DBG_UGCACHE
-  out_log(LEVEL_NORMAL,"No more free space in cache\n");
-  out_log(LEVEL_NORMAL,"%s:%d     group: %s\n",__FILE__,__LINE__,group->groupname);
-#endif
-  return NULL;
-}
-
-wzd_group_t * groupcache_getbyname( const char * name )
-{
-  wzd_group_t * group;
-
-  if ((chtbl_lookup(&index_group_name, name, (void**)&group))==0) {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"group cache hit NAME %s\n",group->groupname);
-#endif
-    return group;
-  }
-
-  return NULL;
-}
-
-wzd_group_t * groupcache_getbygid( unsigned int gid )
-{
-  wzd_group_t * group;
-
-  if ((chtbl_lookup(&index_group_gid, (void*)gid, (void**)&group))==0) {
-#ifdef WZD_DBG_UGCACHE
-    out_err(LEVEL_INFO,"group cache hit GID %s\n",group->groupname);
-#endif
-    return group;
-  }
-
-  return NULL;
-}
-
-wzd_group_t * groupcache_search( predicate_group_t p, void * arg )
-{
-  wzd_group_t * group;
-
-  if ((chtbl_search(&index_group_name, (cmp_function)p, arg, (void**)&group))==0)
-  {
-    return group;
-  }
-
-  return NULL;
-}
-
-void groupcache_invalidate( predicate_group_t p, void * arg )
-{
-  wzd_group_t * group;
-
-  group = groupcache_search( p, arg );
-
-  if (group) chtbl_remove(&index_group_gid, (void*)group->gid);
-  if (group) chtbl_remove(&index_group_name, group->groupname);
-}
-
