@@ -65,6 +65,7 @@
 #include "wzd_mod.h"
 #include "wzd_data.h"
 #include "wzd_socket.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -84,12 +85,13 @@ void update_last_file(wzd_context_t * context)
   context->last_file.token = context->current_action.token;
 }
 
+/** \brief Close data connection (if opened) */
 void data_close(wzd_context_t * context)
 {
   int ret;
 
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
-  if (context->ssl.data_mode == TLS_PRIV)
+  if (context->tls_data_mode == TLS_PRIV)
     ret = tls_close_data(context);
 #endif
 #ifdef DEBUG
@@ -102,6 +104,37 @@ out_err(LEVEL_FLOOD,"closing data connection fd: %d (control fd: %d)\n",context-
   context->state = STATE_UNKNOWN;
 }
 
+/** \brief End current transfer if any, close data connection and send event
+ */
+void data_end_transfer(int is_upload, int end_ok, wzd_context_t * context)
+{
+  file_unlock(context->current_action.current_file);
+  file_close(context->current_action.current_file, context);
+  FD_UNREGISTER(context->current_action.current_file,"Client file (RETR or STOR)");
+
+  out_xferlog(context,end_ok /* complete */);
+  update_last_file(context);
+
+  context->current_action.current_file = -1;
+  context->current_action.bytesnow = 0;
+  context->state = STATE_COMMAND;
+  data_close(context);
+
+  context->current_action.token = TOK_UNKNOWN;
+
+  {
+    u32_t event_id = (is_upload) ? EVENT_POSTUPLOAD : EVENT_POSTDOWNLOAD;
+    unsigned int reply_code = (end_ok) ? 226 : 426;
+    wzd_user_t * user = GetUserByID(context->userid);
+    wzd_string_t * event_args = str_allocate();
+
+    /** \todo Find a way to indicate if transfer was ok in event */
+    str_sprintf(event_args,"%s %s",user->username,context->current_action.arg);
+    event_send(mainConfig->event_mgr, event_id, reply_code, event_args, context);
+    str_deallocate(event_args);
+  }
+}
+
 int data_set_fd(wzd_context_t * context, fd_set *fdr, fd_set *fdw, fd_set *fde)
 {
   unsigned int action;
@@ -112,6 +145,10 @@ int data_set_fd(wzd_context_t * context, fd_set *fdr, fd_set *fdw, fd_set *fde)
 
   switch (action) {
   case TOK_RETR:
+    if (context->state != STATE_XFER) {
+      out_log(LEVEL_HIGH,"Assertion failed: state != XFER but current action is RETR. Please report me to authors\n");
+      return -1;
+    }
     if (context->datafd==(fd_t)-1 || !fd_is_valid(context->datafd)) {
       out_err(LEVEL_HIGH,"Trying to set invalid datafd (%d) %s:%d\n",
           context->datafd,__FILE__,__LINE__);
@@ -121,6 +158,10 @@ int data_set_fd(wzd_context_t * context, fd_set *fdr, fd_set *fdw, fd_set *fde)
     return context->datafd;
     break;
   case TOK_STOR:
+    if (context->state != STATE_XFER) {
+      out_log(LEVEL_HIGH,"Assertion failed: state != XFER but current action is STOR. Please report me to authors\n");
+      return -1;
+    }
     if (context->datafd==(fd_t)-1 || !fd_is_valid(context->datafd)) {
       out_err(LEVEL_HIGH,"Trying to set invalid datafd (%d) %s:%d\n",
           context->datafd,__FILE__,__LINE__);
@@ -172,25 +213,19 @@ int data_execute(wzd_context_t * context, wzd_user_t * user, fd_set *fdr, fd_set
     n = file_read(context->current_action.current_file,context->data_buffer,mainConfig->data_buffer_length);
     if (n>0) {
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
-      if (context->ssl.data_mode == TLS_CLEAR)
+      if (context->tls_data_mode == TLS_CLEAR)
         ret = clear_write(context->datafd,context->data_buffer,(size_t)n,0,HARD_XFER_TIMEOUT,context);
       else
 #endif
         ret = (context->write_fct)(context->datafd,context->data_buffer,(unsigned int)n,0,HARD_XFER_TIMEOUT,context);
       if (ret <= 0) {
-        /* XXX error/timeout sending data */
-        file_close(context->current_action.current_file, context);
-        FD_UNREGISTER(context->current_action.current_file,"Client file (RETR)");
-        context->current_action.current_file = -1;
-        context->current_action.bytesnow = 0;
-        context->current_action.token = TOK_UNKNOWN;
-        data_close(context);
+/*        out_log(LEVEL_INFO,"INFO error or timeout sending data\n");*/
+        /* error/timeout sending data */
+        data_end_transfer(0 /* is_upload */, 0 /* end_ok */, context);
+
         ret = send_message(426,context);
-        out_err(LEVEL_INFO,"Send 426 message returned %d\n",ret);
-        /*	limiter_free(context->current_limiter);
-                context->current_limiter = NULL;*/
+
         context->idle_time_start = time(NULL);
-        context->state = STATE_COMMAND;
         return 1;
       }
       context->current_action.bytesnow += n;
@@ -203,35 +238,20 @@ int data_execute(wzd_context_t * context, wzd_user_t * user, fd_set *fdr, fd_set
         user->credits -= n;
       context->idle_time_data_start = server_time;
     } else { /* end */
-      file_close(context->current_action.current_file, context);
-      FD_UNREGISTER(context->current_action.current_file,"Client file (RETR)");
+      send_message_raw("226- command ok\r\n",context);
 
-      out_xferlog(context,1 /* complete */);
-      update_last_file(context);
-
-      context->current_action.current_file = -1;
-      context->current_action.bytesnow = 0;
-      context->state = STATE_COMMAND;
-      data_close(context);
+      data_end_transfer(0 /* is_upload */, 1 /* end_ok */, context);
 
 /*      limiter_free(context->current_limiter);
       context->current_limiter = NULL;*/
 
-      /* send message header */
-      send_message_raw("226- command ok\r\n",context);
-      {
-        wzd_string_t * event_args = str_allocate();
-        str_sprintf(event_args,"%s %s",user->username,context->current_action.arg);
-        event_send(mainConfig->event_mgr, EVENT_POSTDOWNLOAD, 226, event_args, context);
-        str_deallocate(event_args);
-      }
       ret = send_message(226,context);
 #ifdef DEBUG
 out_err(LEVEL_INFO,"Send 226 message returned %d\n",ret);
 #endif
 
       /* user will be invalidated */
-      backend_mod_user(mainConfig->backend.filename, user->username, user, _USER_BYTESDL | _USER_CREDITS);
+      backend_mod_user(mainConfig->backends->filename, user->uid, user, _USER_BYTESDL | _USER_CREDITS);
 
       context->current_action.token = TOK_UNKNOWN;
       context->idle_time_start = server_time;
@@ -239,7 +259,7 @@ out_err(LEVEL_INFO,"Send 226 message returned %d\n",ret);
     break;
   case TOK_STOR:
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
-      if (context->ssl.data_mode == TLS_CLEAR)
+      if (context->tls_data_mode == TLS_CLEAR)
         n = clear_read(context->datafd,context->data_buffer,mainConfig->data_buffer_length,0,HARD_XFER_TIMEOUT,context);
       else
 #endif
@@ -258,39 +278,22 @@ out_err(LEVEL_INFO,"Send 226 message returned %d\n",ret);
         user->credits += (user->ratio * n);
       context->idle_time_data_start = server_time;
     } else { /* consider it is finished */
-      file_unlock(context->current_action.current_file);
-      file_close(context->current_action.current_file,context);
-      FD_UNREGISTER(context->current_action.current_file,"Client file (STOR)");
+      send_message_raw("226- command ok\r\n",context);
 
-      out_xferlog(context,1 /* complete */);
-      update_last_file(context);
+      file_unlock(context->current_action.current_file);
+      data_end_transfer(1 /* is_upload */, 1 /* end_ok */, context);
+
+      ret = send_message(226,context);
+#ifdef DEBUG
+      out_err(LEVEL_INFO,"Send 226 message returned %d\n",ret);
+#endif
       /* we increment the counter of uploaded files at the end
        * of the upload
        */
       user->stats.files_ul_total++;
 
-      context->current_action.current_file = -1;
-      context->current_action.bytesnow = 0;
-      context->state = STATE_COMMAND;
-      data_close(context);
-/*      limiter_free(context->current_limiter);
-      context->current_limiter = NULL;*/
-
-      /* send message header */
-      send_message_raw("226- command ok\r\n",context);
-      {
-        wzd_string_t * event_args = str_allocate();
-        str_sprintf(event_args,"%s %s",user->username,context->current_action.arg);
-        event_send(mainConfig->event_mgr, EVENT_POSTUPLOAD, 226, event_args, context);
-        str_deallocate(event_args);
-      }
-      ret = send_message(226,context);
-#ifdef DEBUG
-      out_err(LEVEL_INFO,"Send 226 message returned %d\n",ret);
-#endif
-
       /* user will be invalidated */
-      backend_mod_user(mainConfig->backend.filename, user->username, user, _USER_BYTESUL | _USER_CREDITS);
+      backend_mod_user(mainConfig->backends->filename, user->uid, user, _USER_BYTESUL | _USER_CREDITS);
 
       context->current_action.token = TOK_UNKNOWN;
       context->idle_time_start = server_time;
