@@ -117,6 +117,10 @@ typedef struct
 #include <dirent.h>
 #endif
 
+#ifndef HAVE_STRTOK_R
+# include "libwzd-base/wzd_strtok_r.h"
+#endif
+
 #include <fcntl.h> /* O_RDONLY */
 
 #include "wzd_structs.h"
@@ -126,8 +130,10 @@ typedef struct
 #include "wzd_misc.h"
 #include "wzd_file.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_cache.h"
 #include "wzd_perm.h"
+#include "wzd_user.h"
 #include "wzd_vfs.h"
 
 
@@ -1820,12 +1826,12 @@ struct wzd_file_t * file_stat(const char *filename, wzd_context_t * context)
   file = NULL;
 
   /* find the dir containing the perms file */
-  strncpy(perm_filename,filename,WZD_MAX_PATH);
+  wzd_strncpy(perm_filename,filename,WZD_MAX_PATH);
   length = strlen(perm_filename);
   if (length >1 && perm_filename[length-1]=='/')
     perm_filename[--length] = '\0';
   ptr = strrchr(perm_filename,'/');
-  if (!ptr || *(ptr+1)=='\0') return NULL;
+  if (ptr == NULL) return NULL;
 
   if (!fs_file_lstat(filename,&s)) {
     if (S_ISDIR(s.mode)) { /* isdir */
@@ -1864,7 +1870,7 @@ struct wzd_file_t * file_stat(const char *filename, wzd_context_t * context)
     ++length;
     perm_filename[length-1] = '/';
   }
-  strncpy(perm_filename+length,HARD_PERMFILE,neededlength);
+  wzd_strncpy(perm_filename+length,HARD_PERMFILE,neededlength);
 
   if ( ! readPermFile(perm_filename,&file_list) ) {
     /* we have a permission file */
@@ -1876,6 +1882,20 @@ struct wzd_file_t * file_stat(const char *filename, wzd_context_t * context)
 
   if (!file && nx) return NULL;
 
+  if (file == NULL) { /* create minimal struct */
+    /** \bug XXX FIXME we should not allocate anything here, since this will be a memory leak */
+    file = wzd_malloc(sizeof(struct wzd_file_t));
+
+    wzd_strncpy(file->filename,stripped_filename,sizeof(file->filename));
+    file->owner[0] = '\0';
+    file->group[0] = '\0';
+    file->permissions = mainConfig->umask; /** \todo FIXME default permission */
+    file->acl = NULL;
+    file->kind = FILE_NOTSET;
+    file->data = NULL;
+    file->next_file = NULL;
+  }
+
   if (file) {
     if (S_ISDIR(s.mode)) file->kind = FILE_DIR;
     if (S_ISLNK(s.mode)) file->kind = FILE_LNK;
@@ -2006,6 +2026,7 @@ ssize_t file_write(fd_t fd,const void *data,size_t length)
 
 /** \brief create symlink
  * paths must be absolute
+ * paths must not be / terminated !
  * \todo if paths are relative, convert them ?
  */
 int symlink_create(const char *existing, const char *link)
