@@ -51,10 +51,13 @@
 
 #include "wzd_structs.h"
 
+#include "wzd_configfile.h"
+#include "wzd_group.h"
 #include "wzd_ip.h"
 #include "wzd_log.h"
 #include "wzd_misc.h"
 #include "wzd_socket.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -228,40 +231,11 @@ int ip_compare(const char * ip, const char * pattern)
 }
 
 
-/** IP allowing */
-int ip_add(wzd_ip_list_t **list, const char *newip)
-{
-  wzd_ip_list_t * new_ip_t, *insert_point;
-
-  /* of course this should never happen :) */
-  if (list == NULL) return -1;
-
-  if (strlen(newip) < 1) return -1;
-  if (strlen(newip) >= MAX_IP_LENGTH) return -1; /* upper limit for an hostname */
-
-  new_ip_t = malloc(sizeof(wzd_ip_list_t));
-  new_ip_t->regexp = wzd_strndup(newip,MAX_IP_LENGTH);
-  new_ip_t->next_ip = NULL;
-
-  /* tail insertion, be aware that order is important */
-  insert_point = *list;
-  if (insert_point == NULL) {
-    *list = new_ip_t;
-  } else {
-    while (insert_point->next_ip != NULL)
-      insert_point = insert_point->next_ip;
-
-    insert_point->next_ip = new_ip_t;
-  }
-
-  return 0;
-}
-
 /** \brief Add a new ip to be checked when user logs in
  */
-int ip_add_check(wzd_ip_list_t **list, const char *newip, int is_allowed)
+int ip_add_check(struct wzd_ip_list_t **list, const char *newip, int is_allowed)
 {
-  wzd_ip_list_t * new_ip_t, *insert_point;
+  struct wzd_ip_list_t * new_ip_t, *insert_point;
 
   WZD_ASSERT( list != NULL );
 
@@ -278,6 +252,7 @@ int ip_add_check(wzd_ip_list_t **list, const char *newip, int is_allowed)
   if (insert_point == NULL) {
     *list = new_ip_t;
   } else {
+    /** \note using a circular list would be faster here */
     while (insert_point->next_ip != NULL)
       insert_point = insert_point->next_ip;
 
@@ -289,31 +264,97 @@ int ip_add_check(wzd_ip_list_t **list, const char *newip, int is_allowed)
 
 /** \brief Check if ip is allowed by list.
  *
- * \returns: 1 if allowed, 0 if denied, -1 on error
+ * \returns 1 if allowed, 0 if denied, -1 on error or if not found
  */
-int ip_list_check(wzd_ip_list_t *list, const char *ip)
+int ip_list_check(struct wzd_ip_list_t *list, const char *ip)
 {
-  wzd_ip_list_t * current_ip;
-  const char * ptr_ip;
+  struct wzd_ip_list_t * current_ip;
   char * ptr_test;
 
   current_ip = list;
   while (current_ip) {
-    ptr_ip = ip;
     ptr_test = current_ip->regexp;
     if (*ptr_test == '\0') return -1; /* ip has length 0 ! */
 
-    if (ip_compare(ptr_ip,ptr_test)==1) return current_ip->is_allowed;
+    if (ip_compare(ip,ptr_test)==1) return current_ip->is_allowed;
 
     current_ip = current_ip->next_ip;
   } /* while current_ip */
 
-  return 1; /** \bug FIXME XXX set a default value to return */
+  return -1;
 }
 
-int ip_inlist(wzd_ip_list_t *list, const char *ip)
+/** \brief Check if ip is allowed by list, comparing \a ident if present
+ *
+ * \returns 1 if allowed, 0 if denied, -1 on error or if not found
+ */
+int ip_list_check_ident(struct wzd_ip_list_t *list, const char *ip, const char * ident)
 {
-  wzd_ip_list_t * current_ip;
+  struct wzd_ip_list_t * current_ip;
+  char buffer[1024];
+  const char * ptr;
+  const char * ident_ref, * ip_ref;
+
+  for (current_ip = list; current_ip != NULL; current_ip = current_ip->next_ip) {
+    ip_ref = current_ip->regexp;
+
+    if ( (ptr = strchr(current_ip->regexp,'@'))!=NULL ) {
+      /* split regexp into ident_ref and ip_ref */
+      ip_ref = ptr+1;
+      strncpy(buffer,current_ip->regexp,(ptr-current_ip->regexp));
+      buffer[ptr-current_ip->regexp] = '\0';
+      ident_ref = buffer;
+      /* Check ident and exit if different */
+      if (ident == NULL) {
+        /* if ident is NULL, we can still accept it if ident_ref is the wildcard * */
+        if (strcmp(ident_ref,"*")!=0) continue;
+      } else {
+        if (my_str_compare(ident,ident_ref)!=1) continue;
+      }
+    }
+
+    /* if the ident check is ok, check the ip */
+    if (ip_compare(ip,ip_ref)==1) return current_ip->is_allowed;
+  }
+
+  return -1;
+}
+
+/** \brief Remove \a ip from list
+ * \return 0 if ok, -1 if not found
+ */
+int ip_remove(struct wzd_ip_list_t ** list, const char * ip)
+{
+  struct wzd_ip_list_t * current_ip, * free_ip;
+
+  current_ip = *list;
+  if (current_ip == NULL) return -1;
+
+  /* first ? */
+  if (strcmp(current_ip->regexp, ip)==0) {
+    *list = (*list)->next_ip;
+    wzd_free(current_ip->regexp);
+    wzd_free(current_ip);
+    return 0;
+  }
+
+  while (current_ip->next_ip && current_ip->next_ip->regexp) {
+    if (strcmp(current_ip->next_ip->regexp,ip)==0) {
+      free_ip = current_ip->next_ip;
+      current_ip->next_ip = free_ip->next_ip;
+      wzd_free(free_ip->regexp);
+      wzd_free(free_ip);
+      return 0;
+    }
+    current_ip = current_ip->next_ip;
+  }
+
+  return -1;
+}
+
+int ip_inlist(struct wzd_ip_list_t *list, const char *ip)
+{
+  struct wzd_ip_list_t * current_ip;
   const char * ptr_ip;
   char * ptr_test;
 
@@ -331,9 +372,9 @@ int ip_inlist(wzd_ip_list_t *list, const char *ip)
   return 0;
 }
 
-void ip_list_free(wzd_ip_list_t *list)
+void ip_list_free(struct wzd_ip_list_t *list)
 {
-  wzd_ip_list_t * current, *next;
+  struct wzd_ip_list_t * current, *next;
 
   if (!list) return;
   current = list;
@@ -348,126 +389,6 @@ void ip_list_free(wzd_ip_list_t *list)
   }
 }
 
-int user_ip_add(wzd_user_t * user, const char *newip)
-{
-  int i;
-
-  /* of course this should never happen :) */
-  if (user == NULL || newip==NULL) return -1;
-
-  if (strlen(newip) < 1) return -1;
-  if (strlen(newip) >= MAX_IP_LENGTH) return -1; /* upper limit for an hostname */
-
-  /* tail insertion, be aware that order is important */
-  for (i=0; i<HARD_IP_PER_USER; i++) {
-    if (user->ip_allowed[i][0] == '\0') {
-      strncpy(user->ip_allowed[i],newip,MAX_IP_LENGTH-1);
-      return 0;
-    }
-  }
-  return 1; /* full */
-}
-
-int user_ip_inlist(wzd_user_t * user, const char *ip, const char *ident)
-{
-  int i;
-  const char * ptr_ip;
-  char * ptr_test;
-  const char * ptr;
-  const char * ptr_ident;
-  unsigned int ident_length=0;
-
-  for (i=0; i<HARD_IP_PER_USER; i++) {
-    if (user->ip_allowed[i][0] != '\0') {
-      ptr_ip = ip;
-      ptr_test = user->ip_allowed[i];
-      if (*ptr_test == '\0') return 0; /* ip has length 0 ! */
-
-      ptr = strchr(ptr_test,'@');
-      if (ptr) { /* we have an ident to check */
-        ptr_ident = ptr_test;
-        ident_length = ptr - ptr_ident;
-#ifdef WZD_DBG_IDENT
-        out_log(LEVEL_CRITICAL,"user ip with ident: %s:%d\n",ptr_ident,ident_length);
-#endif
-        ptr_test = (char*)ptr+1;
-        if ( !(*ptr_ident=='*' && ident_length==1) ) {
-          if (!ident || ident[0] == '\0') {
-            continue;
-		  }
-		  if (strncmp(ident,ptr_ident,ident_length) != 0) {
-            /* ident does not match */
-            continue;
-		  }
-        }
-      }
-
-      if (ip_compare(ptr_ip,ptr_test)==1) return 1;
-
-    }
-  } /* while ip */
-
-  return 0;
-}
-
-int group_ip_add(wzd_group_t * group, const char *newip)
-{
-  int i;
-
-  /* of course this should never happen :) */
-  if (group == NULL || newip==NULL) return -1;
-
-  if (strlen(newip) < 1) return -1;
-  if (strlen(newip) >= MAX_IP_LENGTH) return -1; /* upper limit for an hostname */
-
-  /* tail insertion, be aware that order is important */
-  for (i=0; i<HARD_IP_PER_GROUP; i++) {
-    if (group->ip_allowed[i][0] == '\0') {
-      strncpy(group->ip_allowed[i],newip,MAX_IP_LENGTH-1);
-      return 0;
-    }
-  }
-  return 1; /* full */
-}
-
-int group_ip_inlist(wzd_group_t * group, const char *ip, const char *ident)
-{
-  int i;
-  const char * ptr_ip;
-  char * ptr_test;
-  const char * ptr;
-  const char * ptr_ident;
-  unsigned int ident_length=0;
-
-  for (i=0; i<HARD_IP_PER_GROUP; i++) {
-    if (group->ip_allowed[i][0] != '\0') {
-      ptr_ip = ip;
-      ptr_test = group->ip_allowed[i];
-      if (*ptr_test == '\0') return 0; /* ip has length 0 ! */
-
-      ptr = strchr(ptr_test,'@');
-      if (ptr) { /* we have an ident to check */
-        if (!ident) {
-          continue;
-        }
-        ptr_ident = ptr_test;
-        ident_length = ptr - ptr_ident;
-        out_log(LEVEL_CRITICAL,"ident: %s:%d\n",ptr_ident,ident_length);
-        ptr_test = (char*)ptr+1;
-        if ( !(*ptr_ident=='*' && ident_length==1) &&
-            strncmp(ident,ptr_ident,ident_length) != 0) {
-          /* ident does not match */
-          continue;
-        }
-      }
-
-      if (ip_compare(ptr_ip,ptr_test)==1) return 1;
-
-    }
-  } /* while current_ip */
-
-  return 0;
-}
 
 int hostnametoip(const char *hostname, char **ip, size_t *length, net_family_t *family)
 {
@@ -483,6 +404,7 @@ int hostnametoip(const char *hostname, char **ip, size_t *length, net_family_t *
     error = getaddrinfo(hostname,NULL,NULL,&result);
     if (error) {
       out_log(LEVEL_NORMAL,"Error using getaddrinfo: %s\n",gai_strerror(error));
+      *ip = NULL;
       return -1;
     }
     out_err(LEVEL_FLOOD,"Family: %d\n",result->ai_family);
@@ -674,3 +596,73 @@ int iptohostname(const char *ip, net_family_t family, char **hostname, size_t *l
   return -1;
 }
 
+/** \brief Test if remote peer is known as a BNC
+ *
+ * \return 1 if peer is a BNC
+ */
+int ip_is_bnc(const char * remote, wzd_config_t * config)
+{
+  wzd_string_t ** bnc_list;
+  wzd_string_t * bnc;
+  int errcode;
+  int i;
+
+  WZD_ASSERT(config != NULL);
+  WZD_ASSERT(remote != NULL);
+
+  if (!config || !remote) return 0;
+
+  bnc_list = config_get_string_list (config->cfg_file, "GLOBAL", "bnc_list", &errcode);
+  if (!bnc_list) return 0;
+
+  for (i=0; bnc_list[i] != NULL; i++) {
+    bnc = bnc_list[i];
+    if (ip_compare(remote,str_tochar(bnc)) == 1) { /* found */
+      str_deallocate_array(bnc_list);
+      return 1;
+    }
+  }
+
+  str_deallocate_array(bnc_list);
+  return 0;
+}
+
+/** \brief Return our own ip
+ *
+ * \a buffer must be at least 16 bytes long
+ */
+unsigned char * getmyip(int sock, net_family_t family, unsigned char * buffer)
+{
+  struct sockaddr_in sa;
+  unsigned int size;
+
+#if defined(IPV6_SUPPORT)
+  if (family == WZD_INET6) {
+    struct sockaddr_in6 sa6;
+
+    size = sizeof(struct sockaddr_in6);
+    memset(buffer,0,16);
+    if (getsockname(sock,(struct sockaddr *)&sa6,&size)!=-1)
+    {
+      memcpy(buffer,&sa6.sin6_addr,16);
+    } else { /* failed, using localhost */
+      out_log(LEVEL_CRITICAL,"getmyip: could not get my own ip !\n");
+      return NULL;
+    }
+
+    return buffer;
+  }
+#endif /* IPV6_SUPPORT */
+  size = sizeof(struct sockaddr_in);
+  memset(buffer,0,16);
+  if (getsockname(sock,(struct sockaddr *)&sa,&size)!=-1)
+  {
+    memcpy(buffer,&sa.sin_addr,4);
+  } else { /* failed, using localhost */
+    out_log(LEVEL_CRITICAL,"getmyip: could not get my own ip !\n");
+    return NULL;
+  }
+
+  return buffer;
+}
+
