@@ -25,6 +25,12 @@
 #ifndef __WZD_IP_H__
 #define __WZD_IP_H__
 
+struct wzd_ip_list_t {
+  char  * regexp;
+  u8_t  is_allowed;
+  struct wzd_ip_list_t * next_ip;
+};
+
 typedef enum {
   WZD_INET_NONE = 0,
   WZD_INET4     = 2,  /* AF_INET */
@@ -44,27 +50,29 @@ void ip_free(wzd_ip_t * ip);
 /* IP comparison */
 int ip_compare(const char * src, const char *dst);
 
-/* IP allowing */
-int ip_add(wzd_ip_list_t **list, const char *newip);
-
 /** \brief Add a new ip to be checked when user logs in
  */
-int ip_add_check(wzd_ip_list_t **list, const char *newip, int is_allowed);
+int ip_add_check(struct wzd_ip_list_t **list, const char *newip, int is_allowed);
 
 /** \brief Check if ip is allowed by list.
  *
- * \returns: 1 if allowed, 0 if denied, -1 on error
+ * \returns: 1 if allowed, 0 if denied, -1 on error or if not found
  */
-int ip_list_check(wzd_ip_list_t *list, const char *ip);
+int ip_list_check(struct wzd_ip_list_t *list, const char *ip);
 
-int ip_inlist(wzd_ip_list_t *list, const char *ip);
-void ip_list_free(wzd_ip_list_t *list);
+/** \brief Check if ip is allowed by list, comparing \a ident if present
+ *
+ * \returns: 1 if allowed, 0 if denied, -1 on error or if not found
+ */
+int ip_list_check_ident(struct wzd_ip_list_t *list, const char *ip, const char * ident);
 
-int user_ip_add(wzd_user_t * user, const char *newip);
-int user_ip_inlist(wzd_user_t * user, const char *ip, const char *ident);
+/** \brief Remove \a ip from list
+ * \return 0 if ok, -1 if not found
+ */
+int ip_remove(struct wzd_ip_list_t ** list, const char * ip);
 
-int group_ip_add(wzd_group_t * group, const char *newip);
-int group_ip_inlist(wzd_group_t * group, const char *ip, const char *ident);
+int ip_inlist(struct wzd_ip_list_t *list, const char *ip);
+void ip_list_free(struct wzd_ip_list_t *list);
 
 /** \brief Convert a host name to a numeric ip
  *
@@ -82,4 +90,16 @@ int hostnametoip(const char *hostname, char **ip, size_t *length, net_family_t *
  */
 int iptohostname(const char *ip, net_family_t family, char **hostname, size_t *length);
 
+/** \brief Test if remote peer is known as a BNC
+ *
+ * \return 1 if peer is a BNC
+ */
+int ip_is_bnc(const char * remote, wzd_config_t * config);
+
+/** \brief Return our own ip
+ *
+ * \a buffer must be at least 16 bytes long
+ */
+unsigned char * getmyip(int sock, net_family_t family, unsigned char * buffer);
+
 #endif /* __WZD_IP_H__ */
