@@ -36,11 +36,14 @@ typedef enum {
 
   SET_MUTEX_BACKEND,
   SET_MUTEX_CACHE,
+  SET_MUTEX_CRONTAB,
   SET_MUTEX_PERMISSION,
   SET_MUTEX_DIRINFO,
   SET_MUTEX_FILE_T,
   SET_MUTEX_ACL_T,
 
+  SET_MUTEX_USER,
+
   SET_MUTEX_NUM /* must be last */
 } wzd_set_mutext_t;
 
@@ -82,6 +85,12 @@ const char * wzd_get_version_long(void);
 /** \brief remove a context from the list */
 int context_remove(List * context_list, wzd_context_t * context);
 
+/** \brief Allocate a new context */
+wzd_context_t * context_alloc(void);
+
+/** \brief Initialize a context struct */
+void context_init(wzd_context_t * context);
+
 /** \brief Frees a context
  */
 void context_free(wzd_context_t * context);
