@@ -25,7 +25,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "../config.h"
+# include "config.h"
 #endif
 
 #include "wzd_all.h"
@@ -89,6 +89,10 @@
 
 #endif /* _MSC_VER */
 
+#ifndef HAVE_STRTOK_R
+# include "libwzd-base/wzd_strtok_r.h"
+#endif
+
 
 #include "wzd_structs.h"
 
@@ -98,6 +102,7 @@
 #include "wzd_misc.h"
 #include "wzd_messages.h"
 #include "wzd_mutex.h"
+#include "wzd_user.h"
 
 #endif /* WZD_USE_PCH */
 
@@ -407,14 +412,30 @@ static int _int_rename(const char * src, const char *dst)
   if (S_ISREG(s.mode)) {
     char buffer[32768];
     int fd_from, fd_to;
+    size_t sz;
+    int mode;
+
+#ifdef WIN32
+    /* since we can't move directories on windows, we can get here while
+     * trying to move files on the same device. So we just try to rename them
+     * before copy/remove, since renaming is much faster !
+     */
+    ret = rename(src,dst);
+    if (ret == 0) return 0;
+    mode = _O_BINARY;
+#endif
 
     /* FIXME XXX would it be wise to test functions return values ? :-P */
-    fd_from = open(src,O_RDONLY);
-    fd_to = open(dst,O_CREAT | O_WRONLY); /* XXX will overwite existing files */
-    while ( (ret=read(fd_from,buffer,sizeof(buffer))) > 0)
-    {
-      ret = write(fd_to,buffer,ret);
-    }
+    fd_from = open(src,O_RDONLY | mode);
+    if (fd_from < 0) return -1;
+    fd_to = open(dst,O_CREAT | O_WRONLY | mode,0666); /** \fixme this will overwite existing files */
+    if (fd_to < 0) { close(fd_from); return -1; }
+    do {
+      sz = read(fd_from,buffer,sizeof(buffer));
+      if (sz > 0) {
+        ret = write(fd_to,buffer,sz);
+      }
+    } while (sz > 0);
     close(fd_from);
     close(fd_to);
     unlink(src);
@@ -428,6 +449,24 @@ int safe_rename(const char *src, const char *dst)
 {
   int ret;
 
+#ifdef WIN32
+  /* Quote from MSDN documentation:
+   * However, you cannot use rename to move a directory. Directories can be renamed, but not moved.
+   * I'm just sick of all this crap
+   */
+  {
+    fs_filestat_t s;
+
+    if (fs_file_lstat(src,&s)) return -1;
+
+    /** \todo kill all users in this path, windows does not allow moving an opened file */
+    if (S_ISDIR(s.mode)) {
+      ret = _int_rename(src,dst);
+      return ret;
+    }
+  }
+#endif
+
   ret = rename(src,dst);
   if (ret == -1 && errno == EXDEV)
   {
@@ -486,74 +525,6 @@ time_t lget_file_ctime(int fd)
   return s.ctime;
 }
 
-int server_get_param(const char *name, void *buffer, unsigned int maxlen, wzd_param_t *param_list)
-{
-  wzd_param_t * param;
-
-  param = param_list;
-  while (param) {
-    if (strcmp(name,param->name)==0) {
-      if (maxlen < param->length) return 2;
-      memcpy(buffer,param->param,param->length);
-      memset((char*)buffer+param->length,0,maxlen-param->length);
-      return 0;
-    }
-    param = param->next_param;
-  }
-
-  return 1;
-}
-
-int server_set_param(const char *name, void *data, unsigned int length, wzd_param_t **plist)
-{
-  wzd_param_t * param, * insert_point;
-
-  if (!plist) return -1;
-  if (!name || !data || length==0) return -1;
-
-  param = malloc(sizeof(wzd_param_t));
-  /** \todo ensure param with same name is not already in list */
-  param->name = strdup(name);
-  param->param = malloc(length);
-  memcpy(param->param,data,length);
-  param->length = length;
-  param->next_param = NULL;
-
-  /* tail insertion */
-  if (*plist) {
-    insert_point = *plist;
-    while (insert_point->next_param) insert_point = insert_point->next_param;
-    insert_point->next_param = param;
-  } else {
-    *plist = param;
-  }
-
-  return 0;
-}
-
-void server_clear_param(wzd_param_t **plist)
-{
-  wzd_param_t * current, *next;
-
-  if (!plist) return;
-  current = *plist;
-
-  while (current) {
-    next = current->next_param;
-
-    if (current->name) free(current->name);
-    if (current->param) free(current->param);
-
-#ifdef DEBUG
-    memset(current,0,sizeof(wzd_param_t));
-#endif
-
-    free(current);
-
-    current = next;
-  }
-}
-
 /** \brief Checks server status
  */
 int server_diagnose(void)
@@ -814,8 +785,7 @@ wzd_mutex_unlock(mutex);
 
 void limiter_free(wzd_bw_limiter *l)
 {
-  if (l)
-    free(l);
+  free(l);
 }
 
 /** \brief Read file, replace cookies and send it to client
@@ -899,7 +869,12 @@ unsigned long right_text2word(const char * text)
 }
 
 
-/** dst can be composed of wildcards */
+/** \brief Compare strings \a src and \a dst
+ *
+ * dst can be composed of wildcards
+ *
+ * \return 1 if string matches, otherwise 0
+ */
 int my_str_compare(const char * src, const char *dst)
 {
   const char * ptr_src;
@@ -1013,7 +988,7 @@ void win_normalize(char * s, unsigned int length, unsigned int lower)
 
 
 /* \return 0 if ok, -1 if error, 1 if trying to kill myself */
-int kill_child(unsigned long pid, wzd_context_t * context)
+int kill_child_signal(unsigned long pid, wzd_context_t * context)
 {
   ListElmt * elmnt;
   wzd_context_t * loop_context;
@@ -1023,7 +998,7 @@ int kill_child(unsigned long pid, wzd_context_t * context)
 #endif
 
   /* preliminary check: i can't kill myself */
-  if (pid==context->pid_child) return 1;
+  if (context != NULL && pid==context->pid_child) return 1;
 
   /* checks that pid is really one of the users */
   for (elmnt=list_head(context_list); elmnt!=NULL; elmnt=list_next(elmnt))
@@ -1051,7 +1026,7 @@ int kill_child_new(unsigned long pid, wzd_context_t * context)
   int found=0;
 
   /* preliminary check: i can't kill myself */
-  if (pid==context->pid_child) return 1;
+  if (context != NULL && pid==context->pid_child) return 1;
 
   /* checks that pid is really one of the users */
   for (elmnt=list_head(context_list); elmnt!=NULL; elmnt=list_next(elmnt))
