@@ -73,10 +73,12 @@
 #include "wzd_events.h"
 #include "wzd_file.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_dir.h"
 #include "wzd_mod.h"
 #include "wzd_perm.h"
 #include "wzd_tls.h"
+#include "wzd_user.h"
 
 #include <libwzd-auth/wzd_tls.h> /* XXX test only */
 
@@ -90,6 +92,8 @@
 
 #endif /* WZD_USE_PCH */
 
+#include "wzd_version.h"
+
 extern int serverstop;
 extern time_t server_start;
 
@@ -121,8 +125,8 @@ int do_site_test(wzd_string_t *command, wzd_string_t *param, wzd_context_t * con
   }
 #endif
   /* prints some stats */
-  out_err(LEVEL_INFO,"# Connections: %d\n",mainConfig->stats.num_connections);
-  out_err(LEVEL_INFO,"# Childs     : %d\n",mainConfig->stats.num_childs);
+  out_err(LEVEL_INFO,"# Connections: %ld\n",mainConfig->stats.num_connections);
+  out_err(LEVEL_INFO,"# Childs     : %ld\n",mainConfig->stats.num_childs);
   ret = 0;
 
   out_err(LEVEL_INFO,"sizeof(wzd_context_t) = %d\n", sizeof(wzd_context_t));
@@ -175,10 +179,17 @@ int do_site_test(wzd_string_t *command, wzd_string_t *param, wzd_context_t * con
 #ifdef HAVE_GNUTLS
   ret = tls_dh_params_regenerate();
 #endif
-  
+
   out_err(LEVEL_CRITICAL,"Ret: %d\n",ret);
 
+/*
   ret = send_message_with_args(200,context,"TEST command ok");
+  */
+
+  reply_push(context,"TEST command ok");
+  reply_set_code(context,200);
+  reply_send(context);
+
   return 0;
 }
 
@@ -314,12 +325,15 @@ int do_site_backend(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   } /* close */
   if (strcasecmp(str_tochar(command),"init")==0) {
     str_deallocate(command);
-    ret = backend_init(str_tochar(name),0 /* max users */,0 /* max groups */);
+#if 0
+    ret = backend_init(str_tochar(name));
     if (ret) {
       ret = send_message_with_args(501,context,"Could not init backend");
     } else {
       ret = send_message_with_args(200,context,"Backend loaded successfully");
     }
+#endif
+    ret = send_message_with_args(501,context,"Not yet implemented");
     str_deallocate(name);
     return 0;
   } /* init */
@@ -621,7 +635,7 @@ int do_site_chpass(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     }
   }
   else {
-    if ( !(me->flags && strchr(me->flags,FLAG_SITEOP)) 
+    if ( !(me->flags && strchr(me->flags,FLAG_SITEOP))
         && me->uid != user->uid )
     {
       ret = send_message_with_args(501,context,"You can't change password for other users");
@@ -629,7 +643,7 @@ int do_site_chpass(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
       return 1;
     }
   }
-  if ( (user->flags && strchr(user->flags,FLAG_SITEOP)) 
+  if ( (user->flags && strchr(user->flags,FLAG_SITEOP))
       && me->uid != user->uid )
   {
     ret = send_message_with_args(501,context,"You can't change password for a siteop");
@@ -642,7 +656,7 @@ int do_site_chpass(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   str_deallocate(new_pass);
 
   /* commit to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,user->username,user,mod_type);
+  ret = backend_mod_user(mainConfig->backends->filename,user->uid,user,mod_type);
 
   if (ret)
     ret = send_message_with_args(501,context,"An error occurred during password change");
@@ -798,7 +812,7 @@ int do_site_invite(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
       path, /* ftp-absolute path */
       user->username,
       (group->groupname)?group->groupname:"No Group",
-      ircnick);
+      str_tochar(ircnick));
 
   ret = send_message_with_args(200,context,"SITE INVITE command ok");
   str_deallocate(ircnick);
@@ -828,15 +842,16 @@ int do_site_link(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     str_deallocate(command);
     return 1;
   }
- 
+
   /* convert file to absolute path, remember _setPerm wants ABSOLUTE paths ! */
-  if (checkpath(str_tochar(dirname),buffer_dir,context)) {
+  if (checkpath_new(str_tochar(dirname),buffer_dir,context)) {
     ret = send_message_with_args(501,context,"dirname is invalid");
     str_deallocate(command); str_deallocate(dirname);
     return 0;
   }
   str_deallocate(dirname);
-/*    buffer[strlen(buffer)-1] = '\0';*/ /* remove '/', appended by checkpath */
+  /* remove the last slash */
+  if(buffer_dir[strlen(buffer_dir)-1]=='/') buffer_dir[strlen(buffer_dir)-1]=0;
 
   if (strcasecmp(str_tochar(command),"CREATE")==0)
   {
@@ -847,13 +862,15 @@ int do_site_link(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
       str_deallocate(linkname);
       return 1;
     }
-    if (checkpath(str_tochar(linkname),buffer_link,context)) {
+    if ( (ret = checkpath_new(str_tochar(linkname),buffer_link,context) ) && ret != E_FILE_NOEXIST) { /* of course it returns no_exist */
       ret = send_message_with_args(501,context,"linkname is invalid");
       str_deallocate(command);
       str_deallocate(linkname);
       return 0;
     }
     str_deallocate(linkname);
+    if(buffer_link[strlen(buffer_link)-1]=='/') buffer_link[strlen(buffer_link)-1]=0;
+
     ret = symlink_create(buffer_dir, buffer_link);
   }
   else if (strcasecmp(str_tochar(command),"REMOVE")==0)
@@ -1167,10 +1184,10 @@ void do_site_print_file(const char *filename, wzd_user_t *user, wzd_group_t *gro
   }
   sz64 = wzd_cache_getsize(fp);
   if (sz64 > INT_MAX) {
-    out_log(LEVEL_HIGH,"%s:%d couldn't allocate" PRIu64 "bytes for file %s\n",__FILE__,__LINE__,sz64,filename);
-	wzd_cache_close(fp);
-	send_message_with_args(501,context,"Internal error (see log)");
-	return;
+    out_log(LEVEL_HIGH,"%s:%d couldn't allocate %" PRIu64 "bytes for file %s\n",__FILE__,__LINE__,sz64,filename);
+    wzd_cache_close(fp);
+    send_message_with_args(501,context,"Internal error (see log)");
+    return;
   }
   filesize = (unsigned int)sz64;
   file_buffer = malloc(filesize+1);
@@ -1179,7 +1196,7 @@ void do_site_print_file(const char *filename, wzd_user_t *user, wzd_group_t *gro
     out_err(LEVEL_HIGH,"Could not read file %s read %u instead of %u (%s:%d)\n",filename,size,filesize,__FILE__,__LINE__);
     free(file_buffer);
     wzd_cache_close(fp);
-	send_message_with_args(501,context,"Internal error (see log)");
+    send_message_with_args(501,context,"Internal error (see log)");
     return;
   }
   file_buffer[filesize]='\0';
@@ -1601,7 +1618,7 @@ int do_site_utime(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   filename = str_tok(command_line," \t\r\n");
   if (!filename) {
     do_site_help("utime",context);
-    return 1; 
+    return 1;
   }
   new_atime = str_tok(command_line," \t\r\n");
   if (!new_atime) {
@@ -1668,7 +1685,7 @@ int do_site_utime(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   }
   str_deallocate(filename);
 /*  buffer[strlen(buffer)-1] = '\0';*/ /* remove '/', appended by checkpath */
-  ret = _checkPerm(buffer,RIGHT_RNFR,user);  
+  ret = _checkPerm(buffer,RIGHT_RNFR,user);
   if (ret) {
     ret = send_message_with_args(501,context,"Access denied");
     return 1;
@@ -1691,7 +1708,7 @@ int do_site_vars(wzd_string_t *ignored, wzd_string_t * command_line, wzd_context
   command = str_tok(command_line," \t\r\n");
   if (!command) {
     do_site_help("vars",context);
-    return 1; 
+    return 1;
   }
   str_tolower(command);
 
@@ -1709,11 +1726,8 @@ int do_site_vars(wzd_string_t *ignored, wzd_string_t * command_line, wzd_context
 
     if (ret)
       send_message_with_args(200,context,"an error occurred");
-    else {
-      send_message_raw("200-\r\n",context);
-      send_message_raw(buffer,context);
-      send_message_raw("\r\n200 Command OK\r\n",context);
-    }
+    else
+      send_message_with_args(200,context,buffer);
 
     free(buffer);
     str_deallocate(varname);
@@ -1758,7 +1772,7 @@ int do_site_vars_group(wzd_string_t *ignored, wzd_string_t * command_line, wzd_c
   command = str_tok(command_line," \t\r\n");
   if (!command) {
     do_site_help("vars_group",context);
-    return 1; 
+    return 1;
   }
   str_tolower(command);
 
@@ -1766,14 +1780,14 @@ int do_site_vars_group(wzd_string_t *ignored, wzd_string_t * command_line, wzd_c
   if (!groupname) {
     do_site_help("vars_group",context);
     str_deallocate(command);
-    return 1; 
+    return 1;
   }
   group = GetGroupByName(str_tochar(groupname));
   str_deallocate(groupname);
   if ( !group ) {
     send_message_with_args(501,context,"group does not exist");
     str_deallocate(command);
-    return 1; 
+    return 1;
   }
 
   varname = str_tok(command_line," \t\r\n");
@@ -1790,11 +1804,8 @@ int do_site_vars_group(wzd_string_t *ignored, wzd_string_t * command_line, wzd_c
 
     if (ret)
       send_message_with_args(200,context,"an error occurred");
-    else {
-      send_message_raw("200-\r\n",context);
-      send_message_raw(buffer,context);
-      send_message_raw("\r\n200 Command OK\r\n",context);
-    }
+    else
+      send_message_with_args(200,context,buffer);
 
     free(buffer);
     str_deallocate(varname);
@@ -1838,7 +1849,7 @@ int do_site_vars_user(wzd_string_t *ignored, wzd_string_t * command_line, wzd_co
   command = str_tok(command_line," \t\r\n");
   if (!command) {
     do_site_help("vars_user",context);
-    return 1; 
+    return 1;
   }
   str_tolower(command);
 
@@ -1846,14 +1857,14 @@ int do_site_vars_user(wzd_string_t *ignored, wzd_string_t * command_line, wzd_co
   if (!username) {
     do_site_help("vars_user",context);
     str_deallocate(command);
-    return 1; 
+    return 1;
   }
   user = GetUserByName(str_tochar(username));
   str_deallocate(username);
   if ( !user ) {
     send_message_with_args(501,context,"user does not exist");
     str_deallocate(command);
-    return 1; 
+    return 1;
   }
 
   varname = str_tok(command_line," \t\r\n");
@@ -1870,11 +1881,8 @@ int do_site_vars_user(wzd_string_t *ignored, wzd_string_t * command_line, wzd_co
 
     if (ret)
       send_message_with_args(200,context,"an error occurred");
-    else {
-      send_message_raw("200-\r\n",context);
-      send_message_raw(buffer,context);
-      send_message_raw("\r\n200 Command OK\r\n",context);
-    }
+    else
+      send_message_with_args(200,context,buffer);
 
     free(buffer);
     str_deallocate(varname);
@@ -1912,7 +1920,7 @@ int do_site_version(wzd_string_t * ignored, wzd_string_t * command_line, wzd_con
 {
   char str[256];
   snprintf(str,256,"%s build %s (%s)",
-      WZD_VERSION_STR,WZD_BUILD_NUM,WZD_BUILD_OPTS);
+      WZD_VERSION_LONG,WZD_BUILD_NUM,WZD_BUILD_OPTS);
   send_message_with_args(200,context,str);
   return 0;
 }
@@ -1973,7 +1981,7 @@ int do_site_vfsadd(wzd_string_t * ignored, wzd_string_t * command_line, wzd_cont
     return 1;
   }
   *dstptr = '\0';
- 
+
   target = NULL;
   ptr++;
 
@@ -1982,7 +1990,7 @@ int do_site_vfsadd(wzd_string_t * ignored, wzd_string_t * command_line, wzd_cont
     if (*ptr)
       target = (char*)ptr;
   }
- 
+
   if( target )
     ret = vfs_add_restricted( &mainConfig->vfs, vpath, ppath, target );
   else
@@ -1997,7 +2005,7 @@ int do_site_vfsadd(wzd_string_t * ignored, wzd_string_t * command_line, wzd_cont
     send_message_with_args(501,context,tmp);
   }  else
     send_message_with_args(200,context,"VFSADD command ok");
-  
+
   free(vpath); free(ppath);
 
   return 0;
@@ -2103,7 +2111,7 @@ int do_site_wipe(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
   firstarg = str_read_token(command_line);
   if (!firstarg) {
     do_site_help("wipe",context);
-    return 1;
+    return E_PARAM_NULL;
   }
   /* check if wiping is recursive */
   if ( strcasecmp(str_tochar(firstarg),"-r")==0 ) {
@@ -2112,7 +2120,7 @@ int do_site_wipe(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     filename = str_read_token(command_line);
     if( !filename) {
       do_site_help("wipe",context);
-      return 1;
+      return E_PARAM_INVALID;
     }
   }
   else
@@ -2121,16 +2129,43 @@ int do_site_wipe(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
   do
   {
     /* convert file to absolute path, remember _setPerm wants ABSOLUTE paths ! */
-    if (!checkpath(str_tochar(filename),buffer,context))
+    if (checkpath_new(str_tochar(filename),buffer,context) != 0)
     {
-      /* wipe file | if_recursive dir/file */
-      ret = do_internal_wipe(buffer,context);
-      if (ret) {
-        ret = send_message_with_args(501,context,"WIPE failed");
-        str_deallocate(filename);
-        return 1;
-      }
+      ret = send_message_with_args(501,context,"File does not exist");
+      str_deallocate(filename);
+      return E_FILE_NOEXIST;
     }
+
+    {
+      wzd_string_t * event_args = str_allocate();
+      wzd_user_t * user = GetUserByID(context->userid);
+      str_sprintf(event_args,"%s %s",user->username,str_tochar(filename));
+      ret = event_send(mainConfig->event_mgr, EVENT_PREWIPE, 0, event_args, context);
+      str_deallocate(event_args);
+    }
+    if (ret != EVENT_OK && ret != EVENT_BREAK) {
+      out_log(LEVEL_NORMAL, "Wipe denied by hook (returned %d)\n", ret);
+      ret = send_message_with_args(501,context,"Wipe denied");
+      str_deallocate(filename);
+      return E_COMMAND_FAILED;
+    }
+
+    /* wipe file | if_recursive dir/file */
+    ret = do_internal_wipe(buffer,context);
+    if (ret) {
+      ret = send_message_with_args(501,context,"WIPE failed");
+      str_deallocate(filename);
+      return E_COMMAND_FAILED;
+    }
+
+    {
+      wzd_string_t * event_args = str_allocate();
+      wzd_user_t * user = GetUserByID(context->userid);
+      str_sprintf(event_args,"%s %s",user->username,str_tochar(filename));
+      event_send(mainConfig->event_mgr, EVENT_WIPE, 200, event_args, context);
+      str_deallocate(event_args);
+    }
+
     str_deallocate(filename);
   }
   while ( (filename = str_read_token(command_line)) );
