@@ -54,11 +54,13 @@
 
 #include "wzd_configfile.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_log.h"
 #include "wzd_misc.h"
 #include "wzd_messages.h"
 #include "wzd_site.h"
 #include "wzd_site_group.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -82,11 +84,9 @@ void do_site_help_grpadd(wzd_context_t * context)
 int do_site_grpadd(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
   wzd_string_t *groupname;
-  char *homedir;
-  int ret;
+  int err;
   wzd_user_t *me;
-  wzd_group_t *mygroup=NULL, newgroup;
-  int i;
+  wzd_group_t *newgroup;
   short is_gadmin;
 
   me = GetUserByID(context->userid);
@@ -97,63 +97,44 @@ int do_site_grpadd(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     do_site_help_grpadd(context);
     return 0;
   }
-  /* TODO read backend */
-
-  /* check if group already exists */
-  if ( GetGroupByName(str_tochar(groupname)) ) {
-    ret = send_message_with_args(501,context,"Group already exists");
-    str_deallocate(groupname);
-    return 0;
-  }
-
-  /* backend will do that */
-  /* check groups limit */
 
   /* Gadmin ? */
   if (is_gadmin)
   {
-    ret = send_message_with_args(501,context,"Gadmins can't add groups !");
+    err = send_message_with_args(501,context,"Gadmins can't add groups !");
     str_deallocate(groupname);
     return 0;
   }
-  mygroup = GetGroupByID(me->groups[0]);
-  if (mygroup) {
-    homedir = mygroup->defaultpath;
-  } else {
-    homedir = me->rootpath;
-  }
-  /* check if homedir exist */
-  {
-    fs_filestat_t s;
-    if (fs_file_stat(homedir,&s) || !S_ISDIR(s.mode)) {
-      ret = send_message_with_args(501,context,"Homedir does not exist");
-      str_deallocate(groupname);
-      return 0;
-    }
-  }
 
-  /* create new group */
-  strncpy(newgroup.groupname,str_tochar(groupname),sizeof(newgroup.groupname));
-  strncpy(newgroup.defaultpath,homedir,WZD_MAX_PATH);
-  newgroup.groupperms = 0;
-  newgroup.max_idle_time = 0;
-  newgroup.max_dl_speed = 0;
-  newgroup.max_ul_speed = 0;
-  newgroup.ratio = 0;
-  newgroup.num_logins = 0;
-  newgroup.tagline[0] = '\0';
-  for (i=0; i<HARD_IP_PER_GROUP; i++)
-    newgroup.ip_allowed[i][0]='\0';
+  newgroup = group_create(str_tochar(groupname),context,mainConfig,&err);
+  if (newgroup == NULL) {
+    switch (err) {
+      case E_PARAM_NULL:
+      case E_PARAM_BIG:
+        err = send_message_with_args(501,context,"Invalid name or parameter");
+        break;
+      case E_PARAM_EXIST:
+        err = send_message_with_args(501,context,"A group already exist with this name");
+        break;
+      default:
+        err = send_message_with_args(501,context,"Error while adding group");
+        break;
+    }
+    str_deallocate(groupname);
+    return 0;
+  }
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),&newgroup,_GROUP_ALL);
+  err = backend_mod_group(mainConfig->backends->filename,0,newgroup,_GROUP_CREATE);
 
   str_deallocate(groupname);
 
-  if (ret) {
-    ret = send_message_with_args(501,context,"Problem adding group");
+  if (err) {
+    err = send_message_with_args(501,context,"Problem adding group");
+    group_free(newgroup);
   } else {
-    ret = send_message_with_args(200,context,"Group added");
+    err = send_message_with_args(200,context,"Group added");
+    /* do not free group, it is stored in registry */
   }
   return 0;
 }
@@ -199,18 +180,18 @@ int do_site_grpdel(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     str_deallocate(groupname);
     return 0;
   }
+  str_deallocate(groupname);
 
   /* GAdmin ? */
   if (is_gadmin)
   {
     ret = send_message_with_args(501,context,"Gadmin can't delete groups");
-    str_deallocate(groupname);
     return 0;
   }
 
   send_message_raw("200-\r\n",context);
   /* iterate through user list and delete all references to group */
-  uid_list = (int*)backend_get_user(-2);
+  uid_list = (int*)backend_get_user(GET_USER_LIST);
   if (uid_list) {
     for (i=0; uid_list[i] >= 0; i++)
     {
@@ -220,7 +201,7 @@ int do_site_grpdel(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
       {
         /* warn for users with no groups and / or primary group
          * changed
-         */ 
+         */
         if (user->groups[0] == gid) {
           snprintf(buffer,sizeof(buffer),"200-WARNING %s main group is changed !\r\n",user->username);
           send_message_raw(buffer,context);
@@ -237,8 +218,7 @@ int do_site_grpdel(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   /* TODO XXX FIXME delete users belonging only to this group ? */
 
   /* commit changes to backend */
-  backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),NULL,_GROUP_ALL);
-  str_deallocate(groupname);
+  backend_mod_group(mainConfig->backends->filename,gid,NULL,_GROUP_ALL);
 
   ret = send_message_raw("200 Group deleted\r\n",context);
   return 0;
@@ -257,7 +237,7 @@ int do_site_grpren(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   wzd_string_t * groupname, *newgroupname;
   int ret;
   wzd_user_t *me;
-  wzd_group_t group, *oldgroup;
+  wzd_group_t *oldgroup;
 /*  unsigned int ratio;*/
   short is_gadmin;
 
@@ -284,7 +264,6 @@ int do_site_grpren(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     str_deallocate(newgroupname);
     return 0;
   }
-  memcpy(&group,oldgroup,sizeof(wzd_group_t));
 
   /* check if group exists */
   if ( (GetGroupByName(str_tochar(newgroupname))) ) {
@@ -300,11 +279,11 @@ int do_site_grpren(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     return 0;
   }
 
-  strncpy(group.groupname,str_tochar(newgroupname),sizeof(group.groupname));
+  strncpy(oldgroup->groupname,str_tochar(newgroupname),HARD_GROUPNAME_LENGTH-1);
   str_deallocate(newgroupname);
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.filename,oldgroup->groupname,&group,_GROUP_GROUPNAME);
+  ret = backend_mod_group(mainConfig->backends->filename,oldgroup->gid,oldgroup,_GROUP_GROUPNAME);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -396,7 +375,6 @@ int do_site_grpaddip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   int ret;
   wzd_user_t * me;
   wzd_group_t * group;
-  int i;
   short is_gadmin;
 
   me = GetUserByID(context->userid);
@@ -429,47 +407,27 @@ int do_site_grpaddip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
     return 0;
   }
 
-  /* check if ip is already present or included in list, or if it shadows one present */
-  for (i=0; i<HARD_IP_PER_GROUP; i++)
-  {
-    if (group->ip_allowed[i][0]=='\0') continue;
-    if (my_str_compare(str_tochar(ip),group->ip_allowed[i])==1) { /* ip is already included in list */
-      ret = send_message_with_args(501,context,"ip is already included in list");
-      str_deallocate(ip);
-      return 0;
-    }
-    if (my_str_compare(group->ip_allowed[i],str_tochar(ip))==1) { /* ip will shadow one ore more ip in list */
-      ret = send_message_with_args(501,context,"ip will shadow some ip in list, remove them before");
-      str_deallocate(ip);
-      return 0;
-    }
-  }
-
-  /* update group */
-  for (i=0; i<HARD_IP_PER_GROUP; i++)
-    if (group->ip_allowed[i][0]=='\0') break;
-
-  /* no more slots ? */
-  if (i==HARD_IP_PER_GROUP) {
-    ret = send_message_with_args(501,context,"No more slots available - either recompile with more slots, or use them more cleverly !");
+  ret = ip_inlist(group->ip_list, str_tochar(ip));
+  if (ret) {
+    ret = send_message_with_args(501,context,"ip is already included in list");
     str_deallocate(ip);
     return 0;
   }
-  /* TODO check ip validity */
-  strncpy(group->ip_allowed[i],str_tochar(ip),MAX_IP_LENGTH-1);
+
+  ret = ip_add_check(&group->ip_list, str_tochar(ip), 1 /* is_allowed */);
+  str_deallocate(ip);
 
   /* commit to backend */
-  backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
+  backend_mod_group(mainConfig->backends->filename,group->gid,group,_GROUP_IP);
 
   ret = send_message_with_args(200,context,"Group ip added");
-  str_deallocate(ip);
   return 0;
 }
 
 void do_site_help_grpdelip(wzd_context_t * context)
 {
   send_message_raw("501-Usage: site grpdelip <group> <ip>\r\n",context);
-  send_message_raw("501  or: site grpdelip <grp> <slot_number> (get it with site ginfo <group>)\r\n",context);
+  send_message_raw("501  or: site grpdelip <grp> <slot_number> (get it with site gsinfo <group>)\r\n",context);
 }
 
 /** site grpdelip: removes ip from group
@@ -485,7 +443,6 @@ int do_site_grpdelip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   int ret;
   wzd_user_t * me;
   wzd_group_t * group;
-  int i;
   unsigned long ul;
   short is_gadmin;
 
@@ -522,40 +479,47 @@ int do_site_grpdelip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   /* try to take argument as a slot number */
   ul = strtoul(str_tochar(ip),&ptr,0);
   if (*ptr=='\0') {
-    if (ul <= 0 || ul > HARD_IP_PER_GROUP) {
-      ret = send_message_with_args(501,context,"Invalid ip slot number");
-      str_deallocate(ip);
-      return 0;
-    }
+    unsigned int i;
+    struct wzd_ip_list_t * current_ip;
+
     str_deallocate(ip);
     ul--; /* to index slot number from 1 */
-    if (group->ip_allowed[ul][0] == '\0') {
-      ret = send_message_with_args(501,context,"Slot is already empty");
+    current_ip = group->ip_list;
+    for (i=0; i<ul && current_ip != NULL; i++) {
+      current_ip = current_ip->next_ip;
+    }
+    if (current_ip == NULL) {
+      char buffer[256];
+      snprintf(buffer,256,"IP slot %lu not found",ul+1);
+      ret = send_message_with_args(501,context,buffer);
       return 0;
     }
-    group->ip_allowed[ul][0] = '\0';
-    backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
+    ret = ip_remove(&group->ip_list,current_ip->regexp);
+    if (ret != 0) {
+      char buffer[256];
+      snprintf(buffer,256,"error removing IP slot %lu",ul+1);
+      ret = send_message_with_args(501,context,buffer);
+      return 0;
+    }
+    backend_mod_group(mainConfig->backends->filename,group->gid,group,_GROUP_IP);
     ret = send_message_with_args(200,context,"Group ip removed");
     return 0;
   } /* if (*ptr=='\0') */
 
-  /* try to find ip in list */
-  for (i=0; i<HARD_IP_PER_GROUP; i++)
-  {
-    if (group->ip_allowed[i][0]=='\0') continue;
-    if (strcmp(str_tochar(ip),group->ip_allowed[i])==0) {
-      group->ip_allowed[i][0] = '\0';
-      /* commit to backend */
-      /* FIXME backend name hardcoded */
-      backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
-      ret = send_message_with_args(200,context,"Group ip removed");
-      str_deallocate(ip);
-      return 0;
-    }
+  ret = ip_remove(&group->ip_list,str_tochar(ip));
+  if (ret != 0) {
+    char buffer[256];
+    snprintf(buffer,256,"IP %s not found",str_tochar(ip));
+    ret = send_message_with_args(501,context,buffer);
+    str_deallocate(ip);
+    return 0;
   }
   str_deallocate(ip);
 
-  ret = send_message_with_args(501,context,"IP not found");
+  /* commit to backend */
+  backend_mod_group(mainConfig->backends->filename,group->gid,group,_GROUP_IP);
+  ret = send_message_with_args(200,context,"Group ip removed");
+
   return 0;
 }
 
@@ -616,7 +580,7 @@ int do_site_grpratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   group->ratio = ratio;
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_RATIO);
+  ret = backend_mod_group(mainConfig->backends->filename,group->gid,group,_GROUP_RATIO);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -661,7 +625,7 @@ int do_site_grpkill(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
       user = GetUserByID(loop_context->userid);
       if (strcmp(me->username,user->username) && is_user_in_group(user,group->gid)) {
         found=1;
-        kill_child(loop_context->pid_child,context);
+        kill_child_new(loop_context->pid_child,context);
       }
     }
   }
@@ -702,6 +666,7 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   int ret;
   wzd_group_t * group;
   wzd_user_t * me;
+  gid_t gid;
 
   me = GetUserByID(context->userid);
 
@@ -733,6 +698,8 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
     str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
     return 0;
   }
+  str_deallocate(groupname);
+  gid = group->gid;
 
   /* find modification type */
   mod_type = _GROUP_NOTHING;
@@ -757,7 +724,7 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
       fs_filestat_t s;
       if (fs_file_stat(str_tochar(value),&s) || !S_ISDIR(s.mode)) {
         ret = send_message_with_args(501,context,"Homedir does not exist");
-        str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
+        str_deallocate(field); str_deallocate(value);
         return 0;
       }
     }
@@ -796,22 +763,22 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
       if ((!me->flags || !strchr(me->flags,FLAG_SITEOP)) && ul==0) {
         /* wants a leech access for group, but is not siteop */
         ret = send_message_with_args(501,context,"Only siteops can do that");
-        str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
+        str_deallocate(field); str_deallocate(value);
         return 0;
       }
       mod_type = _GROUP_RATIO; group->ratio = ul;
     }
   }
   else {
-    str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
+    str_deallocate(field); str_deallocate(value);
     ret = send_message_with_args(501,context,"syntax error, unknow field");
     return 0;
   }
 
   /* commit to backend */
-  ret = backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),group,mod_type);
+  ret = backend_mod_group(mainConfig->backends->filename,gid,group,mod_type);
 
-  str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
+  str_deallocate(field); str_deallocate(value);
 
   if (ret)
     ret = send_message_with_args(501,context,"Problem occured when committing");
