@@ -54,12 +54,15 @@
 
 #include "wzd_structs.h"
 
+#include "wzd_crontab.h"
 #include "wzd_fs.h"
+#include "wzd_group.h"
 #include "wzd_log.h"
 #include "wzd_misc.h"
 #include "wzd_messages.h"
 #include "wzd_site_user.h"
 #include "wzd_vfs.h"
+#include "wzd_user.h"
 
 #include "wzd_debug.h"
 
@@ -70,6 +73,8 @@
 static int _user_changeflags(wzd_user_t * user, const char *newflags);
 static void _flags_simplify(char *flags, size_t length);
 
+static int _kick_and_purge(void);
+
 
 
 void do_site_help_adduser(wzd_context_t * context)
@@ -84,14 +89,12 @@ void do_site_help_adduser(wzd_context_t * context)
 int do_site_adduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
   wzd_string_t * username, *password, *groupname, *ip=NULL;
-  const char * homedir;
   int ret;
-  wzd_user_t user, *me;
+  wzd_user_t *newuser, *me;
   wzd_group_t * group=NULL;
-  int uid;
   int i;
-  unsigned int ratio = 3; /* TODO XXX FIXME default ratio value hardcoded */
   short is_gadmin;
+  int err;
 
   me = GetUserByID(context->userid);
   is_gadmin = (me->flags && strchr(me->flags,FLAG_GADMIN)) ? 1 : 0;
@@ -108,22 +111,18 @@ int do_site_adduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
     return 0;
   }
 
-  /* check users limit -> backend will do that !! */
-
   groupname = str_tok(command_line," \t\r\n");
   group = GetGroupByName(str_tochar(groupname));
 
-  if (!group) ip = groupname; /* it is not a valid group, assume it is an ip */
-  else str_deallocate(groupname);
+  if (group == NULL) {
+    ret = send_message_with_args(501,context,"Invalid group name");
+    str_deallocate(groupname);
+    return 0;
+  }
+  else { str_deallocate(groupname); groupname = NULL; }
 
   groupname = NULL;
 
-  /* check if user already exists */
-  if ( !backend_find_user(str_tochar(username),&user,&uid) ) {
-    ret = send_message_with_args(501,context,"User already exists");
-    str_deallocate(username); str_deallocate(password); str_deallocate(ip);
-    return 0;
-  }
   /* find user group or take current user */
   if (!group) {
     if (me && me->group_num>0) {
@@ -137,7 +136,7 @@ int do_site_adduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
     if (is_gadmin)
     {
       /* GAdmins cannot add user to different group */
-      if (me->group_num==0 || me->groups[0]!=GetGroupIDByName(str_tochar(groupname)))
+      if (me->group_num==0 || me->groups[0]!=group->gid)
       {
         ret = send_message_with_args(501,context,"You are not allowed to add users to this group");
         str_deallocate(username); str_deallocate(password); str_deallocate(ip);
@@ -154,73 +153,45 @@ int do_site_adduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
       return 0;
     }
   }
-  if (group) {
-    homedir = group->defaultpath;
-    groupname = STR(group->groupname);
-    ratio = group->ratio;
-  } else {
-    /* XXX FIXME we should abort here */
-    ret = send_message_with_args(501,context,"I can't find a default_home in your groups - contact the sysadmin");
+
+  newuser = user_create(str_tochar(username),str_tochar(password),(group)?group->groupname:NULL,context,mainConfig,&err);
+  if (newuser == NULL) {
+    switch(err) {
+      case E_PARAM_NULL:
+      case E_PARAM_BIG:
+        err = send_message_with_args(501,context,"Invalid name or parameter");
+        break;
+      case E_PARAM_EXIST:
+        err = send_message_with_args(501,context,"A user already exist with this name");
+        break;
+      default:
+        err = send_message_with_args(501,context,"Error while adding user");
+        break;
+    };
     str_deallocate(username); str_deallocate(password); str_deallocate(ip);
     return 0;
   }
-  /* check if homedir exist */
-  {
-    fs_filestat_t s;
-    if (fs_file_stat(homedir,&s) || !S_ISDIR(s.mode)) {
-      ret = send_message_with_args(501,context,"Homedir does not exist");
-      str_deallocate(username); str_deallocate(password); str_deallocate(ip);
-      return 0;
-    }
-  }
-
-  /* create new user */
-  strncpy(user.username,str_tochar(username),HARD_USERNAME_LENGTH);
-  strncpy(user.userpass,str_tochar(password),MAX_PASS_LENGTH);
-  strncpy(user.rootpath,homedir,WZD_MAX_PATH);
-  user.tagline[0]='\0';
-  user.uid=-1; /* will be changed by backend */
-  user.group_num=0;
-  if (groupname) {
-    user.groups[0] = GetGroupIDByName(str_tochar(groupname));
-    if (user.groups[0]) user.group_num=1;
-  }
-  user.max_idle_time=0;
-  user.userperms=0xffffffff;
-  user.flags[0]='\0';
-  user.max_ul_speed=0;
-  user.max_dl_speed=0;
-  user.num_logins=0;
-  for (i=0; i<HARD_IP_PER_USER; i++)
-    user.ip_allowed[i][0]='\0';
-  user.stats.bytes_ul_total=0;
-  user.stats.bytes_dl_total=0;
-  user.stats.files_ul_total=0;
-  user.stats.files_dl_total=0;
-  user.credits = 0;
-  user.ratio = ratio;
-  user.user_slots=0;
-  user.leech_slots=0;
-  user.last_login = 0;
 
   i = 0;
   if (ip) {
-    wzd_strncpy(user.ip_allowed[i++],str_tochar(ip),MAX_IP_LENGTH);
+    ret = ip_add_check(&newuser->ip_list, str_tochar(ip), 1 /* is_allowed */);
     str_deallocate(ip);
   };
   while ( (ip = str_tok(command_line," \t")) ) {
-    wzd_strncpy(user.ip_allowed[i++],str_tochar(ip),MAX_IP_LENGTH);
+    ret = ip_add_check(&newuser->ip_list, str_tochar(ip), 1 /* is_allowed */);
     str_deallocate(ip);
   }
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_ALL);
+  ret = backend_mod_user(mainConfig->backends->filename,0,newuser,_USER_CREATE);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem adding user");
+    user_free(newuser);
   } else {
     if (is_gadmin) me->user_slots--; /* decrement user slots counter */
     ret = send_message_with_args(200,context,"User added");
+    /* do not free user, it is stored in registry */
   }
   str_deallocate(username); str_deallocate(password); str_deallocate(ip);
   return 0;
@@ -285,7 +256,7 @@ int do_site_deluser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   user->flags[length+1] = '\0';
 
   /* commit changes to backend */
-  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_FLAGS);
+  backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_FLAGS);
 
   ret = send_message_with_args(200,context,"User deleted");
   return 0;
@@ -305,8 +276,7 @@ int do_site_readduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   char *ptr;
   wzd_string_t * username;
   int ret;
-  wzd_user_t user, *me;
-  int uid;
+  wzd_user_t *user, *me;
   int length;
   short is_gadmin;
 
@@ -319,25 +289,25 @@ int do_site_readduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
     return 0;
   }
 
-  /* check if user already exists */
-  if ( backend_find_user(str_tochar(username),&user,&uid) ) {
+  /* check if user exists */
+  user = GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( !user ) {
     ret = send_message_with_args(501,context,"User does not exist");
-    str_deallocate(username);
     return 0;
   }
-  str_deallocate(username);
 
   /* GAdmin ? */
   if (is_gadmin)
   {
-    if (me->group_num==0 || user.group_num==0 || me->groups[0]!=user.groups[0]) {
+  if (me->group_num==0 || user->group_num==0 || me->groups[0]!=user->groups[0]) {
       ret = send_message_with_args(501,context,"You can't change this user");
       return 0;
     }
   }
 
   /* unmark user as deleted */
-  if ( (ptr = strchr(user.flags,FLAG_DELETED)) == NULL ) {
+  if ( (ptr = strchr(user->flags,FLAG_DELETED)) == NULL ) {
     ret = send_message_with_args(501,context,"User is not marked as deleted");
     return 0;
   }
@@ -350,7 +320,7 @@ int do_site_readduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   }
 
   /* commit changes to backend */
-  backend_mod_user(mainConfig->backend.filename,user.username,&user,_USER_FLAGS);
+  backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_FLAGS);
 
   ret = send_message_with_args(200,context,"User undeleted");
   return 0;
@@ -364,8 +334,7 @@ int do_site_purgeuser(wzd_string_t *command_line, wzd_string_t *param, wzd_conte
 {
   wzd_string_t * username;
   int ret;
-  wzd_user_t user, * me;
-  int uid;
+  wzd_user_t * user, * me;
   short is_gadmin;
   const char *ptr;
 
@@ -374,68 +343,62 @@ int do_site_purgeuser(wzd_string_t *command_line, wzd_string_t *param, wzd_conte
 
   username = str_tok(param," \t\r\n");
 
-  /* TODO read backend */
-
   if (username) { /* case 1: name was given */
-    /* check if user already exists */
-    if ( backend_find_user(str_tochar(username),&user,&uid) ) {
+    /* check if user exists */
+    user = GetUserByName(str_tochar(username));
+    str_deallocate(username);
+    if ( !user ) {
       ret = send_message_with_args(501,context,"User does not exist");
-      str_deallocate(username);
       return 0;
     }
 
-    /* unmark user as deleted */
-    if ( (ptr = strchr(user.flags,FLAG_DELETED)) == NULL ) {
+    if ( (ptr = strchr(user->flags,FLAG_DELETED)) == NULL ) {
       ret = send_message_with_args(501,context,"User is not marked as deleted");
-      str_deallocate(username);
+      return 0;
+    }
+
+    if ( user->uid == context->userid ) {
+      ret = send_message_with_args(501,context,"Can't purge myself while logged. Use another user or try site purge without argument");
       return 0;
     }
 
     /* gadmin ? */
     if (is_gadmin)
     {
-      if (me->group_num==0 || user.group_num==0 || me->groups[0]!=user.groups[0]) {
+      if (me->group_num==0 || user->group_num==0 || me->groups[0]!=user->groups[0]) {
         ret = send_message_with_args(501,context,"You can't purge this user (GAdmin limits)");
-        str_deallocate(username);
         return 0;
       }
     }
 
-    /* commit changes to backend */
-    backend_mod_user(mainConfig->backend.filename,str_tochar(username),NULL,_USER_ALL);
-    str_deallocate(username);
-  } else { /* if (username) */
-    /* TODO iterate users and purge those marked as deleted */
-    unsigned int i;
-    wzd_user_t * user;
-    int * uid_list;
-    uid_list = (int*)backend_get_user(-2);
-
-    if (uid_list)
+    /* check if user is connected, and if yes, kick him */
     {
-      for (i=0; uid_list[i] >= 0; i++)
-      {
-        user = GetUserByID(uid_list[i]);
-        if (user && user->flags && strchr(user->flags,FLAG_DELETED))
-        {
-          /* gadmin ? */
-          if (is_gadmin)
-          {
-            if (me->group_num==0 || user->group_num==0 || me->groups[0]!=user->groups[0]) {
-              continue;
-            }
+      ListElmt * elmnt;
+      wzd_context_t * loop_context;
+      for (elmnt=list_head(context_list); elmnt; elmnt=list_next(elmnt)) {
+        loop_context = list_data(elmnt);
+        if (loop_context && loop_context->magic == CONTEXT_MAGIC) {
+          if (loop_context->userid == user->uid) {
+            kill_child_signal(loop_context->pid_child,context);
           }
-          /* commit changes to backend */
-          backend_mod_user(mainConfig->backend.filename,user->username,NULL,_USER_ALL);
         }
       }
-      wzd_free (uid_list);
-    } /* if (uid_list) */
-    ret = send_message_with_args(200,context,"All deleted users have been purged");
+    }
+
+
+    /* commit changes to backend */
+    backend_mod_user(mainConfig->backends->filename,user->uid,NULL,_USER_ALL);
+
+    ret = send_message_with_args(200,context,"User purged");
+    return 0;
+  } else { /* purge all users */
+    ret = cronjob_add_once(&mainConfig->crontab,_kick_and_purge,"fn:_kick_and_purge",time(NULL)+3);
+
+    ret = send_message_with_args(200,context,"All deleted users will be purged");
     return 0;
   } /* if (username) */
 
-  ret = send_message_with_args(200,context,"User purged");
+  ret = send_message_with_args(501,context,"Something wrong happened");
   return 0;
 }
 
@@ -453,7 +416,6 @@ int do_site_addip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   wzd_string_t * username, *ip;
   int ret;
   wzd_user_t *user, *me;
-  int i;
   short is_gadmin;
 
   me = GetUserByID(context->userid);
@@ -491,42 +453,21 @@ int do_site_addip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
 
   do {
 
-    /* check if ip is already present or included in list, or if it shadows one present */
-    for (i=0; i<HARD_IP_PER_USER; i++)
-    {
-      if (user->ip_allowed[i][0]=='\0') continue;
-      if (my_str_compare(str_tochar(ip),user->ip_allowed[i])==1) { /* ip is already included in list */
-        ret = send_message_with_args(501,context,"ip is already included in list");
-        str_deallocate(ip);
-        return 0;
-      }
-      if (my_str_compare(user->ip_allowed[i],str_tochar(ip))==1) { /* ip will shadow one ore more ip in list */
-        ret = send_message_with_args(501,context,"ip will shadow some ip in list, remove them before");
-        str_deallocate(ip);
-        return 0;
-      }
-    }
-
-    /* update user */
-    for (i=0; i<HARD_IP_PER_USER; i++)
-      if (user->ip_allowed[i][0]=='\0') break;
-
-    /* no more slots ? */
-    if (i==HARD_IP_PER_USER) {
-      ret = send_message_with_args(501,context,"No more slots available - either recompile with more slots, or use them more cleverly !");
+    ret = ip_inlist(user->ip_list, str_tochar(ip));
+    if (ret) {
+      ret = send_message_with_args(501,context,"ip is already included in list");
       str_deallocate(ip);
       return 0;
     }
-    /* TODO check ip validity */
-    strncpy(user->ip_allowed[i],str_tochar(ip),MAX_IP_LENGTH-1);
 
+    ret = ip_add_check(&user->ip_list, str_tochar(ip), 1 /* is_allowed */);
     str_deallocate(ip);
 
     ip = str_tok(command_line," \t\r\n");
   } while (ip);
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IP);
+  backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_IP);
 
   ret = send_message_with_args(200,context,"User ip(s) added");
   return 0;
@@ -550,10 +491,8 @@ int do_site_delip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   wzd_string_t * username, *ip;
   int ret;
   wzd_user_t *user, *me;
-  int i;
   unsigned long ul;
   short is_gadmin;
-  int found;
 
   me = GetUserByID(context->userid);
   is_gadmin = (me->flags && strchr(me->flags,FLAG_GADMIN)) ? 1 : 0;
@@ -591,32 +530,31 @@ int do_site_delip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
     /* try to take argument as a slot number */
     ul = strtoul(str_tochar(ip),&ptr_ul,0);
     if (*ptr_ul=='\0') {
-      if (ul <= 0 || ul > HARD_IP_PER_USER) {
-        ret = send_message_with_args(501,context,"Invalid ip slot number");
-        str_deallocate(ip);
-        return 0;
-      }
+      unsigned int i;
+      struct wzd_ip_list_t * current_ip;
+
       str_deallocate(ip);
-      ul--; /* to index slot number from 1 */
-      if (user->ip_allowed[ul][0] == '\0') {
-        ret = send_message_with_args(501,context,"Slot is already empty");
+      current_ip = user->ip_list;
+      for (i=1; i<ul && current_ip != NULL; i++) {
+        current_ip = current_ip->next_ip;
+      }
+      if (current_ip == NULL) {
+        char buffer[256];
+        snprintf(buffer,256,"IP slot %lu not found",ul);
+        ret = send_message_with_args(501,context,buffer);
+        return 0;
+      }
+      ret = ip_remove(&user->ip_list,current_ip->regexp);
+      if (ret != 0) {
+        char buffer[256];
+        snprintf(buffer,256,"error removing IP slot %lu",ul);
+        ret = send_message_with_args(501,context,buffer);
         return 0;
       }
-      user->ip_allowed[ul][0] = '\0';
     } else { /* if (*ptr=='\0') */
 
-      /* try to find ip in list */
-      found = 0;
-      for (i=0; i<HARD_IP_PER_USER; i++)
-      {
-        if (user->ip_allowed[i][0]=='\0') continue;
-        if (strcmp(str_tochar(ip),user->ip_allowed[i])==0) {
-          user->ip_allowed[i][0] = '\0';
-          found = 1;
-        }
-      }
-
-      if (!found) {
+      ret = ip_remove(&user->ip_list,str_tochar(ip));
+      if (ret != 0) {
         char buffer[256];
         snprintf(buffer,256,"IP %s not found",str_tochar(ip));
         ret = send_message_with_args(501,context,buffer);
@@ -624,13 +562,14 @@ int do_site_delip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
         return 0;
       }
       str_deallocate(ip);
+
     } /* if (*ptr=='\0') */
 
     ip = str_tok(command_line," \t\r\n");
   } while (ip);
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IP);
+  backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_IP);
   ret = send_message_with_args(200,context,"User ip(s) removed");
   return 0;
 }
@@ -648,7 +587,7 @@ int do_site_color(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
   int i, found, ret;
 
   me = GetUserByID(context->userid);
-  
+
   found=0;
   src_ptr = me->flags;
   dst_ptr = new_flags;
@@ -661,12 +600,12 @@ int do_site_color(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
     *dst_ptr++ = FLAG_COLOR;
     *dst_ptr='\0';
     memcpy(me->flags,new_flags,MAX_FLAGS_NUM);
-    ret = backend_mod_user(mainConfig->backend.filename,me->username,me,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backends->filename,me->uid,me,_USER_FLAGS);
     ret = send_message_with_args(200,context,"color mode ON");
   } else {
     *dst_ptr='\0';
     memcpy(me->flags,new_flags,MAX_FLAGS_NUM);
-    ret = backend_mod_user(mainConfig->backend.filename,me->username,me,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backends->filename,me->uid,me,_USER_FLAGS);
     ret = send_message_with_args(200,context,"color mode OFF");
   }
   return 0;
@@ -712,6 +651,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   unsigned int i;
   short is_gadmin;
   char old_username[HARD_USERNAME_LENGTH];
+  uid_t uid;
 
   me = GetUserByID(context->userid);
   is_gadmin = (me->flags && strchr(me->flags,FLAG_GADMIN)) ? 1 : 0;
@@ -724,6 +664,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   /* check if user  exists */
   strncpy(old_username,str_tochar(username),HARD_USERNAME_LENGTH);
   str_deallocate(username);
+
   user = GetUserByName( old_username );
   if ( !user) {
     ret = send_message_with_args(501,context,"User does not exist");
@@ -741,6 +682,8 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     return 0;
   }
 
+  uid = user->uid;
+
   /* GAdmin ? */
   if (is_gadmin)
   {
@@ -824,7 +767,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     ul=strtoul(str_tochar(value),&ptr,0);
     if (!*ptr) { mod_type = _USER_PERMS; user->userperms = ul;}
   }
-  /* flags */ /* TODO accept modifications style +f or -f */
+  /* flags */
   else if (strcmp(str_tochar(field),"flags")==0) {
     /* GAdmin ? */
     if (is_gadmin) {
@@ -923,7 +866,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   i = user->uid;
 
   /* commit to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,old_username,user,mod_type);
+  ret = backend_mod_user(mainConfig->backends->filename,uid,user,mod_type);
 
   /* user has been modified, we have to refresh cache entry */
   user = GetUserByID(i);
@@ -980,11 +923,11 @@ int do_site_changegrp(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
     return 0;
   }
 
-  /* check if user  exists */
+  /* check if user exists */
   user=GetUserByName(str_tochar(username));
   str_deallocate(username);
   if ( !user ) {
-    ret = send_message_with_args(501,context,"User does not exist");
+    ret = send_message_with_args(501,context,"user does not exist");
     return 0;
   }
 
@@ -1006,7 +949,7 @@ int do_site_changegrp(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
     if (newgroupid != (unsigned int)-1) {
       ret=0;
       for (i=0; i<user->group_num; i++)
-        if (newgroupid == user->groups[i]) { ret=1; break; } 
+        if (newgroupid == user->groups[i]) { ret=1; break; }
       if (ret) { /* remove from group, shift them */
         user->groups[i] = 0;
         for (;i<user->group_num-1; i++)
@@ -1030,7 +973,7 @@ int do_site_changegrp(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   } /* while (group_name) */
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.filename,user->username,user,mod_type);
+  backend_mod_user(mainConfig->backends->filename,user->uid,user,mod_type);
 
   ret = send_message_with_args(200,context,"User field change successfull");
   return 0;
@@ -1052,8 +995,7 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   char *ptr=NULL;
   wzd_string_t * str_ratio, *username;
   int ret;
-  wzd_user_t user, *me;
-  int uid;
+  wzd_user_t *user, *me;
   unsigned int ratio, oldratio;
   short is_gadmin;
 
@@ -1072,10 +1014,11 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
     return 0;
   }
 
-  /* check if user already exists */
-  if ( backend_find_user(str_tochar(username),&user,&uid) ) {
-    ret = send_message_with_args(501,context,"User does not exists");
-    str_deallocate(username); str_deallocate(str_ratio);
+  /* check if user exists */
+  user=GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( !user ) {
+    ret = send_message_with_args(501,context,"user does not exist");
     return 0;
   }
 
@@ -1083,7 +1026,6 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
 
   if (*ptr!='\0') {
     do_site_help_chratio(context);
-    str_deallocate(username);
     return 0;
   }
   str_deallocate(str_ratio);
@@ -1092,10 +1034,9 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   if (is_gadmin)
   {
     /* GAdmins cannot change user from different group */
-    if (me->group_num==0 || user.group_num==0 || me->groups[0]!=user.groups[0])
+    if (me->group_num==0 || user->group_num==0 || me->groups[0]!=user->groups[0])
     {
       ret = send_message_with_args(501,context,"You are not allowed to change users from this group");
-      str_deallocate(username);
       return 0;
     }
   }
@@ -1105,15 +1046,15 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   {
     if (me->leech_slots == 0) {
       ret = send_message_with_args(501,context,"No more slots available");
-      str_deallocate(username); str_deallocate(str_ratio);
+      str_deallocate(str_ratio);
       return 0;
     }
   }
-  oldratio = user.ratio;
-  user.ratio = ratio;
+  oldratio = user->ratio;
+  user->ratio = ratio;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_RATIO);
+  ret = backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_RATIO);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -1127,7 +1068,6 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
     }
     ret = send_message_with_args(200,context,"User ratio changed");
   }
-  str_deallocate(username);
   return 0;
 }
 
@@ -1143,8 +1083,7 @@ int do_site_flags(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   wzd_string_t *newflags = NULL;
   wzd_string_t * username = NULL;
   int ret;
-  wzd_user_t user;
-  int uid;
+  wzd_user_t * user;
 
   username = str_tok(command_line," \t\r\n");
   if (!username) {
@@ -1155,14 +1094,15 @@ int do_site_flags(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   }
 
   /* check if user exists */
-  if ( backend_find_user(str_tochar(username),&user,&uid) ) {
-    ret = send_message_with_args(501,context,"User does not exist");
-    str_deallocate(username); str_deallocate(newflags);
+  user=GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( !user ) {
+    ret = send_message_with_args(501,context,"user does not exist");
     return 0;
   }
 
   if (!newflags) {
-    snprintf(buffer,1023,"Flags for %s: %s",str_tochar(username),user.flags);
+    snprintf(buffer,1023,"Flags for %s: %s",user->username,user->flags);
 
     ret = send_message_with_args(200,context,buffer);
   } else {
@@ -1174,33 +1114,33 @@ int do_site_flags(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
     /* GAdmin ? */
     if (is_gadmin)
     {
-      if (me->group_num==0 || user.group_num==0 || me->groups[0]!=user.groups[0]) {
+      if (me->group_num==0 || user->group_num==0 || me->groups[0]!=user->groups[0]) {
         ret = send_message_with_args(501,context,"You can't change this user");
-        str_deallocate(username); str_deallocate(newflags);
+        str_deallocate(newflags);
         return 0;
       }
     }
     /* authorized ? */
     if (!strchr(me->flags,FLAG_SITEOP)) {
       ret = send_message_with_args(501,context,"You can't change flags for other users");
-      str_deallocate(username); str_deallocate(newflags);
+      str_deallocate(newflags);
       return 0;
     }
 
-    if (_user_changeflags(&user,str_tochar(newflags))) {
+    if (_user_changeflags(user,str_tochar(newflags))) {
       ret = send_message_with_args(501,context,"Error occurred changing flags");
-      str_deallocate(username); str_deallocate(newflags);
+      str_deallocate(newflags);
       return 0;
     }
     /* commit to backend */
-    ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_FLAGS);
     if (!ret)
       ret = send_message_with_args(200,context,"Flags changed");
     else
       ret = send_message_with_args(501,context,"Flags changed, but error committing changes to backend");
   }
 
-  str_deallocate(username); str_deallocate(newflags);
+  str_deallocate(newflags);
   return 0;
 }
 
@@ -1238,7 +1178,7 @@ int do_site_idle(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     }
     user->max_idle_time = idletime;
     /* commit to backend */
-    backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IDLE);
+    backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_IDLE);
     snprintf(buffer,1023,"%s","Command ok");
   } else { /* if (*command_line != '\0') */
 
@@ -1270,7 +1210,7 @@ int do_site_tagline(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   if (command_line && strlen(str_tochar(command_line))>0) {
     strncpy(user->tagline,str_tochar(command_line),255);
     /* commit to backend */
-    backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_TAGLINE);
+    backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_TAGLINE);
     snprintf(buffer,1023,"%s","Command ok");
   } else { /* if (*command_line != '\0') */
 
@@ -1298,7 +1238,7 @@ int do_site_kill(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
     return 0;
   }
 
-  switch(kill_child(pid,context)) {
+  switch(kill_child_new(pid,context)) {
   case 0:
     ret = send_message_with_args(200,context,"KILL signal sent");
     break;
@@ -1322,31 +1262,27 @@ int do_site_kill(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
  */
 int do_site_kick(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t * context)
 {
-  wzd_string_t *_username;
-  const char *username, *test_username;
+  wzd_string_t *username;
   int ret;
   int found = 0;
-  wzd_user_t user;
-  int uid;
+  wzd_user_t *user;
 
-  _username = str_tok(param," \t\r\n");
-  if (!_username) {
+  username = str_tok(param," \t\r\n");
+  if (!username) {
     ret = send_message_with_args(501,context,"Usage: site kick <user>");
     return 0;
   }
-  username = str_tochar(_username);
   /* check if user  exists */
-  if ( backend_find_user(username,&user,&uid) ) {
+  user = GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( user == NULL ) {
     ret = send_message_with_args(501,context,"User does not exist");
-    str_deallocate(_username);
     return 0;
   }
 
   /* preliminary check: i can't kill myself */
-  test_username = GetUserByID(context->userid)->username;
-  if (strcmp(username,test_username)==0) {
+  if (user->uid == context->userid) {
     ret = send_message_with_args(501,context,"My religion forbids me suicide !");
-    str_deallocate(_username);
     return 0;
   }
 
@@ -1357,8 +1293,7 @@ int do_site_kick(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
     for (elmnt=list_head(context_list); elmnt; elmnt=list_next(elmnt)) {
       loop_context = list_data(elmnt);
       if (loop_context && loop_context->magic == CONTEXT_MAGIC) {
-        test_username = GetUserByID(loop_context->userid)->username;
-        if (strcmp(username,test_username)==0) {
+        if (user->uid == loop_context->userid) {
           found = 1;
           kill_child_new(loop_context->pid_child,context);
         }
@@ -1368,7 +1303,6 @@ int do_site_kick(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
   if (!found) { ret = send_message_with_args(501,context,"User is not logged !"); }
   else { ret = send_message_with_args(200,context,"KILL signal sent"); }
 
-  str_deallocate(_username);
   return 0;
 }
 
@@ -1418,7 +1352,7 @@ int do_site_killpath(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
       ret = send_message_with_args(501,context,"Unknown error");
       break;
   };
-  
+
   return 0;
 }
 
@@ -1436,8 +1370,7 @@ int do_site_give(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
   char *ptr;
   wzd_string_t * str_give, *username;
   int ret;
-  wzd_user_t user, *me;
-  int uid;
+  wzd_user_t *user, *me;
   u64_t kbytes;
   short is_gadmin;
 
@@ -1456,17 +1389,18 @@ int do_site_give(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     return 0;
   }
 
-  /* check if user already exists */
-  if ( backend_find_user(str_tochar(username),&user,&uid) ) {
-    ret = send_message_with_args(501,context,"User does not exists");
-    str_deallocate(username); str_deallocate(str_give);
+  /* check if user exists */
+  user=GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( !user ) {
+    ret = send_message_with_args(501,context,"user does not exist");
     return 0;
   }
 
   kbytes = strtoull(str_tochar(str_give),&ptr,0);
   if (*ptr!='\0') {
     do_site_help_give(context);
-    str_deallocate(username); str_deallocate(str_give);
+    str_deallocate(str_give);
     return 0;
   }
   str_deallocate(str_give);
@@ -1488,23 +1422,21 @@ int do_site_give(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
   /* check user credits */
   if (me->credits && me->credits < kbytes) {
     ret = send_message_with_args(501,context,"You don't have enough credits !");
-    str_deallocate(username);
     return 0;
   }
 
-  user.credits += kbytes;
+  user->credits += kbytes;
   if (me->credits)
     me->credits -= kbytes;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_CREDITS);
+  ret = backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_CREDITS);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
   } else {
     ret = send_message_with_args(200,context,"Credits given");
   }
-  str_deallocate(username);
   return 0;
 }
 
@@ -1522,8 +1454,7 @@ int do_site_take(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
   char *ptr;
   wzd_string_t * str_take, *username;
   int ret;
-  wzd_user_t user, *me;
-  int uid;
+  wzd_user_t *user, *me;
   u64_t kbytes;
   short is_gadmin;
 
@@ -1542,17 +1473,18 @@ int do_site_take(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     return 0;
   }
 
-  /* check if user already exists */
-  if ( backend_find_user(str_tochar(username),&user,&uid) ) {
-    ret = send_message_with_args(501,context,"User does not exists");
-    str_deallocate(username); str_deallocate(str_take);
+  /* check if user exists */
+  user=GetUserByName(str_tochar(username));
+  str_deallocate(username);
+  if ( !user ) {
+    ret = send_message_with_args(501,context,"user does not exist");
     return 0;
   }
 
   kbytes = strtoull(str_tochar(str_take),&ptr,0);
   if (*ptr!='\0') {
     do_site_help_take(context);
-    str_deallocate(username); str_deallocate(str_take);
+    str_deallocate(str_take);
     return 0;
   }
   str_deallocate(str_take);
@@ -1572,26 +1504,24 @@ int do_site_take(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
 #endif /* 0 */
 
   /* check user credits */
-  if (user.ratio==0) {
+  if (user->ratio==0) {
     ret = send_message_with_args(501,context,"User has unlimited credits !");
-    str_deallocate(username);
     return 0;
   }
 
-  if (user.credits > kbytes)
-    user.credits -= kbytes;
+  if (user->credits > kbytes)
+    user->credits -= kbytes;
   else
-    user.credits = 0;
+    user->credits = 0;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_CREDITS);
+  ret = backend_mod_user(mainConfig->backends->filename,user->uid,user,_USER_CREDITS);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
   } else {
     ret = send_message_with_args(200,context,"Credits removed");
   }
-  str_deallocate(username);
   return 0;
 }
 
@@ -1763,3 +1693,46 @@ static void _flags_simplify(char *flags, size_t length)
     }
   }
 }
+
+/** \brief iterate users and purge those marked as deleted, kicking them if logged
+ * One important thing is that a deleted user can't login if deleted,
+ * otherwise we have a race condition here
+ */
+static int _kick_and_purge(void)
+{
+  unsigned int i;
+  int * uid_list;
+  wzd_user_t * user;
+  ListElmt * elmnt;
+  wzd_context_t * loop_context;
+
+  uid_list = (int*)backend_get_user(GET_USER_LIST);
+  if (uid_list == NULL) return -1;
+
+  out_log(LEVEL_FLOOD,"DEBUG calling _kick_and_purge\n");
+
+  /* step 1: kick all deleted users */
+  for (elmnt=list_head(context_list); elmnt; elmnt=list_next(elmnt)) {
+    loop_context = list_data(elmnt);
+    if (loop_context && loop_context->magic == CONTEXT_MAGIC) {
+      user = GetUserByID(loop_context->userid);
+      if (user && user->flags && strchr(user->flags,FLAG_DELETED)) {
+        kill_child_signal(loop_context->pid_child,NULL /* context */);
+      }
+    }
+  }
+
+  /* step 2: purge all deleted users */
+  for (i=0; uid_list[i] >= 0; i++) {
+    user = GetUserByID(uid_list[i]);
+
+    if (user && user->flags && strchr(user->flags,FLAG_DELETED)) {
+      /* commit changes to backend */
+      backend_mod_user(mainConfig->backends->filename,user->uid,NULL,_USER_ALL);
+    }
+  }
+  wzd_free (uid_list);
+
+  return 0;
+}
+
