@@ -566,7 +566,7 @@ int socket_connect(unsigned char * remote_host, int family, int remote_port, int
   return sock;
 }
 
-/* Returns the local/remote port for the socket. */
+/** \brief Returns the local/remote port for the socket. */
 int get_sock_port(int sock, int local)
 {
 #if !defined(WIN32) && !defined(__sun__)
@@ -878,7 +878,7 @@ static fd_t socket_make_v6(const char *ip, unsigned int *port, int nListen)
 #ifdef IPV6_V6ONLY
   if (setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&one, sizeof (one)) < 0)
   {
-    out_log(LEVEL_HIGH,"Could not bind socket to IPv6 only (%s:%d) %s:%d\n", ip, port, __FILE__, __LINE__);
+    out_log(LEVEL_HIGH,"Could not bind socket to IPv6 only (%s:%d) %s:%d\n", ip, *port, __FILE__, __LINE__);
     return -1;
   }
 #endif
