@@ -53,6 +53,7 @@ typedef enum {
   E_PARAM_NULL,		/**< parameter is NULL */
   E_PARAM_BIG,		/**< parameter is too long */
   E_PARAM_INVALID,	/**< parameter is invalid */
+  E_PARAM_EXIST,	/**< parameter already exist */
 
   E_WRONGPATH,		/**< path is invalid */
 
@@ -142,13 +143,6 @@ typedef struct limiter
 /* opaque struct */
 typedef struct wzd_site_fct_t wzd_site_fct_t;
 
-/********************* IP CHECKING ************************/
-typedef struct _wzd_ip_list_t {
-  char  * regexp;
-  u8_t  is_allowed;
-  struct _wzd_ip_list_t * next_ip;
-} wzd_ip_list_t;
-
 /************************ VFS *****************************/
 typedef struct _wzd_vfs_t {
   char	* virtual_dir;
@@ -177,45 +171,9 @@ typedef struct {
 
 /********************** USER, GROUP ***********************/
 
-/** @brief User definition
- */
-typedef struct {
-  u32_t                 uid;
-  char                  username[HARD_USERNAME_LENGTH];
-  char			userpass[MAX_PASS_LENGTH];
-  char                  rootpath[WZD_MAX_PATH];
-  char                  tagline[MAX_TAGLINE_LENGTH];
-  unsigned int          group_num;
-  unsigned int          groups[MAX_GROUPS_PER_USER];
-  u32_t 	        max_idle_time;
-  wzd_perm_t            userperms;      /**< @brief default permissions */
-  char                  flags[MAX_FLAGS_NUM];
-  u32_t                 max_ul_speed;
-  u32_t                 max_dl_speed;   /**< @brief bytes / sec */
-  unsigned short	num_logins;	/**< @brief number of simultaneous logins allowed */
-  char			ip_allowed[HARD_IP_PER_USER][MAX_IP_LENGTH];
-  wzd_stats_t		stats;
-  u64_t         	credits;
-  unsigned int		ratio;
-  unsigned short	user_slots;	/**< @brief user slots for gadmins */
-  unsigned short	leech_slots;	/**< @brief leech slots for gadmins */
-  time_t		last_login;
-} wzd_user_t;
+typedef struct wzd_user_t wzd_user_t;
 
-/** @brief Group definition */
-typedef struct {
-  unsigned int          gid;
-  char                  groupname[HARD_GROUPNAME_LENGTH];
-  char                  tagline[MAX_TAGLINE_LENGTH];
-  wzd_perm_t            groupperms;
-  u32_t 		max_idle_time;
-  unsigned short	num_logins;	/**< number of simultaneous logins allowed */
-  u32_t                 max_ul_speed;
-  u32_t                 max_dl_speed;
-  unsigned int		ratio;
-  char			ip_allowed[HARD_IP_PER_GROUP][MAX_IP_LENGTH];
-  char			defaultpath[WZD_MAX_PATH];
-} wzd_group_t;
+typedef struct wzd_group_t wzd_group_t;
 
 /*********************** BACKEND **************************/
 
@@ -231,13 +189,20 @@ typedef struct {
  */
 
 
+
+typedef int (*backend_init_function_t)(struct wzd_backend_t*);
+
 struct wzd_backend_def_t {
   char * filename;
 
   char * param;
   void * handle;
 
+  backend_init_function_t fcn_init;
+
   struct wzd_backend_t * b;
+
+  struct wzd_backend_def_t * next_backend;
 };
 
 
@@ -302,8 +267,10 @@ enum event_id_t {
   EVENT_PRERMDIR      = 0x00000400,
   EVENT_RMDIR         = 0x00000800,
 
-  EVENT_PREDELE       = 0x00004000,
-  EVENT_DELE          = 0x00008000,
+  EVENT_PREDELE       = 0x00001000,
+  EVENT_DELE          = 0x00002000,
+  EVENT_PREWIPE       = 0x00004000,
+  EVENT_WIPE          = 0x00008000,
 
   EVENT_SITE          = 0x00010000,
   EVENT_CRONTAB       = 0x00100000,
@@ -333,35 +300,13 @@ typedef struct {
   unsigned long num_childs; /**< @brief total # of childs process created since server start */
 } wzd_server_stat_t;
 
-/********************** SERVER PARAMS *********************/
-
-/** @brief Server parameters: stored in server global memory space,
- * accessible to every thread.
- */
-typedef struct _wzd_param_t {
-  char * name;
-  void * param;
-  unsigned int length;
-
-  struct _wzd_param_t	* next_param;
-} wzd_param_t;
-
 /*************************** IP **************************/
 
 #include "wzd_ip.h"
 
 /*************************** TLS **************************/
 
-#ifndef HAVE_OPENSSL
-# define SSL     void
-# define SSL_CTX void
-#else
-# include <openssl/ssl.h>
-# include <openssl/rand.h>
-# include <openssl/err.h>
-#endif
-
-typedef enum { TLS_CLEAR, TLS_PRIV } ssl_data_t; /* data modes */
+typedef enum { TLS_CLEAR, TLS_PRIV } tls_data_mode_t; /* data modes */
 
 typedef enum { TLS_SERVER_MODE=0, TLS_CLIENT_MODE } tls_role_t;
 
@@ -369,14 +314,6 @@ typedef enum { TLS_NOTYPE=0, TLS_EXPLICIT, TLS_STRICT_EXPLICIT, TLS_IMPLICIT } t
 
 typedef enum { TLS_NONE, TLS_READ, TLS_WRITE } ssl_fd_mode_t;
 
-/** @brief SSL connection objects */
-typedef struct {
-  SSL *         obj;
-  ssl_data_t    data_mode;
-  SSL *         data_ssl;
-  ssl_fd_mode_t ssl_fd_mode;
-} wzd_ssl_t;
-
 typedef struct {
   void * session;
   void * data_session;
@@ -422,13 +359,16 @@ struct wzd_context_t {
 
   net_family_t  family; /**< \brief IPv4 or IPv6 */
   unsigned char	hostip[16];
+  int           localport;
   wzd_ip_t      * peer_ip;
   char          * ident;
+  char          * idnt_address;
   connection_state_t state;
   unsigned char	exitclient;
   fd_t          controlfd;
   fd_t          datafd;
   data_mode_t   datamode;
+  tls_data_mode_t    tls_data_mode;
   net_family_t  datafamily; /**< \brief IPv4 or IPv6 */
   unsigned long	pid_child;
   unsigned long	thread_id;
@@ -446,13 +386,15 @@ struct wzd_context_t {
   wzd_action_t	current_action;
   struct last_file_t	last_file;
   char          * data_buffer;
+  char          * control_buffer;
 /*  wzd_bw_limiter * current_limiter;*/
   wzd_bw_limiter current_ul_limiter;
   wzd_bw_limiter current_dl_limiter;
   time_t        login_time;
   time_t	idle_time_start;
   time_t	idle_time_data_start;
-  wzd_ssl_t   	ssl;
+  struct wzd_ssl_t * ssl;
+  struct wzd_reply_t * reply;
   wzd_tls_t   	tls;
   tls_role_t    tls_role; /**< \brief TLS role: server or client */
   struct _auth_gssapi_data_t * gssapi_data;
@@ -475,6 +417,7 @@ struct wzd_context_t {
 #define CFG_OPT_UTF8_CAPABLE                0x00001000
 #define CFG_OPT_CHECKIP_LOGIN               0x00010000
 #define CFG_OPT_REJECT_UNKNOWN_USERS        0x00020000
+#define CFG_OPT_DYNAMIC_IP                  0x00100000
 
 
 #define CFG_CLR_OPTION(c,opt)   (c)->server_opts &= ~(opt)
@@ -491,7 +434,7 @@ struct wzd_config_t {
   time_t	server_start;
   unsigned char	serverstop;
   unsigned char	site_closed;
-  wzd_backend_def_t	backend;
+  wzd_backend_def_t * backends;
   int		max_threads;
   char *	logfilename;
   unsigned int	logfilemode;
@@ -509,19 +452,19 @@ struct wzd_config_t {
   u32_t         pasv_low_range;
   u32_t         pasv_high_range;
   unsigned char	pasv_ip[16];
-  wzd_ip_list_t	*login_pre_ip_checks;
+  struct wzd_ip_list_t	*login_pre_ip_checks;
   wzd_vfs_t	*vfs;
   wzd_hook_t	*hook;
   wzd_module_t	*module;
   unsigned int  data_buffer_length; /**< size of buffer used for transfers. This has a great impact on performances */
   unsigned long	server_opts;
   wzd_server_stat_t	stats;
-  SSL_CTX *	tls_ctx;
+/*  SSL_CTX *	tls_ctx;*/ /** \todo XXX casting with void* is bad ... use correct type ? */
+  void * tls_ctx;
   tls_type_t	tls_type;
   CHTBL          * commands_list;
   wzd_site_fct_t	* site_list;
   wzd_section_t		* section_list;
-  wzd_param_t		* param_list;
 
   wzd_bw_limiter	global_ul_limiter;
   wzd_bw_limiter	global_dl_limiter;
