@@ -39,16 +39,16 @@
 #include <sys/types.h>
 
 #ifdef WIN32
-#define _WIN32_WINNT    0x500
-#include <windows.h>
+# define _WIN32_WINNT    0x500
+# include <windows.h>
 #else
 #include <unistd.h>
-#include <pthread.h>
 
 #include <signal.h>
 #endif
 
-#include "wzd_structs.h"
+struct wzd_context_t;
+
 #include "wzd_log.h"
 #include "wzd_threads.h"
 
@@ -136,7 +136,7 @@ int wzd_thread_join(wzd_thread_t * thread, void ** thread_return)
   {
     out_log(LEVEL_CRITICAL, "Thread join failed.");
     CloseHandle(thread->_t);
-	  
+
     return -1;
   }
   CloseHandle(thread->_t);
@@ -144,3 +144,15 @@ int wzd_thread_join(wzd_thread_t * thread, void ** thread_return)
   return 0;
 #endif
 }
+/** \brief Cancel thread by sending a signal */
+int wzd_thread_cancel(wzd_thread_t * thread)
+{
+#ifndef WIN32
+  return pthread_cancel(thread->_t);
+#else
+  /** \todo use pthread_kill() equivalent for windows */
+  out_log(LEVEL_CRITICAL, "Not Yet Implemented : wzd_thread_cancel\n");
+  return -1;
+#endif
+}
+
