@@ -42,7 +42,7 @@
 #include <direct.h>
 #include <io.h>
 
-#include "../../visual/gnu_regex_dist/regex.h"
+#include "../../visual/gnu_regex/regex.h"
 #else
 #include <dirent.h>
 #include <sys/types.h>
@@ -77,6 +77,7 @@
 #include <libwzd-core/wzd_vfs.h> /* checkpath_new */
 #include <libwzd-core/wzd_mod.h> /* essential to define WZD_MODULE_INIT */
 #include <libwzd-core/wzd_string.h>
+#include <libwzd-core/wzd_user.h>
 #include <libwzd-core/wzd_vars.h> /* needed to access variables */
 
 #include <libwzd-core/wzd_debug.h>
@@ -170,11 +171,11 @@ int WZD_MODULE_INIT(void)
     }
     if (logdir) {
       int fd;
-	  const char * filename;
+      const char * filename;
 
       wzd_string_t *str = str_allocate();
       str_sprintf(str,"%s/%s", logdir, PERL_ERRORLOGNAME);
-	  filename = str_tochar(str);
+      filename = str_tochar(str);
 #ifndef WIN32
       fd = open(filename,O_CREAT|O_WRONLY,S_IRUSR | S_IWUSR);
 #else
@@ -297,7 +298,8 @@ static int perl_hook_protocol(const char *file, const char *args)
   wzd_context_t * context;
   wzd_user_t * user;
   unsigned int reply_code;
-  SV * perl_args;
+  SV * perl_args, *retval;
+  int ret;
 
   current_context = context = GetMyContext();
   user = GetUserByID(context->userid);
@@ -311,13 +313,19 @@ static int perl_hook_protocol(const char *file, const char *args)
     sv_setpv(perl_args, args);
   }
 
+  retval = get_sv("wzd::return",TRUE);
+  if (retval) sv_setpv(retval,0);
+
   execute_perl(newSVpvn("Embed::load", 11), file);
 
 /*  SvREFCNT_dec(perl_args);*/ /* NO !! this will segfault on second call ! */
 
+  retval = get_sv("wzd::return",FALSE);
+  if (retval) ret = SvIV(retval);
+
   current_context = NULL;
 
-  return 0;
+  return ret;
 }
 
 static void do_perl_help(wzd_context_t * context)
@@ -376,7 +384,7 @@ static PerlInterpreter * perl_init(void)
 "}\n"
   };
 
-  char * perl_args[] = { "", "-e", "0", "-w" };
+  char * perl_args[] = { "", "-e", "0" };
   PerlInterpreter * interp = NULL;
 
   interp = perl_alloc();
@@ -445,7 +453,7 @@ static int execute_perl( SV *function, const char *args)
       str_sprintf(str,"Error in %s: %s\n",args,SvPV_nolen(ERRSV));
       write(perl_fd_errlog,str_tochar(str),strlen(str_tochar(str)));
     }
-    POPs; /* remove undef from the top of the stack */
+    sv = POPs; /* remove undef from the top of the stack */
   }
   else if (count != 1) {
     /* error, we expected only 1 value */
@@ -961,18 +969,29 @@ static XS(XS_wzd_vars_group)
   dXSARGS;
 
   if (!current_context) XSRETURN_UNDEF;
-  if (items < 3) XSRETURN_UNDEF;
+  if (items < 2) XSRETURN_UNDEF;
 
   /** \todo print error message */
   if ( ! SvPOK(ST(0)) )
     XSRETURN_UNDEF;
   if ( ! SvPOK(ST(1)) )
     XSRETURN_UNDEF;
-  if ( ! SvPOK(ST(2)) )
-    XSRETURN_UNDEF;
 
   command = SvPV_nolen(ST(0));
   groupname = SvPV_nolen(ST(1));
+
+  if (!strcmp(command,"new")) { /* new user creation */
+    ret = vars_group_new(groupname,getlib_mainConfig());
+    /** \todo handle return */
+    if (!ret)
+      XSRETURN_PV("command ok");
+    else
+      XSRETURN_UNDEF;
+  }
+  
+  if (items < 3) XSRETURN_UNDEF;
+  if ( ! SvPOK(ST(2)) )
+    XSRETURN_UNDEF;
   text = SvPV_nolen(ST(2));
 
   if (!strcmp(command,"get")) {
@@ -992,13 +1011,6 @@ static XS(XS_wzd_vars_group)
       XSRETURN_PV(buffer);
     else
       XSRETURN_UNDEF;
-  } else if (!strcmp(command,"new")) { /* new user creation */
-    ret = vars_group_new(groupname,getlib_mainConfig());
-    /** \todo handle return */
-    if (!ret)
-      XSRETURN_PV("command ok");
-    else
-      XSRETURN_UNDEF;
   }
 
   XSRETURN_UNDEF;
