@@ -46,7 +46,7 @@
 #include <direct.h>
 #include <io.h>
 
-#include "../../visual/gnu_regex_dist/regex.h"
+#include "../../visual/gnu_regex/regex.h"
 #else
 #include <dirent.h>
 #include <sys/types.h>
@@ -74,6 +74,7 @@
 #include <libwzd-core/wzd_libmain.h>
 #include <libwzd-core/wzd_messages.h>
 #include <libwzd-core/wzd_mod.h> /* essential to define WZD_MODULE_INIT */
+#include <libwzd-core/wzd_user.h>
 #include <libwzd-core/wzd_vfs.h> /* checkpath_new */
 #include <libwzd-core/wzd_vars.h> /* needed to access variables */
 
@@ -89,6 +90,7 @@ static int tcl_fd_errlog=-1;
 #define TCL_CURRENT_USER "wzd_current_user"
 #define TCL_REPLY_CODE  "wzd_reply_code"
 #define TCL_HAS_REPLIED "wzd_replied"
+#define TCL_WZD_RETURN "wzd_return"
 #define TCL_ERRORLOGNAME "tclerr.log"
 
 #define WZDOUT  ((ClientData)1)
@@ -370,6 +372,7 @@ static int tcl_hook_protocol(const char *file, const char *args)
   wzd_user_t * user;
   unsigned int reply_code;
   Tcl_Interp * slave = NULL;
+  char * ptr;
 
   current_context = context = GetMyContext();
   user = GetUserByID(context->userid);
@@ -389,6 +392,7 @@ static int tcl_hook_protocol(const char *file, const char *args)
   else
     Tcl_SetVar(slave,TCL_ARGS,"",TCL_GLOBAL_ONLY);
   Tcl_SetVar(slave,TCL_CURRENT_USER,user->username,TCL_GLOBAL_ONLY);
+  Tcl_SetVar(slave,TCL_WZD_RETURN,"",TCL_GLOBAL_ONLY);
 
   ret = Tcl_EvalFile(slave, file);
 
@@ -396,6 +400,7 @@ static int tcl_hook_protocol(const char *file, const char *args)
   current_context = NULL;
   Tcl_UnsetVar(slave,TCL_ARGS,TCL_GLOBAL_ONLY);
   Tcl_UnsetVar(slave,TCL_CURRENT_USER,TCL_GLOBAL_ONLY);
+#if 0
   s = Tcl_GetVar(slave,TCL_HAS_REPLIED,TCL_GLOBAL_ONLY);
 #if 0
   if (!s || *s!='1') {
@@ -405,6 +410,14 @@ static int tcl_hook_protocol(const char *file, const char *args)
       send_message_with_args(200,context,"TCL command ok");
   }
 #endif
+#endif
+
+  s = Tcl_GetVar(slave,TCL_WZD_RETURN,TCL_GLOBAL_ONLY);
+  if (s != NULL && *s != '\0') {
+    ret = strtoul(s,&ptr,0);
+    if (*ptr!='\0') return 0; /** \todo log invalid return code ? */
+    return ret;
+  }
 
   return 0;
 }
