@@ -1,10 +1,14 @@
 #include <stdlib.h> /* malloc */
 #include <string.h> /* memset */
-#include <pthread.h>
+#ifdef HAVE_PTHREAD_H
+# include <pthread.h>
+#endif
 
 #include <libwzd-core/wzd_structs.h>
+#include <libwzd-core/wzd_group.h>
 #include <libwzd-core/wzd_libmain.h>
 #include <libwzd-core/wzd_mod.h>
+#include <libwzd-core/wzd_user.h>
 #include <libwzd-core/wzd_utf8.h>
 
 #include <libwzd-core/wzd_cache.h>
@@ -12,6 +16,7 @@
 #include <libwzd-core/wzd_debug.h>
 
 #include "test_common.h"
+#include "fake_backend.h"
 
 void fake_write_function(fd_t fd, const char * msg, size_t msg_len, unsigned int timeout, wzd_context_t * context);
 
@@ -23,10 +28,18 @@ wzd_context_t * f_context = NULL;
 void fake_mainConfig(void)
 {
   wzd_config_t * config;
+  wzd_backend_def_t * def;
+  int ret;
+
   config = malloc(sizeof(wzd_config_t));
   memset(config, 0, sizeof(wzd_config_t));
 
+  def = backend_register(NULL,fake_backend_init);
+  config->backends = def;
+
   mainConfig = config;
+
+  ret = def->b->backend_init("param");
 }
 
 void fake_backend(void)
@@ -57,6 +70,8 @@ void fake_user(void)
   user->group_num = 1;
 
   f_user = user;
+
+  user_register(user,1 /* backend id */);
 }
 
 void fake_group(void)
@@ -71,6 +86,8 @@ void fake_group(void)
   group->gid = 333;
 
   f_group = group;
+
+  group_register(group,1 /* backend id */);
 }
 
 void fake_context(void)
@@ -83,17 +100,17 @@ void fake_context(void)
 
   wzd_debug_init();
 
-  usercache_init();
-  groupcache_add(f_group);
-  usercache_add(f_user);
-
   context = malloc(sizeof(wzd_context_t));
   memset(context, 0, sizeof(wzd_context_t));
 
   context->magic = CONTEXT_MAGIC;
   context->userid = f_user->uid;
   context->write_fct = (write_fct_t)fake_write_function;
+#ifndef WIN32
   context->thread_id = pthread_self();
+#else
+  context->thread_id = 1;
+#endif
 
   strcpy(context->currentpath, "/");
 
@@ -110,14 +127,13 @@ void fake_exit(void)
   if (f_context) {
     list_destroy(context_list);
     free(context_list);
-    usercache_fini();
     free(f_context);
     f_context = NULL;
 
     wzd_debug_fini();
   }
-  free(f_user); f_user = NULL;
-  free(f_group); f_group = NULL;
+  user_free_registry();
+  group_free_registry();
   if (mainConfig) {
     free(mainConfig);
     mainConfig = NULL;
