@@ -2,6 +2,8 @@
 #include <string.h>
 
 #include <libwzd-core/wzd_structs.h>
+#include <libwzd-core/wzd_user.h>
+#include <libwzd-core/wzd_group.h>
 
 #define C1 0x12345678
 #define C2 0x9abcdef0
