@@ -57,6 +57,13 @@ int main(int argc, const char **argv)
   wzd_parse_args(argc,argv);
   ret = wzd_init();
 
+  ret = wzd_connect();
+  if (ret < 0) {
+    fprintf(stderr,"Could not connect to server\n");
+    wzd_fini();
+    exit(-1);
+  }
+
   if (ret >= 0) {
 
     reply = wzd_send_message(msg,strlen(msg));
@@ -69,6 +76,8 @@ int main(int argc, const char **argv)
         }
       }
       wzd_free_reply(reply);
+    } else {
+      fprintf(stderr,"No reply from server\n");
     }
 
   }
