@@ -92,10 +92,12 @@
 #include <libwzd-core/wzd_configfile.h>
 #include <libwzd-core/wzd_configloader.h>
 #include <libwzd-core/wzd_crontab.h>
+#include <libwzd-core/wzd_group.h>
 #include <libwzd-core/wzd_messages.h>
 #include <libwzd-core/wzd_section.h>
 #include <libwzd-core/wzd_site.h>
 #include <libwzd-core/wzd_threads.h>
+#include <libwzd-core/wzd_user.h>
 #include <libwzd-core/wzd_utf8.h>
 #include <libwzd-core/wzd_vars.h>
 
@@ -103,9 +105,11 @@
 
 #include <libwzd-core/wzd_debug.h>
 
+#include "wzd_version.h"
+
 /************ PROTOTYPES ***********/
-void serverMainThreadProc(void *arg);
-void serverMainThreadExit(int);
+int serverMainThreadProc(void *arg);
+void serverMainThreadCleanup(int);
 void server_crashed(int signum);
 
 void child_interrupt(int signum);
@@ -132,7 +136,8 @@ typedef struct {
 /************ PUBLIC **************/
 int runMainThread(int argc, char **argv)
 {
-  const char * build_version = WZD_VERSION_STR;
+  const char * build_version = WZD_VERSION_LONG;
+  int ret;
 
   out_log(LEVEL_FLOOD,"DEBUG Checking library version\n");
 
@@ -142,23 +147,21 @@ int runMainThread(int argc, char **argv)
    */
   if (strcmp(build_version,wzd_get_version_long()) != 0) {
     out_log(LEVEL_CRITICAL,"FATAL wzdftpd was NOT compiled for this library version\n");
-    out_log(LEVEL_CRITICAL,"FATAL   wzdftpd:     [%s]\n",WZD_VERSION_STR);
+    out_log(LEVEL_CRITICAL,"FATAL   wzdftpd:     [%s]\n",WZD_VERSION_LONG);
     out_log(LEVEL_CRITICAL,"FATAL   libwzd-core: [%s]\n",wzd_get_version_long());
 
     exit (-1);
   }
 
-  serverMainThreadProc(0);
+  ret = serverMainThreadProc(0);
 
-  return 0;
+  return ret;
 }
 
 /************ PRIVATE *************/
 
 static wzd_mutex_t * end_mutex = NULL;
 
-static void free_config(wzd_config_t * config);
-
 static List server_ip_list;
 static List server_ident_list;
 static int server_add_ident_candidate(fd_t socket_accept_fd);
@@ -179,33 +182,14 @@ static void server_ip_free(void * p);
 static int check_ip_before_login(wzd_context_t * context);
 
 
-static void context_init(wzd_context_t * context)
-{
-  memset(context,0,sizeof(wzd_context_t));
-  context->controlfd = -1;
-  context->datafd = -1;
-  context->pasvsock = -1;
-  context->userid = (unsigned int)-1;
-  context->thread_id = (unsigned long)-1;
-  context->state = STATE_UNKNOWN;
-  context->datamode = DATA_PORT;
-  context->current_action.current_file = -1;
-  context->current_action.token = TOK_UNKNOWN;
-  memset(&context->last_file,0,sizeof(context->last_file));
-
-  context->peer_ip = ip_create();
-
-  context->tls_role = TLS_SERVER_MODE;
-  context->read_fct = (read_fct_t)clear_read;
-  context->write_fct = (write_fct_t)clear_write;
-}
 
 static wzd_context_t * context_find_free(List * context_list)
 {
   wzd_context_t * context=NULL;
 
   wzd_mutex_lock(server_mutex);
-  context = wzd_malloc(sizeof(wzd_context_t));
+  context = context_alloc();
+  if (context == NULL) return NULL;
   context_init(context);
   if (list_ins_next(context_list, NULL, context))
   {
@@ -244,6 +228,9 @@ static int global_check_ip_allowed(unsigned char *userip)
 {
   char ip[INET6_ADDRSTRLEN];
 
+  /** \warning If no ip was specified (ok or denied), then the default is to allow */
+  if (mainConfig->login_pre_ip_checks == NULL) return 1;
+
 #if !defined(IPV6_SUPPORT)
   inet_ntop(AF_INET,userip,ip,INET_ADDRSTRLEN);
 #else
@@ -268,7 +255,8 @@ void server_rebind(const char *new_ip, unsigned int new_port)
   if (sock == (fd_t)-1) {
       out_log(LEVEL_CRITICAL,"Error creating socket %s:%d\n",
           __FILE__, __LINE__);
-      serverMainThreadExit(-1);
+      serverMainThreadCleanup(-1);
+      return -1;
   }
   {
     int one=1;
@@ -286,8 +274,19 @@ void server_rebind(const char *new_ip, unsigned int new_port)
  */
 int check_server_dynamic_ip(void)
 {
-  out_log(LEVEL_HIGH,"ERROR check_server_dynamic_ip: not implemented yet\n");
-  return -1;
+  if (CFG_GET_OPTION(mainConfig,CFG_OPT_DYNAMIC_IP)) {
+    out_log(LEVEL_FLOOD,"DEBUG check_server_dynamic_ip: not implemented yet\n");
+
+    /* get ip and port */
+
+    /* update pasv_ip */
+
+    /* rebind server if needed */
+
+    return -1;
+  }
+
+  return 0;
 #if 0
   struct sockaddr_in sa_current, sa_config;
   unsigned int size;
@@ -395,7 +394,7 @@ int commit_backend(void)
 {
   /* TODO XXX FIXME flush backend IFF modified ! */
   if (!mainConfig) return 1;
-  backend_commit_changes(mainConfig->backend.filename);
+  backend_commit_changes(mainConfig->backends->filename);
   return 0;
 }
 
@@ -420,25 +419,25 @@ static int check_ip_before_login(wzd_context_t * context)
   }
 
   /* check for all groups */
-  gid_list = (gid_t*)backend_get_group(-2);
+  gid_list = (gid_t*)backend_get_group(GET_GROUP_LIST);
   if (gid_list) {
     for (i=0; gid_list[i] != (gid_t)-1; i++) {
       loop_group = GetGroupByID(gid_list[i]);
       if (loop_group) {
-        ret = group_ip_inlist(loop_group, inet_buf, context->ident);
-        if (ret) return 0; /* found ! */
+        ret = ip_list_check_ident(loop_group->ip_list, inet_buf, context->ident);
+        if (ret > 1) return 0; /* found ! */
       }
     }
     wzd_free(gid_list);
   }
 
   /* check for all users */
-  uid_list = (uid_t*)backend_get_user(-2);
+  uid_list = (uid_t*)backend_get_user(GET_USER_LIST);
   if (uid_list) {
     for (i=0; uid_list[i] != (uid_t)-1; i++) {
       loop_user = GetUserByID(uid_list[i]);
       if (loop_user) {
-        ret = user_ip_inlist(loop_user, inet_buf, context->ident);
+        ret = ip_list_check_ident(loop_user->ip_list, inet_buf, context->ident);
         if (ret) return 0; /* found ! */
       }
     }
@@ -482,6 +481,7 @@ static int server_add_ident_candidate(fd_t socket_accept_fd)
 {
   unsigned char remote_host[16];
   unsigned int remote_port;
+  int localport;
   char inet_buf[INET6_ADDRSTRLEN]; /* usually 46 */
   unsigned char userip[16];
   fd_t newsock, fd_ident;
@@ -493,11 +493,14 @@ static int server_add_ident_candidate(fd_t socket_accept_fd)
   newsock = socket_accept(socket_accept_fd, remote_host, &remote_port, &family);
   if (newsock == (fd_t)-1)
   {
-    out_log(LEVEL_HIGH,"Error while accepting\n");
-    serverMainThreadExit(-1); /** \todo do not exit server, just client */
+    out_log(LEVEL_HIGH,"FATAL Error while accepting\n");
+    serverMainThreadCleanup(-1); /** \todo do not exit server, just client */
+    return -1;
   }
   FD_REGISTER(newsock,"Client control socket");
 
+  localport = socket_get_local_port(newsock);
+
   memcpy(userip,remote_host,16);
 
 #if defined(IPV6_SUPPORT)
@@ -548,10 +551,18 @@ static int server_add_ident_candidate(fd_t socket_accept_fd)
   context->state = STATE_CONNECTING;
   context->controlfd = newsock;
   context->family = family;
+  context->localport = localport;
   time (&context->login_time);
 
   memcpy(context->hostip,userip,sizeof(context->hostip));
 
+  /* check if peer ip is a BNC. If yes, skip ident check */
+  if (ip_is_bnc(inet_buf, mainConfig)==1) {
+    out_log(LEVEL_FLOOD,"DEBUG connection from BNC %s\n",inet_buf);
+    server_login_accept(context);
+    return 0;
+  }
+
   /* check if ident lookups are disabled */
   if (CFG_GET_OPTION(mainConfig,CFG_OPT_DISABLE_IDENT)) {
     server_login_accept(context);
@@ -809,7 +820,7 @@ static void server_ip_check(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds)
             out_log(LEVEL_NORMAL,"could not add ident candidate for connection: %d (errno: %d: %s) :%s:%d\n",
                 ret, errno, strerror(errno), __FILE__, __LINE__);
           continue; /* possible cause of error: global ip rejected */
-/*          serverMainThreadExit(-1);*/
+/*          serverMainThreadCleanup(-1);*/
           /* we abort, so we never returns */
         }
         mainConfig->stats.num_connections++;
@@ -875,7 +886,7 @@ static void server_login_accept(wzd_context_t * context)
     }
     context->connection_flags |= CONNECTION_TLS;
   }
-  context->ssl.data_mode = TLS_CLEAR;
+  context->tls_data_mode = TLS_CLEAR;
 #endif
 
   if (CFG_GET_OPTION(mainConfig,CFG_OPT_CHECKIP_LOGIN)) {
@@ -963,11 +974,12 @@ fprintf(stderr,"Received signal %d\n",signum);
 #endif
 #endif
   /* commit backend changes */
-  ret = backend_commit_changes(mainConfig->backend.filename);
+  ret = backend_commit_changes(mainConfig->backends->filename);
   if (ret) {
     out_log(LEVEL_CRITICAL,"Could not commit changes to backend !\n");
   }
-  serverMainThreadExit(0);
+  serverMainThreadCleanup(0);
+  exit (0);
 }
 
 
@@ -982,7 +994,8 @@ uid_t get_server_uid(void)
 
 void server_crashed(int signum)
 {
-  printf("Server has crashed of signal %d\n",signum);
+  out_log(LEVEL_CRITICAL,"Server has crashed of signal %d\n",signum);
+  dump_backtrace();
 #ifdef DEBUG
 #ifndef WZD_DBG_NOABORT
   printf("I'll try to dump current memory to a core file (in the current dir)\n");
@@ -1165,11 +1178,11 @@ int server_switch_to_config(wzd_config_t *config)
       /* do we change gid ? */
       if (getlib_server_gid() != getegid())
       {
-        out_log(LEVEL_INFO,"Giving up root rights for group %ld (current gid %ld)\n",getlib_server_gid(),getgid());
+        out_log(LEVEL_INFO,"Giving up root rights for group %ld (current gid %ld)\n",(long)getlib_server_gid(),(long)getgid());
         setgid(getlib_server_gid());
       }
 
-      out_log(LEVEL_INFO,"Giving up root rights for user %ld (current uid %ld)\n",getlib_server_uid(),getuid());
+      out_log(LEVEL_INFO,"Giving up root rights for user %ld (current uid %ld)\n",(long)getlib_server_uid(),(long)getuid());
       setuid(getlib_server_uid());
     }
   }
@@ -1200,13 +1213,13 @@ int server_switch_to_config(wzd_config_t *config)
 
 
   /* if no backend available, we must bail out - otherwise there would be no login/pass ! */
-  if (!mainConfig->backend.filename) {
+  if (mainConfig->backends == NULL) {
     out_log(LEVEL_CRITICAL,"I have no backend ! I must die, otherwise you will have no login/pass !!\n");
     return -1;
   }
-  ret = backend_init(config->backend.filename,0 /* max users */,0 /* max groups */);
+  ret = backend_init(config->backends);
   /* if no backend available, we must bail out - otherwise there would be no login/pass ! */
-  if (ret || config->backend.handle == NULL) {
+  if (ret) {
     out_log(LEVEL_CRITICAL,"I have no backend ! I must die, otherwise you will have no login/pass !!\n");
     return -1;
   }
@@ -1296,8 +1309,8 @@ int server_switch_to_config(wzd_config_t *config)
       snprintf(buf,64,"%ld\n",(unsigned long)getpid());
       if (fd==-1) {
         out_log(LEVEL_CRITICAL,"Unable to open pid file %s: %s\n",config->pid_file,strerror(errno));
-        free_config(config);
-        exit(1);
+        cfg_free(config);
+        return -1;
       }
       ret = write(fd,buf,strlen(buf));
       close(fd);
@@ -1324,13 +1337,17 @@ int server_switch_to_config(wzd_config_t *config)
  * Initialize config and modules, and run the main loop: check for incoming
  * connections / ident connections, run cron jobs
  */
-void serverMainThreadProc(void *arg)
+int serverMainThreadProc(void *arg)
 {
   int ret;
   unsigned long max_wait_time;
   fd_set r_fds, w_fds, e_fds;
   fd_t maxfd;
   struct timeval tv;
+#if defined(WIN32) 
+  WSADATA wsaData;
+  int nCode;
+#endif
 
 #ifndef WIN32
   /* catch broken pipe ! */
@@ -1341,12 +1358,14 @@ void serverMainThreadProc(void *arg)
 #endif
 #endif /* _MSC_VER */
 
+#ifndef WIN32
   signal(SIGINT,interrupt);
   signal(SIGTERM,interrupt);
-#ifndef WIN32
 /*  signal(SIGKILL,interrupt);*/ /* SIGKILL signal is uncatchable */
 
   signal(SIGHUP,server_restart);
+#else
+  SetConsoleCtrlHandler( (PHANDLER_ROUTINE) interrupt,  TRUE ) ;
 #endif
 
 #ifdef SIGSYS
@@ -1361,7 +1380,7 @@ void serverMainThreadProc(void *arg)
     sigset_t oldmask, newmask;
     sigfillset(&newmask);
     ret = pthread_sigmask(SIG_BLOCK,&newmask,&oldmask);
-    WZD_ASSERT_VOID( ret == 0 );
+    WZD_ASSERT( ret == 0 );
   }
 #endif
 
@@ -1391,6 +1410,14 @@ void serverMainThreadProc(void *arg)
 #endif
 #endif /* _WIN32 */
 
+#ifdef WIN32
+  /* Start Winsock up */
+  if ((nCode = WSAStartup(MAKEWORD(2, 0), &wsaData)) != 0) {
+    out_log(LEVEL_CRITICAL,"Error initializing winsock2\n");
+    return -1;
+  }
+#endif
+
   /******** init shm *******/
   /* do this _before_ loading config, config can use it ! */
   vars_shm_init();
@@ -1416,11 +1443,10 @@ void serverMainThreadProc(void *arg)
   if (server_switch_to_config(mainConfig))
   {
     out_log(LEVEL_CRITICAL,"ERROR: couldn't switch to config, aborting !\n");
-    serverMainThreadExit(-1);
+    serverMainThreadCleanup(-1);
+    return -1;
   }
 
-  usercache_init();
-
 
 
   /* clear ident list */
@@ -1428,10 +1454,8 @@ void serverMainThreadProc(void *arg)
 
 
   /********** set up crontab ********/
-#if 0
   cronjob_add(&mainConfig->crontab,check_server_dynamic_ip,"fn:check_server_dynamic_ip",HARD_DYNAMIC_IP_INTVL,
       "*","*","*","*");
-#endif
   cronjob_add(&mainConfig->crontab,commit_backend,"fn:commit_backend",HARD_COMMIT_BACKEND_INTVL,
       "*","*","*","*");
 #ifdef HAVE_GNUTLS
@@ -1441,6 +1465,8 @@ void serverMainThreadProc(void *arg)
     out_log(LEVEL_HIGH,"TLS: error adding cron job _dh_params_regenerate\n");
 #endif
 
+  crontab_start(&mainConfig->crontab);
+
 
   /********** init modules **********/
   {
@@ -1465,7 +1491,7 @@ void serverMainThreadProc(void *arg)
     sigset_t oldmask, newmask;
     sigfillset(&newmask);
     ret = pthread_sigmask(SIG_UNBLOCK,&newmask,&oldmask);
-    WZD_ASSERT_VOID( ret == 0 );
+    WZD_ASSERT( ret == 0 );
   }
 #endif
 
@@ -1484,7 +1510,7 @@ void serverMainThreadProc(void *arg)
   /** \todo load stats ! */
   reset_stats(&mainConfig->stats);
 
-  out_log(LEVEL_HIGH,"%s started (build %s)\n",WZD_VERSION_STR,WZD_BUILD_NUM);
+  out_log(LEVEL_HIGH,"%s started (build %s)\n",WZD_VERSION_LONG,WZD_BUILD_NUM);
 
   /* now waiting for a connection */
   out_log(LEVEL_FLOOD,"Waiting for connections (main)\n");
@@ -1514,12 +1540,13 @@ void serverMainThreadProc(void *arg)
       if (errno == EINTR) continue; /* retry */
       if (errno == EBADF) {
         out_log(LEVEL_CRITICAL,"FATAL Bad file descriptor\n");
-        serverMainThreadExit(-1);
+        serverMainThreadCleanup(-1);
+        return -1;
       }
       out_log(LEVEL_CRITICAL,"select failed (%s) :%s:%d\n",
         strerror(errno), __FILE__, __LINE__);
-      serverMainThreadExit(-1);
-      /* we abort, so we never returns */
+      serverMainThreadCleanup(-1);
+      return -1;
 #if 0
     case 0: /* timeout */
       /* check for timeout logins */
@@ -1535,44 +1562,20 @@ void serverMainThreadProc(void *arg)
       server_ip_check(&r_fds,&w_fds,&e_fds);
     }
 
-    /* check cron jobs */
-    cronjob_run(&mainConfig->crontab);
-
   } /* while (!serverstop) */
 
 
   /* commit backend changes */
-  ret = backend_commit_changes(mainConfig->backend.filename);
+  ret = backend_commit_changes(mainConfig->backends->filename);
   if (ret) {
     out_log(LEVEL_CRITICAL,"Could not commit changes to backend !\n");
   } else
     out_log(LEVEL_INFO,"Backend commited\n");
-  serverMainThreadExit(0);
+  serverMainThreadCleanup(0);
+  return 0;
 }
 
-/** \deprecated ! use \ref cfg_free */
-static void free_config(wzd_config_t * config)
-{
-/*  limiter_free(mainConfig->limiter_ul);
-  limiter_free(mainConfig->limiter_dl);*/
-
-  if (mainConfig->xferlog_fd >= 0)
-    xferlog_close(mainConfig->xferlog_fd);
-
-  if (CFG_GET_OPTION(mainConfig,CFG_OPT_USE_SYSLOG)) {
-#ifndef WIN32
-    closelog();
-#endif
-  }
-  wzd_free(mainConfig->logfilename);
-  wzd_free(mainConfig->config_filename);
-  wzd_free(mainConfig->pid_file);
-  wzd_free(mainConfig->dir_message);
-
-  wzd_free(mainConfig);
-}
-
-void serverMainThreadExit(int retcode)
+void serverMainThreadCleanup(int retcode)
 {
   static int finished = 0;
 
@@ -1583,10 +1586,15 @@ void serverMainThreadExit(int retcode)
   out_log(LEVEL_HIGH,"Server exiting, retcode %d\n",retcode);
 
   /* ignore standard signals from now, we are exiting */
-#ifndef _MSC_VER
+#ifndef WIN32
   signal(SIGINT,SIG_IGN);
+#else
+  /* Disable console handler */
+  SetConsoleCtrlHandler( (PHANDLER_ROUTINE) interrupt,  FALSE ) ;
 #endif
 
+  crontab_stop();
+
   list_destroy(&server_ip_list);
 
   if (mainConfig->controlfd != (fd_t)-1) {
@@ -1602,8 +1610,13 @@ void serverMainThreadExit(int retcode)
     wzd_context_t * loop_context;
     for (elmnt=list_head(context_list); elmnt!=NULL; elmnt=list_next(elmnt))
     {
-      if ((loop_context = list_data(elmnt)))
+      if ((loop_context = list_data(elmnt))) {
+        wzd_thread_cancel(loop_context->pid_child);
+#ifdef WIN32
+        /** \todo remove this when wzd_thread_cancel is implemented on windows */
         loop_context->exitclient = 1;
+#endif
+      }
     }
   }
 #endif
@@ -1649,16 +1662,18 @@ void serverMainThreadExit(int retcode)
   wzd_cache_purge();
   vars_shm_free();
   utf8_end(mainConfig);
-  server_clear_param(&mainConfig->param_list);
   hook_free(&mainConfig->hook);
   hook_free_protocols();
   module_free(&mainConfig->module);
-  backend_close(mainConfig->backend.filename);
+  /** \todo XXX close ALL backends */
+  if (mainConfig->backends != NULL) backend_close(mainConfig->backends->filename);
   cronjob_free(&mainConfig->crontab);
   section_free(&mainConfig->section_list);
   vfs_free(&mainConfig->vfs);
   free_messages();
-  usercache_fini();
+
+  user_free_registry();
+  group_free_registry();
 
   if (limiter_mutex) wzd_mutex_destroy(limiter_mutex);
   if (server_mutex) wzd_mutex_destroy(server_mutex);
@@ -1672,14 +1687,14 @@ void serverMainThreadExit(int retcode)
   log_fini();
 
   /* free(mainConfig); */
-  unlink(mainConfig->pid_file);
+  if (mainConfig->pid_file) unlink(mainConfig->pid_file);
   cfg_free(mainConfig);
   mainConfig = NULL;
   server_mutex_set_fini();
 
   wzd_debug_fini();
 
-#if defined(_MSC_VER)
+#if defined(WIN32)
   WSACleanup();
 #endif
 
@@ -1697,8 +1712,6 @@ void serverMainThreadExit(int retcode)
 
   wzd_mutex_unlock(end_mutex);
   wzd_mutex_destroy(end_mutex);
-
-  exit (retcode);
 }
 
 /*! @} */
